!! Example of a Fortran program using libgrace_np (GraceGTK version)

!! See  P. Vincent, Applied Physics A, Vol 31 ,pp51-54, 1983.

!! In place of the true program, we use a formula obtained 
!! by a non linear fit to compute the frequencies.
!!
!! The program simultaneously draw the figure using libgrace_np.a
!! and write the project file testf90.agr




!!*******************************************************************************
MODULE testf90_mod 
  !!
  INTEGER ,PARAMETER :: LU20 = 20  ! LU to write testf90.dat
  INTEGER ,PARAMETER :: LU21 = 21  ! LU to write testf90.agr
  REAL(8) ,PARAMETER :: H1 = 10.0  ! y_centre
  INTEGER     jer                  ! error indicator

END MODULE testf90_mod
!!*******************************************************************************
PROGRAM testf90
  USE testf90_mod
  IMPLICIT NONE 
  INTEGER ,PARAMETER :: N = 10               ! nb of points in the curve
  REAL(8) ,PARAMETER :: Z = 20.0             ! distance between the plates
  REAL(8) ,PARAMETER :: X0 = 5.0 ,X1 = 10.0  ! the interval to draw
  REAL(8) ,PARAMETER :: A0 = 2.17963         ! coeff. fof non lin. fit
  REAL(8) ,PARAMETER :: A1 = 3.39699
  REAL(8) ,PARAMETER :: A2 = 2.97706
  REAL(8) ,PARAMETER :: A3 = 5.00112

  EXTERNAL MyError
  INTEGER ,EXTERNAL :: GraceOpenf ,graceflushf ,GraceIsOpenf

  !!  Variables locales
  !!  ----------------- 
  CHARACTER(128) cmd
  REAL(8)     gauche ,droite
  INTEGER     i
  REAL(8)     dx ,x ,y


  !!  Instructions executables
  !!  ------------------------ 
  CALL GraceRegistErerrorFunctionf (MyError)
  jer = 0
  OPEN  (LU20 ,FILE="testf90.dat")
  OPEN  (LU21 ,FILE="testf90.agr")
  WRITE (LU21 , '("# -*- sh -*-")')   ! emacs will use sh-mode
  WRITE (LU21 , '("# Example of a Fortran program using libgrace_np") ')
  jer = GraceOpenf (1024)
  IF (jer .NE. 0) THEN
    PRINT *,':dr_pole_gracegtk  ERREUR dans GraceOpenf'
    STOP 
  ENDIF
  CALL GraceCmd ('version 1000500 ')
  CALL GraceCmd ('PAGE SIZE "A4" portrait')
  CALL GraceCmd ('With G0; G0 on')
  CALL GraceCmd ('VIEW 0.1 ,0.3 ,0.8 ,1.2')

  CALL GraceCmd ('title "Resonant frequency of a dielectric resonator"')
  CALL GraceCmd ('title size 1.2')
  CALL GraceCmd ('subtitle "\R{blue}Example produced by the program \f{Courier}testf90.f90"')
  CALL GraceCmd ('subtitle size 0.8')

        ! Setting axes for set S0
  CALL GraceCmd ('target G0.S0  ')
  CALL GraceCmd ('  world 5, 2.5, 10, 5.5')
  CALL GraceCmd ('  xaxis label "cylinder radius R (mm)"')
  CALL GraceCmd ('  yaxis label "resonant frequency (GHz)"')
  CALL GraceCmd ('  xaxis tick major 1;   xaxis  tick minor ticks 1')
  CALL GraceCmd ('  yaxis tick major 0.5; yaxis  tick minor ticks 4')
  CALL GraceCmd ('  s0 symbol 8')

        ! Drawing of the device into G1
  CALL GraceCmd ('With G1')
  CALL GraceCmd ('G1 on')
  CALL GraceCmd (' view 0.4, 0.75, 0.80, 1.15')
  CALL GraceCmd (' world 0, -2, 20, 22')
  CALL GraceCmd ('G1 type Fixed')

  CALL GraceCmd ('xaxis  tick major 5; xaxis  tick minor ticks 1')
  CALL GraceCmd ('yaxis  tick major 5; yaxis  tick minor ticks 1')
  CALL GraceCmd ('xaxis  label char size 0.6; xaxis tick out; xaxis tick major size 0.5; xaxis tick minor size 0.25')
  CALL GraceCmd ('yaxis  label char size 0.6; yaxis tick out; yaxis tick major size 0.5; yaxis tick minor size 0.25')
  CALL GraceCmd ('xaxis  label "mm"')
  CALL GraceCmd ('yaxis  label "mm"')

  CALL GraceCmd ('')
  CALL GraceCmd ('# On trace le resonateur')
  gauche = h1 - 1.5*X0
  droite = h1 + 1.5*X0

10  FORMAT (A ,4(G15.5 : ',') )

  !! On trace le cercle
  CALL GraceCmd (' WITH Arc')
  CALL GraceCmd (' Arc G1;  Arc type 4; Arc loctype world;  Arc color 4')
  WRITE (cmd ,10) '   Arc : ' ,h1-X0 ,h1-X0 ,h1+X0 ,h1+X0
  CALL GraceCmd (cmd)
  CALL GraceCmd ('   Arc linewidth 1.5; Arc fill color 4; Arc fill pattern 15')
  CALL GraceCmd ('    Arc def "Resonator"')

  !! le rayon
  CALL GraceCmd (' WITH Line; Line G1; Line loctype world; Line arrow 2')
  WRITE (cmd ,10) '   Line : ' ,h1 ,h1 ,h1+0.707*X0 ,h1-0.707*X0
  CALL GraceCmd (cmd)
  CALL GraceCmd ('    Line arrow 2')
  CALL GraceCmd ('    Line def "R"')
  CALL GraceCmd ('    Line label type 2')
  CALL GraceCmd ('    Line  just 66')

  !! La cote du centre du cercle
  CALL GraceCmd (' WITH Line; Line G1; Line loctype world; Line arrow 3')
  WRITE (cmd ,10) '   Line : ' ,gauche + 0.2*X0  ,0.0 ,gauche + 0.2*X0 ,h1
  CALL GraceCmd (cmd)
  CALL GraceCmd ('    Line arrow 3')
  CALL GraceCmd ('    Line def "Ordinate of the centre"')
  CALL GraceCmd ('    Line label type 1')
  CALL GraceCmd ('    Line just 6')
  CALL GraceCmd ('    Line char size 0.6')
  CALL GraceCmd ('    Line label format "%.3g"')
  CALL GraceCmd ('    Line label offset 0.012 , -0.012')

  !! La cote entre les plaques
  CALL GraceCmd (' WITH Line; Line G1; Line loctype world; Line arrow 3')
  WRITE (cmd ,10) '   Line : ' ,droite - 0.2*X0  ,0.0 ,droite - 0.2*X0 ,z
  CALL GraceCmd (cmd)
  CALL GraceCmd ('    Line arrow 3')
  CALL GraceCmd ('    Line def "Distance between the two plates"')
  CALL GraceCmd ('    Line label type 1')
  CALL GraceCmd ('    Line just 6')
  CALL GraceCmd ('    Line char size 0.6')
  CALL GraceCmd ('    Line label format "%.3g"')
  CALL GraceCmd ('    Line label offset -0.012 , -0.012')
  
  !! the plate below
  CALL GraceCmd ('')
  CALL GraceCmd ('# On trace l''armature du bas')
  CALL GraceCmd (' WITH Line; Line G1; Line loctype world; Line linewidth 2.0')
  WRITE (cmd ,10) '   Line : ' ,gauche  ,0.0 ,droite ,0.0
  CALL GraceCmd (cmd)
  CALL GraceCmd ('    Line def "Plate below"')
  CALL GraceCmd (' WITH Box; Box G1; Box loctype world; Box fill pattern 14; Box linestyle 0')
  WRITE (cmd ,10) '   Line : ' ,gauche  ,0.0 ,droite ,-0.2*X0
  CALL GraceCmd (cmd)
  CALL GraceCmd ('    Box def "Plate below"')

  !! the plate above
  CALL GraceCmd ('')
  CALL GraceCmd ('# On trace l''armature du haut')
  CALL GraceCmd (' WITH Line; Line G1; Line loctype world; Line linewidth 2.0')
  WRITE (cmd ,10) '   Line : ' ,gauche  ,z ,droite ,z
  CALL GraceCmd (cmd)
  CALL GraceCmd ('    Line def "Plate above"')
  CALL GraceCmd (' WITH Box; Box G1; Box loctype world; Box fill pattern 14; Box linestyle 0')
  WRITE (cmd ,10) '   Line : ' ,gauche  ,z ,droite ,z+0.2*X0
  CALL GraceCmd (cmd)
  CALL GraceCmd ('    Box def "Plate above"')
  

  
20 FORMAT (2G15.5)
21 FORMAT ('g0.s0 point ' ,G15.5 ,',' ,G15.5)
  WRITE (LU21 ,'("@ type xy")')
  WRITE (LU20 ,"('! R  ' ,15X ,'Freq')") 
  dx = (X1 - X0) / N
  DO i = 0 ,10
    x = X0 + i*dx
    y = A0 + A3*EXP(-(x-A1)/A2)
    WRITE (LU20 ,20) x ,y            ! Write testf90.dat
    WRITE (cmd  ,21) x ,y
    CALL GraceCommandf (cmd)         ! send (x,y) to GraceGTK
    CALL GraceCommandf ("redraw")
    !! Wait a second, just to simulate some time needed
    !! for calculations. Your real application shouldn't wait
    CALL SLEEP (1)
  ENDDO
  CLOSE (LU20)

30 FORMAT ('@ READ "testf90.dat"')
  WRITE (LU21 ,30)

  !! Close automatically GraceGTK
  CALL GraceClosef ()

  CLOSE (LU21)
  
END PROGRAM testf90

!!*******************************************************************************
!! Allows to write simustaneously into a file and into the pipe
SUBROUTINE GraceCmd (cmd)
  USE testf90_mod
  IMPLICIT NONE 
  CHARACTER(*) ,INTENT(in) :: cmd
  WRITE (LU21 ,'(A ,A)') '@ ' ,TRIM(cmd)
  CALL GraceCommandf (cmd)
END SUBROUTINE GraceCmd


!!*******************************************************************************

SUBROUTINE MyError (str)
  !!
  IMPLICIT NONE
  CHARACTER*(*) str
  !!
  WRITE (*, '(''GraceGTK library message : "'', a, ''"'')') str
  !!
  RETURN
END SUBROUTINE MyError

!!*******************************************************************************



