/* for the meaning of these fields, see struct.m */
/* longs are used here so that the codes can be called from S */


#ifndef __LOESS_H_
#define __LOESS_H_

#ifndef Sint
#define Sint int
#endif

typedef enum {
  NL_LEVENBERG
  ,NL_LOESS
  ,NL_BY_HAND
} NL_method;

#ifndef NL_GAUSSIAN
typedef enum {
  NL_SYMMETRIC
  ,NL_GAUSSIAN
} NL_family;
#endif /* NL_GAUSSIAN */

typedef struct loess_struct loess_typ;
typedef struct pred_struct  pred_typ;
typedef struct anova_struct  anova_typ;
typedef struct ci_struct  ci_typ;

struct loess_struct {
	struct {
		Sint    n;
	        Sint    p;
	        double  *y;
	        double  *x;
		double	*weights;
	} in;
	struct {
	        double  span;
	        Sint    degree;
	        Sint    normalize;
	        Sint    parametric[8];
	        Sint    drop_square[8];
	        Sint    family;
	  // char    *family;
	} model;
	struct {
	        char    *surface;
	        char    *statistics;
	        double  cell;
	        char    *trace_hat;
	        Sint    iterations;
	} control;
	struct {
		Sint	*parameter;
		Sint	*a;
		double	*xi;
		double	*vert;
		double	*vval;
	} kd_tree;
	struct {
		double	*fitted_values;
	        double  *fitted_residuals;
		double  enp;
		double	s;
		double  one_delta;
		double	two_delta;
		double	*pseudovalues;
		double	trace_hat;
		double	*diagonal;
		double	*robust;
		double  *divisor;
	} out;
};

struct pred_struct {
	double	*fit;
	double	*se_fit;
	double  residual_scale;
	double  df;
};

struct anova_struct {
	double	dfn;
	double	dfd;
	double  F_value;
	double  Pr_F;
};

struct ci_struct {
	double	*fit;
	double	*upper;
	double  *lower;
};


void loess_setup    (double *x ,double *y ,Sint n ,Sint p ,loess_typ *lo);
int  loess    	    (loess_typ *lo);
void loess_dfit     (double *y, double *x, double *x_evaluate, double *weights, double *span, Sint *degree, Sint *nonparametric, 
		     Sint *drop_square, Sint *sum_drop_sqr, Sint *d, Sint *n, Sint *m, double *fit);
void loess_dfitse   (double *y, double *x, double *x_evaluate, double *weights, double *robust, Sint family, double *span, Sint *degree, 
		     Sint *nonparametric, Sint *drop_square, Sint *sum_drop_sqr, Sint *d, Sint *n, Sint *m, double *fit, double *L);
void loess_ifit     (Sint *parameter ,Sint *a ,double *xi ,double *vert ,double *vval ,Sint *m ,double *x_evaluate ,double *fit);
void loess_ise 	    (double *y, double *x, double *x_evaluate, double *weights, double *span, Sint *degree, Sint *nonparametric, 
		     Sint *drop_square, Sint *sum_drop_sqr, double *cell, Sint *d, Sint *n, Sint *m, double *fit,double * L);

void loess_summary  (loess_typ *lo);
void anova 	    (loess_typ *one ,loess_typ *two ,anova_typ *out);
void loess_free_mem (loess_typ *lo);

void predict	    (double *eval, Sint m, loess_typ *lo, pred_typ *pre, Sint se);
void pointwise 	    (pred_typ *pre ,Sint m ,double coverage, ci_typ *ci);
void pred_free_mem  (pred_typ *pre);
void pw_free_mem    (ci_typ *ci);

int IERRGET_FC (void);

#endif /* __LOESS_H_ */
