
#include <string.h>

#include "../config.h"

#include "S.h"

#define	min(x,y)  ((x) < (y) ? (x) : (y))
#define	max(x,y)  ((x) > (y) ? (x) : (y))

/* Fortran wrappers */
#define LOWESA_FC FC_FUNC(lowesa,LOWESA)
#define LOWESB_FC FC_FUNC(lowesb,LOWESB)
#define LOWESC_FC FC_FUNC(lowesc,LOWESC)
#define LOWESD_FC FC_FUNC(lowesd,LOWESD)
#define LOWESE_FC FC_FUNC(lowese,LOWESE)
#define LOWESF_FC FC_FUNC(lowesf,LOWESF)
#define LOWESL_FC FC_FUNC(lowesl,LOWESL)
#define EHG169_FC FC_FUNC(ehg169,EHG169)
#define EHG182_FC FC_FUNC(ehg182,EHG182)
#define EHG196_FC FC_FUNC(ehg196,EHG196)

#define IERRGET_FC FC_FUNC(ierrget,IERRGET)


static Sint	*iv, liv, lv, tau;
static double	*v;
void LOWESA_FC (double *trl ,Sint *n   ,Sint *d    ,Sint *tau     ,Sint *nsing ,double *delta1 ,double *delta2);
/*                           1         2          3              4           5               6           7         8         9         10 */
void LOWESB_FC (double *x 	 ,double *y ,double *ww ,double *diagl ,Sint *infl     ,Sint *iv   ,double *v);
void LOWESC_FC (Sint *n     ,double *l ,double *ll ,double *trl   ,double *delta1 ,double *delta2);
void LOWESD_FC (Sint *vers  ,Sint *iv  ,Sint *liv  ,Sint *lv      ,double *v      ,Sint *d   ,Sint *n   ,double *f ,Sint *ideg ,Sint *nvmax 
		     ,Sint *setLf);
void LOWESE_FC (Sint *iv  	 ,double *wv ,Sint *m   ,double *z     ,double *s);
void LOWESF_FC (double *x 	 ,double *y ,double *ww ,Sint *iv      ,double *wv     ,Sint *m   ,double *z  ,double *l 
		     ,Sint *ihat ,double *s);
void LOWESL_FC (Sint *iv  	 ,double *wv ,Sint *m   ,double *z     ,double *l);


void EHG169_FC (Sint *d    ,Sint *vc   ,Sint *nc ,Sint *ncmax     ,Sint *nv     ,Sint *nvmax ,double *v ,Sint *a   ,double *xi ,Sint *c 
		     ,Sint *hi ,Sint *lo);
void EHG196_FC (Sint *tau  ,Sint *d ,double *f ,double *trl);

void Recover (char *a, int *b);
void Warning (char *a, int *b);

static void loess_workspace (Sint *d, Sint *n, double *span, Sint *degree, Sint *nonparametric, Sint *drop_square, 
			     Sint *sum_drop_sqr, Sint *setLf);
static void loess_free ();
static void loess_grow (Sint *parameter ,Sint *a ,double *xi ,double *vert ,double *vval);


void loess_raw (double *y ,double *x ,double *weights ,double *robust 
		,Sint *d ,Sint *n ,double *span ,Sint *degree 
		,Sint *nonparametric ,Sint *drop_square ,Sint *sum_drop_sqr ,double *cell 
		,char **surf_stat ,double *surface ,Sint *parameter ,Sint *a 
		,double *xi ,double *vert ,double *vval ,double *diagonal
		,double *trL ,double *one_delta ,double *two_delta ,Sint *setLf)

{
	Sint	zero = 0, one = 1, two = 2, nsing, i, k;
	double	*hat_matrix, *LL ,dzero = 0.0;

	*trL = 0;
	loess_workspace(d, n, span, degree, nonparametric, drop_square, 
		sum_drop_sqr, setLf);
        v[1] = *cell;
	if(!strcmp(*surf_stat, "interpolate/none")) {
		LOWESB_FC(x, y, robust, &dzero, &zero, iv, v);
		LOWESE_FC(iv, v, n, x, surface);
		loess_prune(parameter, a, xi, vert, vval);
	}			
	else if (!strcmp(*surf_stat, "direct/none")) {
		LOWESF_FC(x, y, robust, iv, v, n, x,
			&dzero, &zero, surface);
	}
	else if (!strcmp(*surf_stat, "interpolate/1.approx")) {
		LOWESB_FC(x, y, weights, diagonal, &one, iv, v);
		LOWESE_FC(iv, v, n, x, surface);
		nsing = iv[29];
		for(i = 0; i < (*n); i++) *trL = *trL + diagonal[i];
		LOWESA_FC(trL, n, d, &tau, &nsing, one_delta, two_delta);
		loess_prune(parameter, a, xi, vert, vval);
	}
        else if (!strcmp(*surf_stat, "interpolate/2.approx")) {
		LOWESB_FC(x, y, robust, &dzero, &zero, iv, v);
		LOWESE_FC(iv, v, n, x, surface);
		nsing = iv[29];
		EHG196_FC(&tau, d, span, trL);
		LOWESA_FC(trL, n, d, &tau, &nsing, one_delta, two_delta);
		loess_prune(parameter, a, xi, vert, vval);
	}
	else if (!strcmp(*surf_stat, "direct/approximate")) {
		LOWESF_FC(x, y, weights, iv, v, n, x,
			diagonal, &one, surface);
		nsing = iv[29];
		for(i = 0; i < (*n); i++) *trL = *trL + diagonal[i];
		LOWESA_FC(trL, n, d, &tau, &nsing, one_delta, two_delta);
	}
	else if (!strcmp(*surf_stat, "interpolate/exact")) {
		hat_matrix = Calloc((*n)*(*n), double);
		LL = Calloc((*n)*(*n), double);
		LOWESB_FC(x, y, weights, diagonal, &one, iv, v);
		LOWESL_FC(iv, v, n, x, hat_matrix);
		LOWESC_FC(n, hat_matrix, LL, trL, one_delta, two_delta);
		LOWESE_FC(iv, v, n, x, surface);
		loess_prune(parameter, a, xi, vert, vval);
		Free(hat_matrix);
		Free(LL);
	}
	else if (!strcmp(*surf_stat, "direct/exact")) {
		hat_matrix = Calloc((*n)*(*n), double);
		LL = Calloc((*n)*(*n), double);
		LOWESF_FC(x, y, weights, iv, v, n, x,
			hat_matrix, &two, surface);
		LOWESC_FC(n, hat_matrix, LL, trL, one_delta, two_delta);
                k = (*n) + 1;
		for(i = 0; i < (*n); i++)
			diagonal[i] = hat_matrix[i * k];
		Free(hat_matrix);
		Free(LL);
	}
	loess_free();
}

void loess_dfit (double *y, double *x, double *x_evaluate, double *weights, double *span, Sint *degree, Sint *nonparametric, 
		 Sint *drop_square, Sint *sum_drop_sqr, Sint *d, Sint *n, Sint *m, double *fit)
{
  Sint	zero = 0;
	double dzero = 0.0;
        loess_workspace(d, n, span, degree, nonparametric, drop_square,
                sum_drop_sqr, &zero);
	LOWESF_FC(x, y, weights, iv, v, m, x_evaluate,
			&dzero, &zero, fit);
	loess_free();
}

/*                       1          2               3                  4                 5           6               7           8*/
void loess_dfitse (double *y, double *x, double *x_evaluate, double *weights, double *robust, Sint family, double *span, Sint *degree, 
		   Sint *nonparametric, Sint *drop_square, Sint *sum_drop_sqr, Sint *d, Sint *n, Sint *m, double *fit, double *L)
/*                          9                     10                 11           12       13       14          15          16    */
{
	Sint	zero = 0, two = 2;
	double dzero = 0.0;	
        loess_workspace(d, n, span, degree, nonparametric, drop_square,
                sum_drop_sqr, &zero);
	if(family == NL_GAUSSIAN)
		LOWESF_FC(x, y, weights, iv, v, m, 
				x_evaluate, L, &two, fit);
	else if(family == NL_SYMMETRIC)
	{
		LOWESF_FC(x, y, weights, iv, v, m, x_evaluate, L, &two, fit);
		LOWESF_FC(x, y, robust, iv, v, m, x_evaluate, &dzero, &zero, fit);
	}	
	loess_free();
}
void loess_ifit (Sint *parameter ,Sint *a ,double *xi ,double *vert ,double *vval ,Sint *m ,double *x_evaluate ,double *fit)
{
	loess_grow(parameter, a, xi, vert, vval);
	LOWESE_FC(iv, v, m, x_evaluate, fit);
	loess_free();
}

void loess_ise (double *y, double *x, double *x_evaluate, double *weights, double *span, Sint *degree, Sint *nonparametric, 
	Sint *drop_square, Sint *sum_drop_sqr, double *cell, Sint *d, Sint *n, Sint *m, double *fit,double * L)
{
	Sint	zero = 0, one = 1;
	double dzero = 0.0;
        loess_workspace(d, n, span, degree, nonparametric, drop_square,
                sum_drop_sqr, &one);
	v[1] = *cell;
	LOWESB_FC(x, y, weights, &dzero, &zero, iv, v);
	LOWESL_FC(iv, v, m, x_evaluate, L);
	loess_free();
}

void loess_workspace(d, n, span, degree, nonparametric, drop_square, 
	sum_drop_sqr, setLf)
Sint	*d, *n, *degree, *nonparametric, *drop_square, *sum_drop_sqr, 
	*setLf;
double	*span;
{
	Sint	D, N, tau0, nvmax, nf, version = 106, i;

	D = *d;
	N = *n;
	nvmax = max(200, N);
        nf = min(N, floor(N * (*span)));
        tau0 = ((*degree) > 1) ? ((D + 2) * (D + 1) * 0.5) : (D + 1);
        tau = tau0 - (*sum_drop_sqr);
        lv = 50 + (3 * D + 3) * nvmax + N + (tau0 + 2) * nf;
	liv = 50 + ((Sint)pow((double)2, (double)D) + 4) * nvmax + 2 * N;
	if(*setLf) {
		lv = lv + (D + 1) * nf * nvmax;
		liv = liv + nf * nvmax;	
	}
        iv = Calloc(liv, Sint);
        v = Calloc(lv, double);
	/*                  1      2    3    4   5  6  7   8      9       10     11 */
        LOWESD_FC(&version, iv, &liv, &lv, v, d, n, span, degree, &nvmax, setLf);
        iv[32] = *nonparametric;
        for(i = 0; i < D; i++)
                iv[i + 40] = drop_square[i];
}

void loess_prune (Sint *parameter ,Sint *a ,double *xi ,double *vert ,double *vval)
{
	Sint	d, vc, a1, v1, xi1, vv1, nc, nv, nvmax, i, k;
	
	d = iv[1];
	vc = iv[3] - 1;
	nc = iv[4];
	nv = iv[5];
	a1 = iv[6] - 1;
	v1 = iv[10] - 1;
	xi1 = iv[11] - 1;
	vv1 = iv[12] - 1;
	nvmax = iv[13];

	for(i = 0; i < 5; i++)
		parameter[i] = iv[i + 1];
	parameter[5] = iv[21] - 1;
	parameter[6] = iv[14] - 1;

	for(i = 0; i < d; i++){
		k = nvmax * i;
		vert[i] = v[v1 + k];
		vert[i + d] = v[v1 + vc + k];
	}
	for(i = 0; i < nc; i++) {
		xi[i] = v[xi1 + i];
		a[i] = iv[a1 + i];
	}
	k = (d + 1) * nv;
	for(i = 0; i < k; i++)
		vval[i] = v[vv1 + i];
}

static void loess_grow (Sint *parameter ,Sint *a ,double *xi ,double *vert ,double *vval)
{
	Sint	d, vc, nc, nv, a1, v1, xi1, vv1, i, k;

	d = parameter[0];
	vc = parameter[2];
	nc = parameter[3];
	nv = parameter[4];
	liv = parameter[5];
	lv = parameter[6];
	iv = Calloc(liv, Sint);
	v = Calloc(lv, double);

	iv[1] = d;
	iv[2] = parameter[1];
	iv[3] = vc;
	iv[5] = iv[13] = nv;
	iv[4] = iv[16] = nc;
	iv[6] = 50;
	iv[7] = iv[6] + nc;
	iv[8] = iv[7] + vc * nc;
	iv[9] = iv[8] + nc;
	iv[10] = 50;
	iv[12] = iv[10] + nv * d;
	iv[11] = iv[12] + (d + 1) * nv;
	iv[27] = 173;

	v1 = iv[10] - 1;
	xi1 = iv[11] - 1;
	a1 = iv[6] - 1;
	vv1 = iv[12] - 1;
	
        for(i = 0; i < d; i++) {
		k = nv * i;
		v[v1 + k] = vert[i];
		v[v1 + vc - 1 + k] = vert[i + d];
	}
        for(i = 0; i < nc; i++) {
                v[xi1 + i] = xi[i];
                iv[a1 + i] = a[i];
        }
	k = (d + 1) * nv;
	for(i = 0; i < k; i++)
		v[vv1 + i] = vval[i];

	EHG169_FC (&d, &vc, &nc, &nc, &nv, &nv, v+v1, iv+a1,
			v+xi1, iv+iv[7]-1, iv+iv[8]-1, iv+iv[9]-1);
}

static void loess_free()
{
        Free(v);
        Free(iv);
}


/* begin ehg's FORTRAN-callable C-codes */

static int ierrnum;

void EHG182_FC (int *i)
{
  char *mess, mess2[60];
  ierrnum = *i;
  switch(*i){
  case 0: break;
  case 100: mess="LOESS: wrong version number in lowesd.  Probably typo in caller."; break;
  case 101: mess="LOESS: d>dMAX in ehg131.  Need to recompile with increased dimensions."; break;
  case 102: mess="LOESS: liv too small.   (Discovered by lowesd)"; break;
  case 103: mess="LOESS: lv too small.    (Discovered by lowesd)"; break;
  case 104: mess="LOESS: span too small.  fewer data values than degrees of freedom."; break;
  case 105: mess="LOESS: k>d2MAX in ehg136.  Need to recompile with increased dimensions."; break;
  case 106: mess="LOESS: lwork too small"; break;
  case 107: mess="LOESS: invalid value for kernel"; break;
  case 108: mess="LOESS: invalid value for ideg"; break;
  case 109: mess="LOESS: lowstt only applies when kernel=1."; break;
  case 110: mess="LOESS: not enough extra workspace for robustness calculation"; break;
  case 120: mess="LOESS: zero-width neighborhood. make span bigger"; break;
  case 121: mess="LOESS: all data on boundary of neighborhood. make span bigger"; break;
  case 122: mess="LOESS: extrapolation not allowed with blending"; break;
  case 123: mess="LOESS: ihat=1 (diag L) in l2fit only makes sense if z=x (eval=data)."; break;
  case 171: mess="LOESS: lowesd must be called first."; break;
  case 172: mess="LOESS: lowesf must not come between lowesb and lowese, lowesr, or lowesl."; break;
  case 173: mess="LOESS: lowesb must come before lowese, lowesr, or lowesl."; break;
  case 174: mess="LOESS: lowesb need not be called twice."; break;
  case 175: mess="LOESS: need setLf=.true. for lowesl."; break;
  case 180: mess="LOESS: nv>nvmax in cpvert."; break;
  case 181: mess="LOESS: nt>20 in eval."; break;
  case 182: mess="LOESS: svddc failed in l2fit."; break;
  case 183: mess="LOESS: didnt find edge in vleaf."; break;
  case 184: mess="LOESS: zero-width cell found in vleaf."; break;
  case 185: mess="LOESS: trouble descending to leaf in vleaf."; break;
  case 186: mess="LOESS: insufficient workspace for lowesf."; break;
  case 187: mess="LOESS: insufficient stack space"; break;
  case 188: mess="LOESS: lv too small for computing explicit L"; break;
  case 191: mess="LOESS: computed trace L was negative; something is wrong!"; break;
  case 192: mess="LOESS: computed delta was negative; something is wrong!"; break;
  case 193: mess="LOESS: workspace in loread appears to be corrupted"; break;
  case 194: mess="LOESS: trouble in l2fit/l2tr"; break;
  case 195: mess="LOESS: only constant, linear, or quadratic local models allowed"; break;
  case 196: mess="LOESS: degree must be at least 1 for vertex influence matrix"; break;
  case 999: mess="LOESS: not yet implemented"; break;
  default: sprintf(mess=mess2,"LOESS: Assert failed; error code %d\n",*i); break;
  }
  if (ierrnum != 0) errmsg (mess);
}


/* ierrget */
int IERRGET_FC (void)
{
  return ierrnum;
}
