/**  File cairo_drv.c
 *
 * Output drivers using cairo library
 *  formats: screen, PDF, PostScript, PNG and SVG
 */
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <cairo.h>
#include <cairo-ps.h>
#include <cairo-pdf.h>
#include <cairo-svg.h>

#include "device.h"
#include "draw.h"
#include "files.h"
#include "graphs.h"
#include "patterns.h"
#include "plotone.h"
#include "gg_protos.h"
#include "utils.h"


/* CAIRO functions are prefixed with cairo_,
 * functions provided by this driver by cr_
 */

#include "cairo_drv.h"

extern GdkColor gg_colors[MAXCOLORS];
extern GdkColor  bleuciel ,wheat;

void gimp_get_screen_resolution (GdkScreen *screen,
				 gdouble   *xres,
				 gdouble   *yres);
extern char print_file[];

typedef struct {
  cairo_surface_t *surface;
  cairo_t         *cr;        /* the CAIRO context */
  cairo_matrix_t  matrix;     /* transform viewport coord. in device coord. */
  int    color;               /* store current color   to detect change */
  int    pattern;             /* store current pattern to detect change */
  Page_geometry pg;           /* width, height in pixels, dpi of the device */
  double pixel_size;          /* in pixel**-1 units */
  double dscale;              /* current page min(width,height) in pixels  */
  double scale;          /* factor to convert VPoint in pixels */
  double height;        /* factor to convert VPoint in pixels */
  double dheight ,dwidth;     /* in points (1/72.0 inch) for PS, PDF and SVG */
  int bg;
} CairoDriver;
  
static CairoDriver *pcr = NULL; /* pointer on current context */
static CairoDriver hcr; /* harcopy context */
static CairoDriver tcr; /* terminal (console) context */

/* provisoire: cr_initcmap doit pouvoir etre fait dans le driver */
void cr_initcmap (void);
static void cr_sync_canvas_size (gint *w ,gint *h ,CanvasType canvas_type);

static void tcr_leavegraphics (void);
static void cr_setpen (const Pen *pen);


/******************** Utilities common to all drivers ********************/

/* To be done */
void cr_updatecmap (void) {}

/**
 * cr_init initalise variables common to all the Cairo drivers 
 */
static void cr_init (CairoDriver *crd)
{
  crd->surface = NULL;
  crd->cr      = NULL;
  /* undefine graphics state parameters */
  crd->color = -1;
  crd->pattern = -1;
  /* aliasing of device-independent routines */
  devdrawarc       = cr_drawarc;
  devdrawpixel     = cr_drawpixel;
  devdrawpolyline  = cr_drawpolyline;
  devfillarc       = cr_fillarc;
  devfillpolygon   = cr_fillpolygon;
  devputpixbuf     = cr_putpixbuf;
  devputpixmap     = cr_putpixmap;
  devputtext       = cr_puttext;
  devsetfrgbcolor  = cr_setfrgbcolor;
  devupdatecmap    = cr_updatecmap;
  /* routine devleavegraphics depends upon the driver */
  /* page geometry */
  crd->pg      	    = get_page_geometry();                 /* return page dimensions in pixels */
  crd->dheight 	    = crd->pg.height * 72.0 / crd->pg.dpi; /* in points  (1 point == 1/72.0 inch) */
  crd->dwidth  	    = crd->pg.width  * 72.0 / crd->pg.dpi; /* in points  (1 point == 1/72.0 inch) */
  crd->dscale       = (double) MIN2 (crd->pg.height ,crd->pg.width);
  crd->pixel_size   = 1.0 / crd->dscale;
  crd->scale   = crd->dscale * 72.0 / crd->pg.dpi;
  crd->height = ((double)crd->pg.height) * 72.0 / crd->pg.dpi;
  /* background color */
  crd->bg = getbgcolor ();
  cr_initcmap ();                          		   /* Init GraceGTK colormap  */
}

//053b static GdkColormap *gg_cmap = NULL;
static int gg_color_last = 0;
/** 
 * Initialize GdkColor gg_colors[] array from cmap_table[],
 *        (the internal colormap of Grace-5)
 *  Called  via devinitcmap
 *  Replace gg_initcmap and  xlibinitcmap     x11drv.c  264
 */
void cr_initcmap (void)
{
  int i ,n;
  RGB *prgb;
  for (i = 0; i < MAXCOLORS; i++) {
    gg_colors[i].pixel = 0;
    gg_colors[i].red   = 0;
    gg_colors[i].green = 0;
    gg_colors[i].blue  = 0;
  }
  n = number_of_colors();                /* n=maxcolors */
  for (i = 0; i < n; i++) {
    prgb = get_rgb(i);
    if (prgb != NULL) {
      gg_colors[i].red   = (gint16)(prgb->red  *257);
      gg_colors[i].green = (gint16)(prgb->green*257);
      gg_colors[i].blue  = (gint16)(prgb->blue *257);
    } else {
      printf ("  gg_initcmap: prgb NULL for i=%d \n" ,i);
    }
  }
  gg_color_last = n;
}

static void cr_set_scale_unity (void)
{
  cairo_matrix_init (&pcr->matrix ,1.0 ,0. ,0. ,1.0 ,0. ,0.0);
  cairo_set_matrix  (pcr->cr ,&(pcr->matrix));
}

static void cr_VPoint2dev (VPoint vp, int *x, int *y)
{
  *x = pcr->scale  * vp.x;
  *y = pcr->height - vp.y * pcr->scale;
}

static void cr_set_scale (void)
{
  cairo_matrix_init (&(pcr->matrix) ,pcr->scale ,0. ,0. ,-pcr->scale ,0. ,pcr->height);
  cairo_set_matrix (pcr->cr ,&(pcr->matrix));
}
/******************** GTK canvas driver ********************/
/******************** NOT YET FUNCTIONNAL  ********************/
/*
 *  displaybuf is the GdkPixbuf buffer for w_canvas
 *  It is copied to w_canvas by gdk_draw_pixbuf() when a "expose_event"
 *  signal is emmited.
 *
 */

/*
main.c 	   218	tdevice = register_cairo_gtk_drv ();
gg.c   	   700	tcr_gtk_init_driver ();
gg.c   	   730 	g_signal_connect (w_canvas ,"expose_event" ,G_CALLBACK (tcr_expose_event_CB), NULL);
gg_files.c 154	cr_initcmap (); 

  tcr_expose_event_CB 
     init_batch_or_pipe
     gg_drawgraph
     	drawgraph
 	   initgraphics
	      tcr_gtk_initgraphics
	        cr_init
		cr_initcmap
	      Appels aux fonctions de tracer du driver: 
	   leavegraphics
	      tcr_leavegraphics
*/

extern GtkWidget *w_canvas;
extern gint gg_win_h ,gg_win_w;   // win_h ,win_w A REVOIR
#define gg_win_scale ((gg_win_h < gg_win_w) ? gg_win_h:gg_win_w)

extern GdkGC *gg_gcxor;   // gc , gcxor

static GdkScreen  *gdkscreen = NULL;
static GdkWindow  *gdkroot   = NULL;  /* needed to compute gg_gcxor */
//053 static GdkPixbuf *displaybuf = NULL;  /* the pixbuf we write into */


/** Declare and initialize the Device_entry for the GTK driver
 *  Replace dev_x11 or dev_gtk
 *  See device.h 80
 */
static Device_entry dev_cairo_gtk = {
  DEVICE_TERM,     	    /* type */
  "GTKcairo",			        	    /* *name : name of device */                            
  tcr_gtk_initgraphics,		        	    /* function to initialize device initgraphics() */             
  NULL,                                 	    /* function to parse device-specific commands */
  NULL,						    /* function (GUI interface) to setup device */  
  "",						    /* filename extension */                        
  FALSE,					    /* device has its own fonts */                  
  TRUE,						    /* font antialiasing */                         
  {DEFAULT_PAGE_WIDTH, DEFAULT_PAGE_HEIGHT, 72.0},  /* device Page_geometry defaults */                           
  NULL						    /* device private data */                       
};

/** 
 * Register the Device_entry structure
 *  Replace register_x11_drv or register_gtk_drv
 */
int register_cairo_gtk_drv (void)
{
  int max_path_length  = MAX_DRAWING_PATH;
  set_max_path_limit (max_path_length);
//053   gdkscreen  	       = gdk_screen_get_default ();
//053   dev_cairo_gtk.pg.dpi = rint (MM_PER_INCH * gdk_screen_get_width (gdkscreen) /
//053 			       gdk_screen_get_width_mm (gdkscreen) 
//053 			       );
  return register_device (dev_cairo_gtk);
}

static gint clipx ,clipy ,clipw ,cliph;

/**
 * tcr_wipeout:
 *
 * wipeout the canvas, reset background color to bg and clipping to w_canvas surface
 **/
void tcr_wipeout ()
{
  cairo_t 	       *cr = gdk_cairo_create (GDK_DRAWABLE (w_canvas->window));
  cairo_surface_t *surface = cairo_get_target (cr);
  gdk_cairo_set_source_color (cr ,&wheat);  	     /* default background is wheat */
  cairo_paint (cr);
  cairo_surface_finish (surface);
  cairo_destroy (cr);
  clipx = 0;
  clipy = 0;
  clipw = (gint)tcr.pg.width;
  cliph = (gint)tcr.pg.height;

  printf ("tcr_wipeout:\t clipw = %d cliph = %d\n" ,clipw ,cliph);

}

/**
 * Initialise the Cairo GTK driver struct
 */
int tcr_gtk_init_driver (void)
{
  double xres ,yres;
  GdkGCValues gc_val;

  gdkscreen  = gdk_screen_get_default  ();
  gdkroot    = gdk_get_default_root_window ();
  dev_cairo_gtk.pg.dpi = (float) gdk_screen_get_width (gdkscreen)
    / 			 (float) gdk_screen_get_width_mm (gdkscreen) 
    * 		   	 MM_PER_INCH;

  cr_init (&tcr);

  printf ("\ntcr_gtk_init_driver: scale = %f \t dscale = %f \n" ,tcr.scale ,tcr.dscale);

  /* Parameters and functions special to this driver */
  devleavegraphics = tcr_leavegraphics;	
//053   set_max_path_limit (MAX_DRAWING_PATH);
  /* set gg_gcxor to be used with handles and rubber stuff  */
  gc_val.foreground = gg_colors[0];
  gc_val.background = gg_colors[1];
  //  gc_val.function   = GDK_XOR;
  //  gc_val.fill 	    = GDK_SOLID;
  gg_gcxor = gdk_gc_new_with_values (gdkroot ,&gc_val ,GDK_GC_FUNCTION | GDK_GC_FOREGROUND);

  /* un essai */ //053
  gimp_get_screen_resolution (gdkscreen ,&xres ,&yres);
  printf ("\n tcr_gtk_init_driver: dpi = %f xres==%f  yres=%f\n" ,dev_cairo_gtk.pg.dpi ,xres ,yres);

  //053  cr_initcmap ();                          		   /* Init GraceGTK colormap  */
  cr_sync_canvas_size (&gg_win_w ,&gg_win_h ,PAGE_FIXED);
  tcr_wipeout (&wheat);
  init_batch_or_pipe ();
  return RETURN_SUCCESS;
}

gboolean tcr_configure_event_CB (GtkWidget *w ,GdkEventConfigure *event ,gpointer data)
{
  printf ("\n\t tcr_configure_event_CB \n\t x = %d \t y = %d \t width = %d \t height = %d \n"
	  ,event->x
	  ,event->y
	  ,event->width
	  ,event->height
	  );
  return TRUE;
}

/** 
 * tcr_expose_event_CB:
 * @w: the connected GtkWidget, i.e. w_canvas
 * @event: the structure used to retreive the rectangle to refresh
 * @data: not used
 *
 * Callback for "expose-event" emitted by w_canvas
 *  After the "expose_event", redraw the area exposed
 *  Replace gg_expose_resize_CB in gg_gutils.c 
 **/
gboolean tcr_expose_event_CB (GtkWidget *w ,GdkEventExpose *event, gpointer data)
{
  printf ("tcr_expose_event_CB: x = %d \t y = %d \t width = %d \t height = %d \n"
	  ,event->area.x
	  ,event->area.y
	  ,event->area.width
	  ,event->area.height
	  );
  // dans  drawgraph:  tcr_gtk_initgraphics ();
  /* We store clipping region to use in tcr_gtk_initgraphics */
  clipx = event->area.x;
  clipy = event->area.y;
  clipw = event->area.width;
  cliph = event->area.height;
  drawgraph       ();
  return TRUE;
}

/**
 *  Replace sync_canvas_size et gg_sync_canvas_size
 */
static void cr_sync_canvas_size (gint *w ,gint *h ,CanvasType canvas_type)
{
  Page_geometry pg = get_page_geometry();
  if (canvas_type == PAGE_FREE) {
    //053   GtkRequisition req;
    //053     gtk_widget_size_request (w_canvas ,&req);
    //053     *w = (unsigned int)req.width;
    //053     *h = (unsigned int)req.height;
    //053     /* Ce trafic me semble bizarre */
    //053     set_page_dimensions (*w*72.0/pg.dpi ,*h*72.0/pg.dpi ,TRUE);
    printf ("cr_sync_canvas_size: option PAGE_FREE A FAIRE\n");
  } else {          /* PAGE_FIXED */
    *w = (gint) pg.width;
    *h = (gint) pg.height;
    gtk_widget_set_size_request (w_canvas ,*w ,*h);
    /* Note that gtk_window_set_default_size () 
     * is incompatible with GtkDrawingArea */
  }

  printf ("cr_sync_canvas_size: w = %d \t h = %d\n" ,*w ,*h);

}



/** 
 *  Create the Cairo graphic context to draw on the canvas
 *  Replace xlibinitgraphics  x11drv.c  309  or gg_init graphics
 *  Called by drawgraph   in plotone.c   59
 */
int tcr_gtk_initgraphics (void)           /* initgraphics */
{
  cr_init (&tcr);
  if (tcr.surface == NULL) {
    tcr.cr      = gdk_cairo_create (GDK_DRAWABLE (w_canvas->window));
    tcr.surface = cairo_get_target (tcr.cr);
  } else {
    

  }
  printf ("tcr_gtk_initgraphics: gg_win_w = %d gg_win_h = %d  dpi = %f\n" ,gg_win_w ,gg_win_h ,tcr.pg.dpi);

  /* set the clipping rectangle according to  "expose-event" */
  printf ("\t clipx = %d \t clipy = %d \t clipw = %d \t cliph = %d \n"
	  ,clipx ,clipy ,clipw ,cliph);
  cairo_matrix_init_identity (&tcr.matrix);
  cairo_set_matrix  	     (tcr.cr ,&tcr.matrix);
  cairo_rectangle 	     (tcr.cr ,clipx ,clipy ,clipw ,cliph);
  cairo_clip 	      	     (tcr.cr);


  printf ("tcr_gtk_initgraphics: dscale = %f \n" ,tcr.dscale);

  pcr = &tcr;
  tcr.scale   = tcr.dscale;
  tcr.height = (double)tcr.pg.height;
  /*                                    xx  	   yx  xy      yy  	     x0       y0       */
  //053  cairo_matrix_init (&tcr.matrix ,tcr.scale ,0. ,0. ,-tcr.scale ,0. ,tcr.height);
  //053  cairo_set_matrix  (tcr.cr ,&tcr.matrix);
  cr_set_scale ();

  cr_sync_canvas_size (&gg_win_w ,&gg_win_h ,PAGE_FIXED);

  return RETURN_SUCCESS;
}

/**
 * A replacement for gdk_draw_rectangle.
 *  x ,y ,width and height are in pixels
 */
void cr_draw_rectangle (gboolean filled ,gint x ,gint y ,gint width ,gint height)
{
  cairo_t *cr_rect = gdk_cairo_create (GDK_DRAWABLE (w_canvas->window));
  cairo_rectangle  (cr_rect ,x ,y ,width ,height);
  if (filled) {
    cairo_fill   (cr_rect);
  } else {
    cairo_stroke (cr_rect);
  }
  cairo_destroy  (cr_rect);
}

void cr_handles (view v ,int nh ,int center)  // gg_handles
{
  double d  = 3.0 * tcr.pixel_size;
  double d2 = 2.0 * d;
  cr_draw_rectangle (TRUE ,v.xv1-d ,v.yv1-d ,d2 ,d2);

}

static void tcr_leavegraphics (void)  /* devleavegraphics gg_leavegraphics */
{
  printf ("tcr_leavegraphics: tcr.cr = %p tcr.surface = %p\n" ,tcr.cr ,tcr.surface);

  cairo_stroke         (tcr.cr);
  cairo_surface_finish (tcr.surface);
  cairo_destroy (tcr.cr);
  tcr.cr = NULL;
}


/******************** Common to PDF driver and SVG driver ********************/

static void hcr_leavegraphics (void)
{
  cairo_stroke (hcr.cr);
  cairo_surface_finish  (hcr.surface);
  cairo_surface_destroy (hcr.surface);
//053   cairo_destroy (hcr.cr);
}


/******************** PostScript driver ********************/

static Device_entry dev_cairo_eps = {
  DEVICE_FILE,                              /* type */
  "EPScairo",                               /* *name : name of device */
  hcr_eps_initgraphics,                     /* function to initialize device */
  NULL,  //22  cr_eps_op_parser,            /* function to parse device-specific commands */
  NULL,  //22  cr_eps_gui_setup,            /* function (GUI interface) to setup device */
  "eps",                                    /* filename extension */
  FALSE, //  TRUE,                          /* device has its own fonts */
  TRUE,  // FALSE,		   	    /* font antialiasing */
   {3300, 2550, 300.0},                     /* device defaults */
  NULL				   	    /* device private data */
};

/* TODO load fonts in device */

int register_cairo_eps_drv (void)
{
  return register_device (dev_cairo_eps);
}

int hcr_eps_initgraphics (void)           /* initgraphics */
{
  cr_init (&hcr);   		
  devleavegraphics = hcr_leavegraphics;		 
  /* cairo_eps_surface_create () needs a filename here */
  hcr.surface = cairo_ps_surface_create (print_file ,hcr.dwidth ,hcr.dheight);
  cairo_ps_surface_set_eps (hcr.surface ,TRUE);
  hcr.cr      = cairo_create (hcr.surface);
  pcr = &hcr;
  cr_set_scale ();
  return RETURN_SUCCESS;
}


/******************** PDF driver ********************/

static Device_entry dev_cairo_pdf = {
  DEVICE_FILE,                              /* type */
  "PDFcairo",                               /* *name : name of device */
  hcr_pdf_initgraphics,                      /* function to initialize device */
  NULL,  //22  cr_pdf_op_parser,            /* function to parse device-specific commands */
  NULL,  //22  cr_pdf_gui_setup,            /* function (GUI interface) to setup device */
  "pdf",                                    /* filename extension */
  FALSE, //  TRUE,                          /* device has its own fonts */
  TRUE,  // FALSE,		   	    /* font antialiasing */
   {3300, 2550, 300.0},                     /* device defaults */
  NULL				   	    /* device private data */
};

/* TODO load fonts in device */

int register_cairo_pdf_drv (void)
{
  return register_device (dev_cairo_pdf);
}

int hcr_pdf_initgraphics (void)           /* initgraphics */
{
  cr_init (&hcr);   		
  devleavegraphics = hcr_leavegraphics;		 
  /* cairo_pdf_surface_create () needs a filename here */
  hcr.surface = cairo_pdf_surface_create (print_file ,hcr.dwidth ,hcr.dheight);
  hcr.cr      = cairo_create (hcr.surface);
  /*                               	 xx   yx  xy        yy      x0   y0       */
  cairo_matrix_init (&hcr.matrix ,hcr.scale ,0. ,0. ,-hcr.scale ,0. ,hcr.height);
  cairo_set_matrix  (hcr.cr ,&hcr.matrix);
  pcr = &hcr;
  cr_set_scale ();
  return RETURN_SUCCESS;
}

/******************** PNG driver ********************/

static Device_entry dev_cairo_png = {
  DEVICE_FILE,                              /* type */
  "PNGcairo",                               /* *name : name of device */
  hcr_png_initgraphics,                     /* function to initialize device */
  NULL,           			    /* function to parse device-specific commands */
  NULL,           			    /* function (GUI interface) to setup device */
  "png",                                    /* filename extension */
  FALSE,                          	    /* device has its own fonts */
  TRUE, 		   	    	    /* font antialiasing */
  {DEFAULT_PAGE_WIDTH, DEFAULT_PAGE_HEIGHT, 72.0},  /* device defaults */
  NULL				   	    /* device private data */
};

int register_cairo_png_drv (void)
{
  return register_device (dev_cairo_png);
}

/**
 * PNG needs cairo_surface_write_to_png()
 */
void hcr_png_leavegraphics (void)
{
  cairo_stroke (hcr.cr);
  cairo_surface_write_to_png (hcr.surface ,print_file);
  cairo_surface_finish  (hcr.surface);
  cairo_surface_destroy (hcr.surface);
  cairo_destroy (hcr.cr);
}

int hcr_png_initgraphics (void)           /* initgraphics */
{
  Page_geometry pg;
  cr_init (&hcr);   	
  devleavegraphics = hcr_png_leavegraphics;
  pg      	   = get_page_geometry();                 /* return page dimensions in pixels */
  hcr.surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32 ,(int)pg.width ,(int)pg.height);
  hcr.cr      = cairo_create (hcr.surface);
  /*                               xx  	   yx  xy   yy  	 x0   y0       */
  cairo_matrix_init (&hcr.matrix ,hcr.scale ,0. ,0. ,-hcr.scale ,0. ,hcr.height);
  cairo_set_matrix  (hcr.cr ,&hcr.matrix);
  pcr = &hcr;
  return RETURN_SUCCESS;
}


/******************** SVG driver ********************/

static Device_entry dev_cairo_svg = {
  DEVICE_FILE,                              /* type */
  "SVGcairo",                               /* *name : name of device */
  hcr_svg_initgraphics,                      /* function to initialize device */
  NULL,         			    /* function to parse device-specific commands */
  NULL,         			    /* function (GUI interface) to setup device */
  "svg",                                    /* filename extension */
  FALSE, //  TRUE,                          /* device has its own fonts */
  TRUE,  // FALSE,		   	    /* font antialiasing */
  {DEFAULT_PAGE_WIDTH, DEFAULT_PAGE_HEIGHT, 72.0},  /* device defaults */
  NULL				   	    /* device private data */
};

int register_cairo_svg_drv (void)
{
  return register_device (dev_cairo_svg);
}

int hcr_svg_initgraphics (void)           /* initgraphics */
{
  cr_init (&hcr); 
  devleavegraphics = hcr_leavegraphics;
  hcr.surface = cairo_svg_surface_create (print_file ,hcr.dwidth ,hcr.dheight);
  hcr.cr      = cairo_create (hcr.surface);
  /*                               xx  	   yx  xy   yy  	 x0   y0       */
  cairo_matrix_init (&hcr.matrix ,hcr.scale ,0. ,0. ,-hcr.scale ,0. ,hcr.height);
  cairo_set_matrix  (hcr.cr ,&hcr.matrix);
  pcr = &hcr;
  return RETURN_SUCCESS;
}


/*************************************************************************/
/******               Functions common to all drivers               ******/
/*************************************************************************/

static void cr_setpen (const Pen *pen)
{
  if (pen->color != hcr.color || pen->pattern != pcr->pattern) {
    fRGB *frgb  = get_frgb (pen->color);
    pcr->color 	= pen->color;
    pcr->pattern = pen->pattern;
    cairo_set_source_rgb (pcr->cr ,frgb->red ,frgb->green ,frgb->blue);  /* setcolor */
  }
}

/**
 *  Set foreground color in cmap_table[fg] directly
 */
void cr_setfrgbcolor (double fred ,double fgreen ,double fblue)
{
  CMap_entry ce ,*ce_old;
  int fg;
  cairo_set_source_rgb (pcr->cr ,fred ,fgreen ,fblue);
  fg = getcolor ();
  ce_old = get_cmap_entry (fg);
  if (ce_old != NULL) {
    fred   *= (MAXCOLORS - 1);
    fgreen *= (MAXCOLORS - 1);
    fblue  *= (MAXCOLORS - 1);
    ce.rgb.red   = (int) fred;
    ce.rgb.green = (int) fgreen;
    ce.rgb.blue  = (int) fblue;
    ce.cname     = ce_old->cname;
    ce.ctype     = COLOR_MAIN;
    ce.tstamp    = 0;
    store_color (fg ,ce);
  }
}


static void cr_setdrawbrush (void)  /* model: pdf_setdrawbrush */
{
  int i ,ls;
  double *cdarr ,darr_len ,scale ,lw;
  Pen pen;
  pen = getpen ();  	  cr_setpen (&pen);
  lw  = getlinewidth ();  cairo_set_line_width (pcr->cr ,lw);
  ls  = getlinestyle ();
  if (ls > 1) {
    darr_len = dash_array_length[ls];
    cdarr = xmalloc ( darr_len * SIZEOF_DOUBLE);
    scale = MAX2 (1., lw) / pcr->scale;
    for (i = 0; i < darr_len; i++) {
      cdarr[i] = scale * (double)dash_array[ls][i];
    }
    cairo_set_dash (pcr->cr ,cdarr ,darr_len ,cdarr[0]);
//053     free (cdarr);
  } else {
    /* dashing disabled */
    cairo_set_dash (pcr->cr ,NULL ,0 ,0.);
  }
}

/* il semble que cette fonction n'est pas utilise */
void cr_drawpixel (VPoint vp)  // drawpixel
{
  printf ("cr_drawpixel : A FAIRE\n");
}

void cr_drawpolyline (VPoint *vps ,int n ,int mode)   // gg_drawpolyline
{
  int i;
  if (getlinestyle() == 0) return;
  cr_setdrawbrush ();
  //22  pdf_setlineprops();
  cairo_move_to (pcr->cr ,vps[0].x ,vps[0].y);
  for (i = 1; i < n; i++) {
    cairo_line_to (pcr->cr ,vps[i].x ,vps[i].y);
  }
  if (mode == POLYLINE_CLOSED) cairo_line_to (pcr->cr ,vps[0].x ,vps[0].y);
  cairo_save   	(pcr->cr);                          /* stack-pen-size-color */
  cairo_stroke 	(pcr->cr);
  cairo_restore (pcr->cr);                          /* stack-pen-size-color */
}


void cr_fillpolygon (VPoint *vps ,int n)
{
  int i ,j ,k ,l ,stride ,cindex;
  cairo_pattern_t * source;
  Pen pen;
  cairo_surface_t *patsurf;
  unsigned char data[16][16][4];

  pen = getpen ();
  if (pen.pattern == 0) return;
  cr_setpen (&pen);
  cairo_save (pcr->cr);                            /* stack-pen-size-color */
  if (getfillrule () == FILLRULE_WINDING) {
    cairo_set_fill_rule (pcr->cr ,CAIRO_FILL_RULE_WINDING);
  } else {
    cairo_set_fill_rule (pcr->cr ,CAIRO_FILL_RULE_EVEN_ODD);
  }
  cairo_move_to (pcr->cr ,vps[0].x ,vps[0].y);
  for (i = 1; i < n; i++) {
    cairo_line_to (pcr->cr ,vps[i].x ,vps[i].y);
  }
  if (pen.pattern > 1) {
    cairo_clip (pcr->cr);
    /* set scale to render pixmap at correct scale */
    cairo_identity_matrix (pcr->cr);
    stride  = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32 ,16);
    patsurf = cairo_image_surface_create_for_data ((unsigned char*) data ,CAIRO_FORMAT_ARGB32 ,16 ,16 ,stride);
    /* Convert XBM to CAIRO_FORMAT_ARGB32   */
    cindex = pen.color;
    for (j = 0; j < 256; j++) {
      k = j%16;
      l = j/16;
      if ((pat_bits[pen.pattern][j/8] >> (j%8)) & 0x01) {
	/* the bit is set */
#if G_BYTE_ORDER == G_LITTLE_ENDIAN
	data[l][k][0] = gg_colors[cindex].blue;
	data[l][k][1] = gg_colors[cindex].green;
	data[l][k][2] = gg_colors[cindex].red;
	data[l][k][3] = 0xff;
#else
	data[l][k][0] = 0xff;
	data[l][k][1] = gg_colors[cindex].red;
	data[l][k][2] = gg_colors[cindex].green;
	data[l][k][3] = gg_colors[cindex].blue;
#endif
      } else {
	/* transparent */
	data[l][k][0] = data[l][k][1] = data[l][k][2] = data[l][k][3] = 0x00;
      }
    }
    cairo_set_source_surface  (pcr->cr ,patsurf ,0. ,0.);
    source  = cairo_get_source (pcr->cr);
    cairo_pattern_set_extend (source ,CAIRO_EXTEND_REPEAT);
    cairo_paint (pcr->cr);
    cairo_restore (pcr->cr);                          /* stack-pen-size-color */
  } else {
    cairo_fill (pcr->cr);
    cairo_restore (pcr->cr);                          /* stack-pen-size-color */
  }
}


/**
 *
 */
void cr_drawarc (VPoint vp1 ,VPoint vp2 ,int angle1 ,int angle2)  // pdf_drawarc
{
  double xc ,yc ,rx ,ry ,a1 ,a2;
  cr_setdrawbrush ();
  cairo_save (pcr->cr);
  xc = (vp1.x + vp2.x) / 2;
  yc = (vp1.y + vp2.y) / 2;
  rx    = fabs (vp2.x - vp1.x) / 2;
  ry    = fabs (vp2.y - vp1.y) / 2;
  a1 = ((double)angle1) * M_PI / 180.0;
  a2 = ((double)angle2) * M_PI / 180.0;
  cairo_new_sub_path (pcr->cr);
  if (rx == ry) {
    /* cairo_arc() draws only circles */
    cairo_arc (pcr->cr ,xc ,yc ,rx ,a1 ,a2);
  } else {
    /* we change the scale along Ox */
    double ell = rx / ry;
  /*                               	 xx  	         yx  xy        yy    	 	  x0                     y0       */
    cairo_matrix_init (&pcr->matrix ,pcr->scale*ell ,0. ,0. ,-pcr->scale ,(1.-ell)*pcr->scale*xc ,pcr->height);
    cairo_set_matrix  (pcr->cr ,&pcr->matrix);
    cairo_arc (pcr->cr ,xc ,yc ,ry ,a1 ,a2);
  }
  cairo_stroke  (pcr->cr);
  cairo_restore (pcr->cr);                          /* stack-pen-size-color */
}

void cr_fillarc(VPoint vp1 ,VPoint vp2 ,int angle1 ,int angle2 ,int mode)
{
  double xc ,yc ,rx ,ry ,a1 ,a2;
  int j ,k ,l ,stride ,cindex;
  cairo_pattern_t * source;
  Pen pen;
  cairo_surface_t *patsurf;
  unsigned char data[16][16][4];

  pen = getpen ();
  if (pen.pattern == 0) return;
  cr_setpen (&pen);
  cairo_save (pcr->cr);
  xc = (vp1.x + vp2.x) / 2;
  yc = (vp1.y + vp2.y) / 2;
  rx    = fabs (vp2.x - vp1.x) / 2;
  ry    = fabs (vp2.y - vp1.y) / 2;
  a1 = ((double)angle1) * M_PI / 180.0;
  a2 = ((double)angle2) * M_PI / 180.0;
  cairo_new_sub_path (pcr->cr);
  if (mode == ARCFILL_PIESLICE) {
    cairo_move_to (pcr->cr ,xc ,yc);
  }
  if (rx == ry) {
    /* cairo_arc() draws only circles */
    cairo_arc (pcr->cr ,xc ,yc ,rx ,a1 ,a2);
  } else {
    /* we change the scale along Ox */
    double ell = rx / ry;
    /*                               xx  	         yx  xy   yy    	 	 x0                     y0       */
    cairo_matrix_init (&pcr->matrix ,pcr->scale*ell ,0. ,0. ,-pcr->scale ,(1.-ell)*pcr->scale*xc ,pcr->height);
    cairo_set_matrix  (pcr->cr ,&pcr->matrix);
    cairo_arc (pcr->cr ,xc ,yc ,ry ,a1 ,a2);
  }
  if (mode == ARCFILL_PIESLICE) {
    cairo_close_path (pcr->cr);
  }
  if (pen.pattern > 1) {
    cairo_clip (pcr->cr);
    /* set scale to render pixmap at correct scale */
    cairo_identity_matrix (pcr->cr);
    stride  = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32 ,16);
    patsurf = cairo_image_surface_create_for_data ((unsigned char*) data ,CAIRO_FORMAT_ARGB32 ,16 ,16 ,stride);
    /* Convert XBM to CAIRO_FORMAT_ARGB32   */
    cindex = pen.color;
    for (j = 0; j < 256; j++) {
      k = j%16;
      l = j/16;
      if ((pat_bits[pen.pattern][j/8] >> (j%8)) & 0x01) {
	/* the bit is set */
#if G_BYTE_ORDER == G_LITTLE_ENDIAN
	data[l][k][0] = gg_colors[cindex].blue;
	data[l][k][1] = gg_colors[cindex].green;
	data[l][k][2] = gg_colors[cindex].red;
	data[l][k][3] = 0xff;
#else
	data[l][k][0] = 0xff;
	data[l][k][1] = gg_colors[cindex].red;
	data[l][k][2] = gg_colors[cindex].green;
	data[l][k][3] = gg_colors[cindex].blue;
#endif
      } else {
	/* transparent */
	data[l][k][0] = data[l][k][1] = data[l][k][2] = data[l][k][3] = 0x00;
      }
    }
    cairo_set_source_surface  (pcr->cr ,patsurf ,0. ,0.);
    source  = cairo_get_source (pcr->cr);
    cairo_pattern_set_extend (source ,CAIRO_EXTEND_REPEAT);
    cairo_paint (pcr->cr);
    cairo_restore (pcr->cr);                          /* stack-pen-size-color */
  } else {
    cairo_fill (pcr->cr);
    cairo_restore (pcr->cr);                          /* stack-pen-size-color */
  }
}

int cr_op_parser(char *opstring)
{
        return RETURN_SUCCESS;
}

/* TODO: transparent pixmaps */

/**
 * Used by t1fonts.c to draw glyphs via devputpixmap
 */
void cr_putpixmap (VPoint vp ,int width ,int height ,char *databits ,   // devputpixmap
			  int pixmap_bpp ,int bitmap_pad ,int pixmap_type)
{
  unsigned char *data;
  int cindex;
  int i ,j, k ,stride;
  double x ,y;
  double pixscale;
  cairo_surface_t  *s;
  Page_geometry    pg;
  cairo_save (pcr->cr);
  pg = get_page_geometry ();                 /* return page dimensions in pixels */
  pixscale = 72. / pg.dpi;                   /* scaling if page resolution differs from 72 dpi */
  cairo_matrix_init (&pcr->matrix ,pixscale ,0. ,0. ,pixscale ,0. ,0.0);
  cairo_set_matrix  (pcr->cr ,&pcr->matrix);

  data   = xmalloc (width * height * 4);
  stride = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32 ,width);
  s      = cairo_image_surface_create_for_data (data ,CAIRO_FORMAT_ARGB32 ,width ,height ,stride);
  i = 0;
  for (k = 0; k < height; k++) {
    for (j = 0; j < width; j++) {
      cindex = (unsigned char) (databits)[k*width+j];
      if (cindex != pcr->bg) {
#if G_BYTE_ORDER == G_LITTLE_ENDIAN
	data[i] = gg_colors[cindex].blue;  i++;
	data[i] = gg_colors[cindex].green; i++;
	data[i] = gg_colors[cindex].red;   i++;
#else
	data[i] = gg_colors[cindex].red;   i++;
	data[i] = gg_colors[cindex].green; i++;
	data[i] = gg_colors[cindex].blue;  i++;
#endif
	data[i] = 0xff;                    i++;  /* opaque */
      } else {
	data[i] = 0; i++;
	data[i] = 0; i++;
	data[i] = 0; i++;
	data[i] = 0; i++;                        /* transparent */
      }
    }
  }
  x = pcr->scale  * vp.x 	        / pixscale;
  y = (pcr->height - vp.y * pcr->scale) / pixscale;
  cairo_set_source_surface (pcr->cr ,s ,x ,y);
  cairo_paint (pcr->cr);
  cr_set_scale ();                /* return to initial scaling. */
  cairo_restore (pcr->cr);        /* It seems that freeing data and s is done by cairo_restore */
}

/************************* cr_putpixbuf **********************/

/**
 * Scales if necessary and draw a pixbuf
 */
void cr_putpixbuf (void *data ,VPoint *vp1 ,VPoint *vp2  		   /* devputpixbuf */
		   ,int scaling_type ,double *scale_x ,double *scale_y)
{
  GdkPixbuf *pixbuf = (GdkPixbuf *)data;
  int x1, y1, x2, y2 ,x_min ,y_min;
  cairo_save (pcr->cr);
  cr_set_scale_unity ();
  cr_VPoint2dev (*vp1 ,&x1 ,&y1);
  cr_VPoint2dev (*vp2 ,&x2 ,&y2);
  x_min = MIN2 (x1 ,x2);
  y_min = MIN2 (y1 ,y2);
  gdk_cairo_set_source_pixbuf (pcr->cr ,pixbuf ,x_min ,y_min);

  cairo_rectangle (pcr->cr ,x_min ,y_min ,abs(x2-x1) ,abs(y2-y1));
  cairo_clip 	  (pcr->cr);

  cairo_paint (pcr->cr);
  /* return to initial scaling */
  cairo_matrix_init (&pcr->matrix ,pcr->scale ,0. ,0. ,-pcr->scale ,0. ,pcr->height);
  cairo_set_matrix  (pcr->cr ,&pcr->matrix);
  cairo_restore (pcr->cr);
}


void cr_puttext (VPoint vp ,char *s ,int len ,int font,
		 TextMatrix *tm ,int underline ,int overline ,int kerning)  /* devputtext */
{
  fprintf (stderr ,"cr_puttext not yet implemented\n");
}


#define GIMP_MIN_RESOLUTION  5e-3      /*  shouldn't display as 0.000  */
#define GIMP_MAX_RESOLUTION  65536.0
#define ROUND(x) ((int) ((x) + 0.5))



/**
 * gimp_get_screen_resolution:
 * @screen: a #GdkScreen or %NULL
 * @xres: returns the horizontal screen resolution (in dpi)
 * @yres: returns the vertical screen resolution (in dpi)
 *
 * Retrieves the screen resolution from GDK. If @screen is %NULL, the
 * default screen is used.
 *
 * From /usr/local/src/gimp-2.8.0/app/widgets/gimpwidgets-utils.c 717
 **/
void gimp_get_screen_resolution (GdkScreen *screen,
				 gdouble   *xres,
				 gdouble   *yres)
{
  gint    width, height;
  gint    width_mm, height_mm;
  gdouble x = 0.0;
  gdouble y = 0.0;

  g_return_if_fail (screen == NULL || GDK_IS_SCREEN (screen));
  g_return_if_fail (xres != NULL);
  g_return_if_fail (yres != NULL);

  if (!screen)
    screen = gdk_screen_get_default ();

  width  = gdk_screen_get_width (screen);
  height = gdk_screen_get_height (screen);

  width_mm  = gdk_screen_get_width_mm (screen);
  height_mm = gdk_screen_get_height_mm (screen);

  /*
   * From xdpyinfo.c:
   *
   * there are 2.54 centimeters to an inch; so there are 25.4 millimeters.
   *
   *     dpi = N pixels / (M millimeters / (25.4 millimeters / 1 inch))
   *         = N pixels / (M inch / 25.4)
   *         = N * 25.4 pixels / M inch
   */

  if (width_mm > 0 && height_mm > 0)
    {
      x = (width  * 25.4) / (gdouble) width_mm;
      y = (height * 25.4) / (gdouble) height_mm;
    }

  if (x < GIMP_MIN_RESOLUTION || x > GIMP_MAX_RESOLUTION ||
      y < GIMP_MIN_RESOLUTION || y > GIMP_MAX_RESOLUTION)
    {
      g_warning ("GDK returned bogus values for the screen resolution, "
                 "using 96 dpi instead.");

      x = 96.0;
      y = 96.0;
    }

  /*  round the value to full integers to give more pleasant results  */
  *xres = ROUND (x);
  *yres = ROUND (y);
}
