/** File fourier.h
 */

#ifndef __FOURIER_H_
#define __FOURIER_H_

void gg_fourier_update (int gno1);

typedef enum {
  FFT_NORM_NONE = 0
  ,FFT_NORM_SYMMETRIC
  ,FFT_NORM_FORWARD
  ,FFT_NORM_BACKWARD
} FFT_norm;

typedef enum {
  FFT_COMPLEX_IN_NONE = 0  /* Input data is real */
  ,FFT_COMPLEX_IN_COL3     /* The imaginary part is in DATA_Y1 */
  ,FFT_COMPLEX_IN_SET      /* The imaginary part is another set */
} FFT_complex_in;

typedef enum {
  FFT_INPUT_SEG_INDEX
  ,FFT_INPUT_SEG_X
} FFT_input_segment;

typedef enum {
  FFT_XSCALE_INDEX = 0
  ,FFT_XSCALE_NU
  ,FFT_XSCALE_OMEGA
} FFT_xscale;

typedef enum {
  FFT_WINDOW_NONE = 0
  ,FFT_WINDOW_TRIANGULAR
  ,FFT_WINDOW_PARZEN    
  ,FFT_WINDOW_WELCH     
  ,FFT_WINDOW_HANNING   
  ,FFT_WINDOW_HAMMING   
  ,FFT_WINDOW_FLATTOP   
  ,FFT_WINDOW_BLACKMAN  
} FFT_filters;


typedef enum {
  FFT_OUTPUT_MAGNITUDE = 0
  ,FFT_OUTPUT_PHASE
  ,FFT_OUTPUT_RE
  ,FFT_OUTPUT_IM
  ,FFT_OUTPUT_REIM
  ,FFT_OUTPUT_APHI
  ,FFT_OUTPUT_COEFF
} FFT_output;

typedef enum {
  FFT_OUTSEGMT_POSITIVE
  ,FFT_OUTSEGMT_HALF_LENGTH
  ,FFT_OUTSEGMT_CENTERED
} FFT_outsegmt;

#ifdef __FOURIER_WIN_
/* To be used in the follow_me_on mode. Names are also used in pars.yacc */

static char *fouriernorm[]  = {"None" ,"Symmetric" ,"Forward" ,"Backward"};
static char *fourierinseg[] = {"Index" ,"Insegx"};
static char *windowtype[] = {"None"
			 ,"Triangular"		     /*	FFT_WINDOW_TRIANGULAR */
			 ,"Parzen"		     /*	FFT_WINDOW_PARZEN     */
			 ,"Welch"		     /*	FFT_WINDOW_WELCH      */
			 ,"Hanning"		     /*	FFT_WINDOW_HANNING    */
			 ,"Hamming"		     /*	FFT_WINDOW_HAMMING    */
			 ,"Flat top"		     /*	FFT_WINDOW_FLATTOP    */
			 ,"Blackman"		     /*	FFT_WINDOW_BLACKMAN   */
};
static char *fourierloadx[] = {"Index" ,"Frequency" ,"Period"};
static char *fourierloady[] = {"Magnitude" 	     /* FFT_OUTPUT_MAGNITUDE */
			 ,"Phase"           	     /* FFT_OUTPUT_PHASE     */
			 ,"Real part"       	     /* FFT_OUTPUT_RE        */
			 ,"Imaginary part"  	     /* FFT_OUTPUT_IM        */
			 ,"Complex"         	     /* FFT_OUTPUT_REIM      */
			 ,"Complex (A, Phi)"	     /* FFT_OUTPUT_APHI      */
};
static char *fourieroutsg[] = {"Positive"            /* FFT_OUTSEGMT_POSITIVE    */
			 ,"Half"     		     /* FFT_OUTSEGMT_HALF_LENGTH */
			 ,"Center"                   /* FFT_OUTSEGMT_CENTERED    */
};
#endif /* __FOURIER_WIN_ */


typedef struct _FFTParams FFTParams;

struct _FFTParams {
  int               invflag;       /* 0: forward 1: backward */
  FFT_xscale        xscale;
  FFT_norm          norm;
  FFT_input_segment insegmt;
  FFT_complex_in    complexin;     
  int          	    set1_im;       /* if complexin = FFT_COMPLEX_IN_SET, the setno */
  int          	    dcdump;
  int          	    round2n;
  FFT_filters  	    filter;
  double       	    padding;
  FFT_outsegmt 	    outsegmt;
  FFT_output        output;
};

void fourier_set_defaults (void);
int fourier_params_get  (FFTParams *p);
int fourier_params_set  (FFTParams *p);
int fourier_nb_out_sets (FFT_output output);


  
void do_fourier_command (int gno1 ,int setno1 ,int invflag ,FFT_filters filter);
  
void do_fourier_command_1 (int gno1 ,int setno1 			 
			   ,int invflag
			   ,FFT_xscale xscale
			   ,int complexin
			   ,FFT_filters filter
			   ,FFT_output output
			   ,int insegmt ,int outsegmt);
void do_fourier_command_2 (int gno1 ,int setno1 ,int gno2 ,int setno2);
void do_fourier_featext   (int gno1 ,int setno1 ,int *setno2);
  

int do_fourier (int gno1 ,int setno1 ,int gno2 ,int setno2 ,int setno2im);

int fourier (double *jr, double *ji, int n, int iflag);


#endif /* __FOURIER_H_ */
