/** File gg_compwin.c
 *  Transcripted from Motif to GTK+ by P. Vincent to
 *  Replace compwin.c
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <cmath.h>

#include <gtk/gtk.h>

#include "defines.h"
#include "globals.h"
#include "graphs.h"
#include "utils.h"
#include "noxprotos.h"
#include "ssdata.h"

#include "gw_list.h"

#include "gg_gtkinc.h"
#include "gg_protos.h"
#include "ge_protos.h"

#include "computils.h"
#include "draw.h"
#include "zc.h"


static gint popup_width = 450  ,list_height = 250;

extern GdkColor  bleuciel ,wheat;

static void gg_compute_AAC_CB 	(GtkWidget *w ,gint reponse);
static void gg_interp_AAC_CB  	(GtkWidget *w ,gint reponse);
static void gg_histo_AAC_CB   	(GtkWidget *w ,gint reponse);

static void gg_runavg_AAC_CB    (GtkWidget *w ,gint reponse);
static void gg_regui_AAC_CB   	(GtkWidget *w ,gint reponse);

static void gg_iui_AAC_CB     	(GtkWidget *w ,gint reponse);
static void gg_sui_AAC_CB     	(GtkWidget *w ,gint reponse);
static void gg_crossui_AAC_CB 	(GtkWidget *w ,gint reponse);
static void gg_samp_AAC_CB    	(GtkWidget *w ,gint reponse);
static void gg_prune_AAC_CB   	(GtkWidget *w ,gint reponse);
static void gg_digfilter_AAC_CB (GtkWidget *w ,gint reponse);
static void gg_linearc_AAC_CB   (GtkWidget *w ,gint reponse);
static void gg_geom_AAC_CB      (GtkWidget *w ,gint reponse);

static void gg_prune_toggle (void);

static char bufcmd[256];

/********** Evaluate expression *************/

typedef struct _gg_Eval_ui {
  GtkWidget *top;
  gg_SrcDestStructure *srcdest;
  GtkTextBuffer *formula_item;
  gg_RestrictionStructure *restr_item;
} gg_Eval_ui;

static gg_Eval_ui eui;

/**
 *  Replace  create_eval_frame    compwin.c   90
 */
void gg_create_eval_popup (void)
{
  GtkWidget *vb;

  if (eui.top == NULL) {
    eui.top          = gg_CreateAACDialog ("evaluateExpression" ,gg_compute_AAC_CB ,0 ,0 ,&vb);
    eui.srcdest      = gg_CreateSrcDestSelector   (vb ,GTK_SELECTION_MULTIPLE);
    eui.formula_item = gg_CreateScrollTextItem2   (vb ,popup_width ,40 ,"Formula:");
    eui.restr_item   = gg_CreateRestrictionChoice (vb ,"Source data filtering");
  }
  gtk_widget_show_all (eui.top);
}



/**
 *  Replace  compute_aac    compwin.c   153
 */
static void gg_compute_AAC_CB (GtkWidget *w ,gint reponse)
{
  int gno1, gno2 ,ns1 ,ns2 ,resno ,i;
  int setno1 = -1;
  int setno2 = -1;
  gint *sl1 ,*sl2;
  GtkTextIter start ,end;
  char fstr[256] ,buf[300];
  int restr_type, restr_negate;
  char *rarray;

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY   ) {
    restr_type   = gg_get_int (eui.restr_item->r_sel) - 1;
    if (restr_type == 6) restr_type = RESTRICT_WORLD;
    restr_negate = gg_get_int (eui.restr_item->negate);
    if (gg_GetSrcDestSettings (eui.srcdest ,SD_NSET1_EQ_NSET2
                               ,&gno1 ,&gno2
                               ,&ns1  ,&ns2
                               ,&sl1  ,&sl2) == RETURN_FAILURE) return;

    gtk_text_buffer_get_bounds (eui.formula_item ,&start ,&end);
    strcpy (fstr, gtk_text_buffer_get_text (eui.formula_item
                                            ,&start ,&end ,FALSE));
    for (i = 0; i < ns1; i++) {
      setno1 = sl1[i];
      setno2 = sl2[i];
      resno = get_restriction_array (gno1, setno1 ,restr_type
                                     ,restr_negate ,&rarray);
      if (resno != RETURN_SUCCESS) {
        errmsg("Error in evaluation restriction");
        break;
      }
      resno = do_compute (gno1, setno1, gno2, setno2, rarray, fstr);
      XCFREE(rarray);
      if (resno != RETURN_SUCCESS) {
        errmsg ("Error in do_compute(), check formula");
        break;
      }
    }
    sprintf (buf ,"(%s) on G%d.s%d" ,fstr ,gno1 ,setno1);
    comment_and_legend_set (gno2 ,setno2 ,buf);
    zc_label_free_list (gno2 ,setno2);
    gg_update_all_sets_lists ();
    ge_update_explorer ();
    gg_drawgraph ();
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (eui.top);
  }
}


/**********  Interpolation *************/

#define SAMPLING_MESH   0
#define SAMPLING_SET    1

/* interpolation */

typedef struct _gg_Interp_ui {
  GtkWidget *dialog;
  gg_SrcDestStructure *srcdest;
  GtkWidget *method;
  GtkWidget *sarray;
  GtkWidget *sampling;
  GtkWidget *strict;
  GtkWidget *mrc;
  GtkWidget *mstart;
  GtkWidget *mstop;
  GtkWidget *mlength;
  GtkWidget *xspline_s;
  GtkWidget *xspline_p;
  GtkWidget *sset_sel;
  SetsMenu3Struct  *sset_menu3;
} gg_Interp_ui;

/* See interpmethod in pars.yacc */
static char *interpmethod[] = {"LINEAR" ,"SPLINE" ,"ASPLINE" ,"XSPLINE" ,"NOT USED"};


static gg_Interp_ui *interpui = NULL;

static void method_changed_CB (GtkWidget *w ,gpointer p);

/**
 *  Replace  sampling_cb    compwin.c   258
 */
static void gg_sampling_CB (GtkWidget *w ,gpointer p)
{
  gint value = gg_get_int (interpui->sampling);
  if (value == SAMPLING_MESH) {
    gtk_widget_set_sensitive (interpui->mrc      ,TRUE);
    gtk_widget_set_sensitive (interpui->sset_sel ,FALSE);
  } else {
    gtk_widget_set_sensitive (interpui->mrc      ,FALSE);
    gtk_widget_set_sensitive (interpui->sset_sel ,TRUE);
  }
}

/**
 *  Replace  create_interp_frame    compwin.c   271
 */
void gg_create_interp_popup (void)
{
  GtkWidget *vb ,*rc2;
  if (interpui == NULL) {
    interpui = xmalloc (sizeof (gg_Interp_ui));
    interpui->dialog  = gg_CreateAACDialog ("Interpolation" ,gg_interp_AAC_CB ,0 ,0 ,&vb);
    interpui->srcdest = gg_CreateSrcDestSelector (vb ,GTK_SELECTION_MULTIPLE);

    rc2 = gg_CreateHContainer (vb);
    interpui->method   = gg_combo_new (rc2 ,"Method:" ,6
				       ,"Linear"        /* INTERP_LINEAR  */
				       ,"Cubic spline"  /* INTERP_SPLINE  */
				       ,"Akima spline"  /* INTERP_ASPLINE */
				       ,"X-spline"      /* INTERP_XSPLINE */
				       ,"XY-spline");   /* INTERP_XYSPLINE */

    interpui->sarray   = gg_frame (vb ,"Sampling array" ,&wheat ,0 ,1);
    interpui->strict   = gg_check_new (interpui->sarray ,"Strict (within source set bounds)");
    interpui->sampling = gg_combo_new (interpui->sarray ,"Sampling:" ,3
				       ,"Linear mesh"                /* SAMPLING_MESH */
				       ,"Abscissas of another set"); /* SAMPLING_SET */
    interpui->mrc = gg_CreateHContainer (interpui->sarray);
    interpui->mstart  = gg_entry_new (interpui->mrc ,"Start at:" ,10);
    interpui->mstop   = gg_entry_new (interpui->mrc ,"Stop at:" ,10);
    interpui->mlength = gg_entry_new (interpui->mrc ,"Length:" ,6);

    interpui->xspline_s  = gg_spin_new (vb ,"X-spline param s"   ,-1.0 ,1.0 ,0.1);
    interpui->xspline_p  = gg_spin_new (vb ,"X-spline param p"   ,0.002  ,0.5  ,0.002);
    gg_set_dble (interpui->xspline_s ,-0.5);
    gg_set_dble (interpui->xspline_p ,0.002);

    interpui->sset_menu3 = NULL;
    interpui->sset_sel = gg_CreateSetChoice ("Sampling set" ,GTK_SELECTION_SINGLE
                                             ,popup_width ,list_height
                                             ,NULL
					     ,&(interpui->sset_menu3)
					     ,NULL);
    gtk_box_pack_start_defaults (GTK_BOX (vb) ,interpui->sset_sel);
    gtk_widget_set_sensitive (interpui->sset_sel ,FALSE);

    g_signal_connect (interpui->sampling ,"changed" ,G_CALLBACK (gg_sampling_CB) ,interpui);
    g_signal_connect (interpui->method   ,"changed" ,G_CALLBACK (method_changed_CB)      ,NULL);

    method_changed_CB (interpui->method,NULL);

    //051 Provisoire pour mise au point
    gg_set_int (interpui->method ,INTERP_XYSPLINE);
    gtk_entry_set_text (GTK_ENTRY (interpui->mstart)    ,"0.0");
    gtk_entry_set_text (GTK_ENTRY (interpui->mstop)    ,"20.0");
    gtk_entry_set_text (GTK_ENTRY (interpui->mlength) ,"100.0");



  }
  gtk_widget_show_all (interpui->dialog);

}


/**
 *  Replace  do_interp_proc    compwin.c   323
 */
static void gg_interp_AAC_CB (GtkWidget *w ,gint reponse)
{
  int error, res;
  int gno1, gno2 ,ns1 ,ns2;
  gint *sl1 ,*sl2;
  int method, sampling, strict;
  int i, meshlen ,setlen;
  double *mesh = NULL;
  double start, stop;
  int setnosrc, setnodest;
  double *x ,*y ,*xint ,*yint ,xspline_s ,xspline_p;
  int interp_setno ,n_interp;
  int gsampl = 0;
  int setnosampl;
  char buf[64];

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY   ) {
    error = FALSE;
    if (gg_GetSrcDestSettings (interpui->srcdest ,SD_NSET1_EQ_NSET2 | SD_EXCLUSIVE
                               ,&gno1 ,&gno2
                               ,&ns1  ,&ns2
                               ,&sl1  ,&sl2) == RETURN_FAILURE) return;
    method   = gg_get_int (interpui->method);
    if (method == INTERP_XYSPLINE) {
      if (ns1 != 1 && ns2 != 1) {
	errmsg ("XY-spline requires only one source and destination  set");
	return;
      }
      setlen       = getsetlength (gno1 ,sl1[0]);
      xspline_s    = gg_get_dble (interpui->xspline_s);
      xspline_p    = gg_get_dble (interpui->xspline_p);
      interp_setno = sl2[0];
      xint 	   = xcalloc (setlen ,SIZEOF_DOUBLE);
      yint 	   = xcalloc (setlen ,SIZEOF_DOUBLE);
      x 	   = getx (gno1 ,sl1[0]);
      y 	   = gety (gno1 ,sl1[0]);
      if (xspline_eval_xy (&xint ,&yint ,setlen ,x ,y ,setlen ,&n_interp ,xspline_s ,xspline_p) == RETURN_FAILURE) {
	xfree (xint);
	xfree (yint);
	return;
      }
      copyset     (gno1 ,sl1[0] ,gno2 ,sl2[0]);
      killsetdata (gno2 ,sl2[0]);
      setcol 	  (gno2 ,interp_setno ,DATA_X ,xint ,n_interp);
      setcol 	  (gno2 ,interp_setno ,DATA_Y ,yint ,n_interp);
      sprintf (buf, "XY-spline of G%d.s%d", gno1 ,sl1[0]);
      comment_and_legend_set (gno2 ,interp_setno ,buf);
      sampling = SAMPLING_MESH;
      if (follow_me_on) {
	snprintf (bufcmd ,255 ,"@ XYSPLINE ( G%d.s%d , G%d.s%d , %g , %g )\n" 
		  ,gno1 ,sl1[0]  ,gno2 ,interp_setno ,xspline_s ,xspline_p);
	gg_cmd_prt (bufcmd);
      }
    } else {              /* method != INTERP_XYSPLINE */
      sampling = gg_get_int (interpui->sampling);
      strict   = gg_get_int (interpui->strict);
      if (sampling == SAMPLING_SET) {
	gsampl = get_cg ();
	res = gg_GetSingleListChoice (GW_LIST (interpui->sset_sel) ,&setnosampl);
	if (res != TRUE) {
	  errmsg("Please select single sampling set");
	  return;
	} else {
	  meshlen = getsetlength (gsampl, setnosampl);
	  mesh = getcol (gsampl, setnosampl, DATA_X);
	}
      } else {
	if (gg_evalexpr  (interpui->mstart, &start)    != RETURN_SUCCESS ||
	    gg_evalexpr  (interpui->mstop,  &stop)     != RETURN_SUCCESS ||
	    gg_evalexpri (interpui->mlength, &meshlen) != RETURN_SUCCESS ) {
	  errmsg ("Can't parse mesh settings");
	  return;
	} else {
	  mesh = allocate_mesh (start, stop, meshlen);
	  if (mesh == NULL) {
	    errmsg ("Can't allocate mesh");
	    return;
	  }
	}
      }
      for (i = 0; i < ns1; i++) {
	setnosrc = sl1[i];
	if (ns2 != 0) {
	  setnodest = sl2[i];
	} else {
	  setnodest = SET_SELECT_NEXT;
	}
	res = do_interp (gno1, setnosrc, gno2, setnodest,
			 mesh, meshlen, method, strict
			 ,  /* Default for INTERP_XSPLINE xspline_s = */ -0.5);
	if (res != RETURN_SUCCESS) {
	  errmsg ("Error in do_interp()");
	  error = TRUE;
	  break;
	}
	if (follow_me_on) {
	  if (sampling == SAMPLING_SET) {
	    snprintf (bufcmd ,255 ,"@ INTERPOLATE ( G%d.s%d , G%d.s%d  , G%d.s%d.x ,%s , %s )\n"
		      ,gno1 ,sl1[i] ,gno2 ,sl2[i] ,gsampl ,setnosampl ,interpmethod[method] ,on_or_off(strict));
	  } else {
	    snprintf (bufcmd ,255 ,"@ INTERPOLATE ( G%d.s%d , G%d.s%d , MESH (%g ,%g ,%d) ,%s , %s )\n"
		      ,gno1 ,sl1[i] ,gno2 ,sl2[i] ,start, stop, meshlen ,interpmethod[method] ,on_or_off(strict));
	  }
	  gg_cmd_prt (bufcmd);
	}
      }
    }
    xfree (sl1);
    if (ns2 > 0) xfree (sl2);
    if (sampling == SAMPLING_MESH) xfree (mesh);

    gg_update_all_sets_lists ();
    gg_drawgraph ();
    if (error) return;
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}

static void method_changed_CB (GtkWidget *w ,gpointer p)
{
  if (interpui != NULL) {
    int m = gg_get_int (w);
    gtk_widget_set_sensitive (interpui->sarray    ,m != INTERP_XYSPLINE);
    gtk_widget_set_sensitive (interpui->xspline_p ,m == INTERP_XYSPLINE);
    gtk_widget_set_sensitive (interpui->xspline_s ,m == INTERP_XYSPLINE || m == INTERP_XSPLINE);

  }
}


/******************* Histograms  *********************/

typedef struct _gg_Histo_ui {
  GtkWidget *dialog;
  gg_SrcDestStructure *srcdest;
  GtkWidget *cumulative;
  GtkWidget *normalize;
  GtkWidget *sampling;
  GtkWidget *mrc;
  GtkWidget *mstart;
  GtkWidget *mstop;
  GtkWidget *mlength;
  GtkWidget *sset_sel;
  SetsMenu3Struct * sset_menu3;
} gg_Histo_ui;

static gg_Histo_ui *histoui = NULL;

/**
 *  Replace binsampling_cb  compwin.c   435
 */
static void gg_binsampling_CB (GtkWidget *w ,gpointer p)
{

  gint value = gg_get_int (histoui->sampling);
  if (value == SAMPLING_MESH) {
    gtk_widget_set_sensitive (histoui->mrc      ,TRUE);
    gtk_widget_set_sensitive (histoui->sset_sel ,FALSE);
  } else {
    gtk_widget_set_sensitive (histoui->mrc      ,FALSE);
    gtk_widget_set_sensitive (histoui->sset_sel ,TRUE);
  }
}

/**
 *  Replace create_histo_frame  compwin.c   448
 */
void gg_create_histo_popup (void)
{
  GtkWidget *vb ,*rc2;

  if (histoui == NULL) {
    histoui = xmalloc (sizeof (gg_Histo_ui));
    histoui->dialog  = gg_CreateAACDialog ("Histograms" ,gg_histo_AAC_CB ,0 ,0 ,&vb);
    histoui->srcdest = gg_CreateSrcDestSelector (vb ,GTK_SELECTION_MULTIPLE);

    rc2 = gg_CreateHContainer (vb);
    histoui->cumulative = gg_check_new (rc2 ,"Cumulative histogram");
    histoui->normalize  = gg_check_new (rc2, "Normalize");
    histoui->sampling   = gg_combo_new (vb ,"Bin sampling:" ,3
                                               ,"Linear mesh"                /* SAMPLING_MESH */
                                               ,"Abscissas of another set"); /* SAMPLING_SET */
    histoui->mrc = gg_CreateHContainer (vb);
    histoui->mstart  = gg_entry_new (histoui->mrc ,"Start at:" ,10);
    histoui->mstop   = gg_entry_new (histoui->mrc ,"Stop at:" ,10);
    histoui->mlength = gg_entry_new (histoui->mrc ,"# of bins:" ,6);

    histoui->sset_menu3 = NULL;
    histoui->sset_sel = gg_CreateSetChoice ("Sampling set" ,GTK_SELECTION_SINGLE
                                             ,popup_width ,list_height
                                             ,NULL
					    ,&(histoui->sset_menu3)
					    ,NULL);
    gtk_box_pack_start_defaults (GTK_BOX (vb) ,histoui->sset_sel);
    gtk_widget_set_sensitive (histoui->sset_sel ,FALSE);
    g_signal_connect (histoui->sampling ,"changed" ,G_CALLBACK (gg_binsampling_CB) ,NULL);
  }
  gtk_widget_show_all (histoui->dialog);
}

/**
 *  Replace  do_histo_proc    compwin.c   492
 */
static void gg_histo_AAC_CB (GtkWidget *w ,gint reponse)
{
  int error, res;
  int gno1, gno2 ,ns1 ,ns2;
  gint *sl1 ,*sl2;
  int cumulative, normalize, sampling;
  int i, nbins;
  double *bins = NULL;
  int istart = 0;
  int istop = 0;
  int gsampl = 0;
  int setnosampl;

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY   ) {
    error = FALSE;
    cumulative = gg_get_int (histoui->cumulative);
    normalize  = gg_get_int (histoui->normalize);
    sampling   = gg_get_int (histoui->sampling);
    if (sampling == SAMPLING_SET) {
      gsampl = get_cg ();
      res = gg_GetSingleListChoice (GW_LIST (histoui->sset_sel) , &setnosampl);
      if (res == FALSE) {
        errmsg("Please select single sampling set");
	return;
      } else {
        nbins = getsetlength (gsampl, setnosampl) - 1;
        bins  = getcol (gsampl, setnosampl, DATA_X);
      }
    } else {
      double start, stop;
      if (gg_evalexpr  (histoui->mstart, &start)  != RETURN_SUCCESS ||
          gg_evalexpr  (histoui->mstop,  &stop)   != RETURN_SUCCESS ||
          gg_evalexpri (histoui->mlength, &nbins) != RETURN_SUCCESS ) {
        errmsg ("Can't parse mesh settings");
	return;
      } else {
        bins = allocate_mesh (start, stop, nbins + 1);
        if (bins == NULL) {
          errmsg("Can't allocate mesh");
	  return;
        }
	istart = (int)start;
	istop  = (int)stop;
      }
    }
    if (gg_GetSrcDestSettings (histoui->srcdest ,SD_NSET1_EQ_NSET2 | SD_EXCLUSIVE  // ,FALSE ,TRUE ,TRUE
                               ,&gno1 ,&gno2
                               ,&ns1  ,&ns2
                               ,&sl1  ,&sl2) == RETURN_FAILURE) return;
    for (i = 0; i < ns1; i++) {
      int setnosrc, setnodest;
      setnosrc = sl1[i];
      if (ns2 != 0) {
        setnodest = sl2[i];
      } else {
        setnodest = SET_SELECT_NEXT;
      }
      res = do_histo (gno1, setnosrc, gno2, setnodest,
                       bins, nbins, cumulative, normalize);
      if (res != RETURN_SUCCESS) {
        errmsg("Error in do_histo()");
        error = TRUE;
        break;
      }
      if (follow_me_on) {
	if (sampling == SAMPLING_MESH) {
	  snprintf (bufcmd ,255 ,"@ HISTOGRAM ( G%d.s%d ,Mesh (%d ,%d ,%d) , %s ,%s ,  G%d.s%d )\n" 
		   ,gno1 ,setnosrc
		   ,istart, istop, nbins+1 ,on_or_off(cumulative) ,on_or_off(normalize)
		   ,gno2, setnodest
		   );
	} else {
	  snprintf (bufcmd ,255 ,"@ HISTOGRAM ( G%d.s%d , G%d.s%d  , %s ,%s , G%d.s%d )\n" 
		   ,gno1 ,setnosrc 
		   ,gsampl, setnosampl ,on_or_off(cumulative) ,on_or_off(normalize)
		   ,gno2, setnodest
		   );
	}
	gg_cmd_prt (bufcmd);
      }
    }
    xfree (sl1);
    if (ns2 > 0) xfree (sl2);
    if (sampling == SAMPLING_MESH) xfree (bins);

    gg_switch_current_graph (gno1);
    gg_update_all_sets_lists ();
    gg_switch_current_graph (gno2);
    gg_update_all_sets_lists ();
    gg_drawgraph ();
    if (error) return;
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}


/************ Running averages, medians, min, max, std. deviation   ********/

typedef struct _gg_Run_ui {
  GtkWidget *top;
  gg_SrcDestStructure *srcdest;
  GtkWidget *len;
  GtkWidget *type;
  GtkWidget *region;
  GtkWidget *rinvert;
  SetsMenu3Struct *menu3;
} gg_Run_ui;

static gg_Run_ui rui;

/* Same order as defines.h 311 */
static char *runtype_name[] = {"RUNAVG" ,"RUNMED" ,"RUNMIN" ,"RUNMAX" ,"RUNSTD"};

/*********** Running averages ************/


/**
 *  Replace  create_run_frame    compwin.c   798
 */
void gg_create_run_popup (void)
{
  GtkWidget *vb;
  if (rui.top == NULL) {
    rui.top = gg_CreateAACDialog ("Running properties" ,gg_runavg_AAC_CB ,0 ,0 ,&vb);
    rui.menu3 = NULL;
    rui.srcdest = gg_CreateSrcDestSelector (vb ,GTK_SELECTION_MULTIPLE);
    rui.type   = gg_combo_new (vb ,"Type:" ,6
			       ,"Average"  	    /* RUN_AVG */
			       ,"Median"            /* RUN_MED */
			       ,"Minimum"           /* RUN_MIN */
			       ,"Maximum"           /* RUN_MAX */
			       ,"Std. dev.");       /* RUN_STD */

    rui.len    = gg_spin_new (vb ,"Length of sample:" ,2. ,1000. ,1.);
    rui.region = gg_combo_new (vb ,"Restrictions:" ,9
				       ,"None"
				       ,"Region 0"
				       ,"Region 1"
				       ,"Region 2"
				       ,"Region 3"
				       ,"Region 4"
				       ,"Inside graph"
				       ,"Outside graph");

    rui.rinvert = gg_check_new (vb ,"Invert region");
  }
  gtk_widget_show_all (rui.top);
}


/**
 *  Replace  do_runavg_proc    compwin.c   872
 */
static void gg_runavg_AAC_CB (GtkWidget *w ,gint reponse)
{
  int runlen ,runtype ,rno ,invr;
  int gno1 ,gno2 ,ns1 ,ns2 ,i;
  gint *sl1 ,*sl2;

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY ) {
    if (gg_GetSrcDestSettings (rui.srcdest ,SD_NSET1_EQ_NSET2
			       ,&gno1 ,&gno2
			       ,&ns1  ,&ns2
			       ,&sl1  ,&sl2) == RETURN_FAILURE) return;
    if (gg_evalexpri (rui.len ,&runlen ) != RETURN_SUCCESS) return;
    runtype = gg_get_int (rui.type);
    rno     = gg_get_int (rui.region) - 1;
    invr    = gg_get_int (rui.rinvert);
    for (i = 0; i < ns1; i++) {
      do_runavg12 (gno1 ,sl1[i] ,gno2 ,sl2[i] ,runlen ,runtype ,rno ,invr);
      if (follow_me_on) {
	snprintf (bufcmd ,255 ,"@ %s ( G%d.s%d , %d ,  G%d.s%d , %d , %d )\n"
		 ,runtype_name[runtype] ,gno1 ,sl1[i] ,runlen  ,gno2 ,sl2[i] ,rno ,invr);
	gg_cmd_prt (bufcmd);
      }
    }
    set_dirtystate();
    gg_drawgraph ();
    gg_update_all_sets_lists ();
    ge_update_explorer ();
    xfree (sl1);
    xfree (sl2);
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}

/***********  Regressions ************/

typedef struct _gg_Reg_ui {
  GtkWidget *top;
  GtkWidget *sel;
  GtkWidget *degree;
  GtkWidget *zero;
  GtkWidget *resid;
  GtkWidget *region;
  GtkWidget *rinvert;
  GtkWidget *start;
  GtkWidget *stop;
  GtkWidget *step;
  GtkWidget *fload_rc;
  GtkWidget *method;
  SetsMenu3Struct *menu3;
} gg_Reg_ui;

static gg_Reg_ui regui;

/**
 *    set_regr_sensitivity    compwin.c   927
 */

/**
 *  Replace   create_reg_frame   compwin.c   937
 */
void gg_create_reg_popup (void)
{

  GtkWidget *vb ,*rc2 ,*sep;

  if (regui.top == NULL) {
    regui.top = gg_CreateAACDialog ("Regression" ,gg_regui_AAC_CB ,0 ,0 ,&vb);
    regui.menu3 = NULL;
    regui.sel = gg_CreateSetChoice ("Apply to set:" ,GTK_SELECTION_MULTIPLE
                                    ,popup_width ,list_height
                                    ,NULL
				    ,&(regui.menu3)
				    ,NULL);
    gtk_box_pack_start_defaults (GTK_BOX (vb) ,regui.sel);

    regui.degree = gg_combo_new (vb ," " ,16
                                         ,"Linear"
                                         ,"Quadratic"
                                         ,"Cubic"
                                         ,"4th degree"
                                         ,"5th degree"
                                         ,"6th degree"
                                         ,"7th degree"
                                         ,"8th degree"
                                         ,"9th degree"
                                         ,"10th degree"
                                         ,"1-10"
                                         ,"Power y=A*x^B"
                                         ,"Exponential y=A*exp(B*x)"
                                         ,"Logarithmic y=A+B*ln(x)"
                                         ,"Inverse y=1/(A+Bx)");

    regui.resid = gg_combo_new (vb ,"Load: " ,4
                                        ,"Fitted values"
                                        ,"Residuals"
                                        ,"Function");
    rc2 = gg_CreateHContainer (vb);
    regui.region = gg_combo_new (rc2 ,"Restrictions:" ,9
                                         ,"None"
                                         ,"Region 0"
                                         ,"Region 1"
                                         ,"Region 2"
                                         ,"Region 3"
                                         ,"Region 4"
                                         ,"Inside graph"
                                         ,"Outside graph");
    regui.rinvert = gg_check_new (rc2 ,"Invert region");

    sep = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (vb) ,sep ,FALSE ,TRUE ,1);

    regui.fload_rc  = gg_CreateHContainer (vb);
    regui.start = gg_entry_new (regui.fload_rc ,"Start load at:" ,6);
    regui.stop  = gg_entry_new (regui.fload_rc ,"Stop load at:"  ,6);
    regui.step  = gg_entry_new (regui.fload_rc ,"# of points:"   ,4);
    gtk_widget_set_sensitive (regui.fload_rc ,FALSE);
  }
  gtk_widget_show_all (regui.top);
}

/**
 *  Replace  do_regress_proc    compwin.c   1055
 */
static void gg_regui_AAC_CB (GtkWidget *w ,gint reponse)
{
  int gno = get_cg();
  gint *selsets;
  int cnt ,rno ,invr;
  int setno, ideg, iresid, i, j, k;
  int nstep = 0, rx, rset = 0;
  double xstart, xstop, stepsize = 0.0, *xr;

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY ) {
    selsets = gw_list_selected_list (GW_LIST (regui.sel) ,&cnt);
    if (cnt == 0)  {
      errmsg  ("No sets selected");
      return;
    }
    rno  = gg_get_int (regui.region) - 1;
    invr = gg_get_int (regui.rinvert);
    ideg = gg_get_int (regui.degree) + 1;
    rx   = gg_get_int (regui.resid);
    switch (rx) {
    case 0:             /* evaluate fitted function at original x's */
      iresid = 0;
      rset = -1;
      break;
    case 1:               /* load residue at original x points */
      iresid = 1;
      rset = -1;
      break;
    case 2:               /* evaluate fitted function at new x points */
      iresid = 0;
      if(gg_evalexpri (regui.step ,&nstep) != RETURN_SUCCESS || nstep < 2 ) {
	errmsg  ("Number points < 2");
	return;
      }
      if (gg_evalexpr (regui.start ,&xstart ) != RETURN_SUCCESS) {
	errmsg ("Specify starting value");
	return;
      }
      if (gg_evalexpr (regui.stop ,&xstop) != RETURN_SUCCESS) {
	errmsg  ("Specify stopping value");
	return;
      } else {
	stepsize = (xstop - xstart)/(nstep-1);
      }
      break;
    default:
      errmsg  ("Internal error");
      return;
    }
    for (i = (ideg==11?1:ideg); i <= (ideg==11?10:ideg); i++) {
      for (j = 0; j < cnt; j++) {
	setno = selsets[j];
	if( rx == 2 ) {
	  if( (rset = nextset( gno )) == -1 ){
	    errmsg  ("Not enough sets");
	    return;
	  }
	  activateset (gno ,rset);
	  setlength (gno ,rset ,nstep);
	  xr = getx( gno ,rset );
	  for( k=0; k<nstep; k++ )
	    xr[k] = xstart + k*stepsize;
	}
	do_regress (gno ,setno ,i ,iresid ,rno ,invr ,rset);
      }
    }
    gg_update_all_sets_lists ();
    xfree (selsets);
    gg_drawgraph ();
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}


/****** Finite differencing/derivating *******/


typedef struct _gg_Diff_ui {
  GtkWidget *top;
  gg_SrcDestStructure *srcdest;
  GtkWidget *type;
  GtkWidget *xplace;
  GtkWidget *period;
  GtkWidget *region;
  GtkWidget *rinvert;
} gg_Diff_ui;

static gg_Diff_ui dui;

static void gg_diff_AAC_CB (GtkWidget *w ,gint reponse);


/**
 *  Replace create_diff_frame  compwin.c   1139
 */
void gg_diff_create_win (void)
{
  GtkWidget *vb;

  if (dui.top == NULL) {
    dui.top     = gg_CreateAACDialog ("Differences/derivatives" ,gg_diff_AAC_CB ,0 ,0 ,&vb);
    dui.srcdest = gg_CreateSrcDestSelector (vb ,GTK_SELECTION_MULTIPLE);
    dui.type    = gg_combo_new (vb ,"Type:" ,3
					,"Plain differences"  /* DIFF_TYPE_PLAIN      */
					,"Derivative");       /* DIFF_TYPE_DERIVATIVE */
    dui.xplace  = gg_combo_new (vb ,"X placement:" ,4
					,"Left"               /* DIFF_XPLACE_LEFT   */
					,"Center"             /* DIFF_XPLACE_CENTER */
					,"Right");            /* DIFF_XPLACE_RIGHT  */
    dui.period  = gg_spin_new  (vb ,"Stride (period):" ,1.0 ,999999.0 ,1.0);
    /* set defaults */
    gg_diff_update ();
  }
  gtk_widget_show_all (dui.top);
}

void gg_diff_update (void)
{
  if (dui.top != NULL) {
    DIFFParams p;
    if (diff_params_get (&p) == RETURN_SUCCESS) {
      gg_set_int (dui.type   ,p.derivative);
      gg_set_int (dui.xplace ,p.xplace);
      gg_set_int (dui.period ,p.period);
    }
  }
}

/**
 *  Replace do_differ_proc  compwin.c   1180
 */
static void gg_diff_AAC_CB (GtkWidget *w ,gint reponse)
{
  DIFFParams p;
  int gno1 ,gno2 ,ns1 ,ns2 ,i;
  gint *sl1 ,*sl2;

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY ) {
    if (gg_GetSrcDestSettings (dui.srcdest ,SD_NSET1_EQ_NSET2
			       ,&gno1 ,&gno2
			       ,&ns1  ,&ns2
			       ,&sl1  ,&sl2) == RETURN_FAILURE) return;
    p.derivative = gg_get_int (dui.type);
    p.xplace     = gg_get_int (dui.xplace);
    p.period     = gg_get_int (dui.period);
    diff_params_set (&p);

    for (i = 0; i < ns1; i++) {
      do_diff (gno1 ,sl1[i] ,gno2 ,sl2[i]);
      if (follow_me_on) {
	snprintf (bufcmd ,255 ,"@ DIFFERENCE ( G%d.s%d ,  G%d.s%d , %d , %d , %d )\n"
		 ,gno1 ,sl1[i] ,gno2 ,sl2[i] ,p.xplace ,p.derivative ,p.period);
	gg_cmd_prt (bufcmd);
      }
    }
    gg_drawgraph ();
    gg_update_all_sets_lists ();

    ge_update_explorer ();
    xfree (sl1);
    xfree (sl2);
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}


/******* Numerical integration **********/

typedef struct _gg_Int_ui {
    GtkWidget *top;
  gg_SrcDestStructure *srcdest;
    GtkWidget *type;
    GtkWidget *region;
    GtkWidget *rinvert;
  SetsMenu3Struct *menu3;
} gg_Int_ui;

static gg_Int_ui iui;

/**
 *  Replace create_int_frame  compwin.c   1217
 */
void gg_create_int_popup (void)
{
  GtkWidget *vb;

  if (iui.top == NULL) {
    iui.top = gg_CreateAACDialog ("Integration"  ,gg_iui_AAC_CB ,0 ,0 ,&vb);
    iui.menu3 = NULL;
    iui.srcdest = gg_CreateSrcDestSelector (vb ,GTK_SELECTION_MULTIPLE);
    iui.type = gg_combo_new (vb  ,"Load:" ,3
				     ,"Cumulative sum"
				     ,"Sum only");
  }
  gtk_widget_show_all (iui.top);
}

/**
 *  Replace do_int_proc  compwin.c   1258
 */
static void gg_iui_AAC_CB (GtkWidget *w ,gint reponse)
{
  int gno1 ,gno2 ,ns1 ,ns2 ,i ,itype;
  gint *sl1 ,*sl2;
  double sum;
  char buf[64];
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY ) {
    itype = gg_get_int (iui.type);
    if (itype == 0) {
      /* Plot cumulative sums */
      if (gg_GetSrcDestSettings (iui.srcdest ,SD_NSET1_EQ_NSET2
				 ,&gno1 ,&gno2
				 ,&ns1  ,&ns2
				 ,&sl1  ,&sl2) == RETURN_FAILURE) return;
      for (i = 0; i < ns1; i++) {
	sum = do_integrate (gno1 ,sl1[i] ,gno2 ,sl2[i]);
	if (follow_me_on) {
	  snprintf (bufcmd ,255 ,"@ INTEGRATE ( G%d.s%d , G%d.s%d )\n" ,gno1 ,sl1[i]  ,gno2 ,sl2[i]);
	  gg_cmd_prt (bufcmd);
	}
	snprintf (buf ,63 ,"Sum of G%d.s%d = %g\n" ,gno1 ,sl1[i] ,sum);
	stufftext (buf);
      }
      gg_drawgraph ();
      gg_update_all_sets_lists ();
      ge_update_explorer ();
    } else {
      /* Only compute sums, don't use dest selectors */
      if (!gg_GetSingleListChoice (GW_LIST (iui.srcdest->src->graphs) ,&gno1)) {
	errmsg("Please select single source graph");    
	return;
      }
      sl1 = gw_list_selected_list (GW_LIST (iui.srcdest->src->sets) ,&ns1);
      sl2 = gw_list_selected_list (GW_LIST (iui.srcdest->dest->sets) ,&ns2);
      for (i = 0; i < ns1; i++) {
	sum = trapint (getx(gno1,sl1[i]) ,gety(gno1,sl1[i]), NULL, NULL, getsetlength(gno1,sl1[i]));
	snprintf (buf ,63 ,"Sum of G%d.s%d = %g\n" ,gno1 ,sl1[i] ,sum);
	stufftext (buf);
      }
    }
    xfree (sl1);
    xfree (sl2);
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}


/******** Seasonal differencing ************/

typedef struct _gg_Seas_ui {
  GtkWidget *top;
  GtkWidget *sel;
  GtkWidget *type;
  GtkWidget *period;
  GtkWidget *region;
  GtkWidget *rinvert;
  SetsMenu3Struct *menu3;
} gg_Seas_ui;

static gg_Seas_ui sui;

/**
 *  Replace create_seasonal_frame  compwin.c   1300
 */
void gg_create_seasonal_popup (void)
{
  GtkWidget *vb;

  if (sui.top == NULL) {
    sui.top = gg_CreateAACDialog ("Seasonal differences"  ,gg_sui_AAC_CB ,0 ,0 ,&vb);
    sui.menu3 = NULL;
    sui.sel = gg_CreateSetChoice ("Apply to set:" ,GTK_SELECTION_MULTIPLE
				  ,popup_width ,list_height
				  ,NULL
				  ,&(sui.menu3)
				  ,NULL);
    gtk_box_pack_start_defaults (GTK_BOX (vb) ,sui.sel);
    sui.period = gg_entry_new (vb ,"Period:" ,10);
  }
  gtk_widget_show_all (sui.top);
}

/**
 *  Replace do_seasonal_proc  compwin.c   1335
 */
static void gg_sui_AAC_CB (GtkWidget *w ,gint reponse)
{
  int *selsets;
  int i ,cnt;
  int setno, period;

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY ) {
    selsets = gw_list_selected_list (GW_LIST (sui.sel) ,&cnt);
    if (cnt == 0)  {
      errmsg  ("No sets selected");
      return;
    }
    if (gg_evalexpri (sui.period, &period ) != RETURN_SUCCESS)
      return;
    for (i = 0; i < cnt; i++) {
      setno = selsets[i];
      do_seasonal_diff (setno, period);
    }
    gg_update_all_sets_lists ();
    xfree (selsets);
    gg_drawgraph ();
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}

/********** Cross correlation ************/

typedef struct _gg_Cross_ui {
  GtkWidget *top;
  gg_SrcDestStructure *srcdest;
  GtkWidget *autocor;
  GtkWidget *lag;
  GtkWidget *covar;
  gg_GraphSetStructure *corset;
  GtkWidget *boxcor;
} gg_Cross_ui;

static gg_Cross_ui crossui;

static void gg_autocor_CB (GtkToggleButton *w ,gpointer p)
{
  int autocor = gg_get_int (crossui.autocor);
  gtk_widget_set_sensitive (crossui.boxcor ,!autocor);
}

/**
 *  Replace create_xcor_frame  compwin.c   1373
 */
void gg_create_xcor_popup (void)
{
  GtkWidget *vb ,*fr;
  int width ,height;
  if (crossui.top == NULL) {
    crossui.top     = gg_CreateAACDialog ("Correlation/Covariance"  ,gg_crossui_AAC_CB ,0 ,0 ,&vb);
    crossui.srcdest = gg_CreateSrcDestSelector 	(vb ,GTK_SELECTION_MULTIPLE);
    fr = gg_frame (vb ," " ,&wheat ,2 ,2);
    crossui.lag     = gg_spin_new   	       	(fr ,"Maximum lag:" ,1 ,100 ,1);
    gg_label_new (fr ,NULL);
    crossui.autocor = gg_check_new   	       	(fr ,"Auto-correlation");
    crossui.covar   = gg_check_new   	       	(fr ,"Calculate covariance");
    crossui.corset  = gg_CreateGraphSetSelector (vb ,"Correlate with" ,GTK_SELECTION_BROWSE ,&wheat);
    crossui.boxcor  = gg_get_w_frame_box ();
    gtk_widget_get_size_request (crossui.corset->sets ,&width ,&height);
    gtk_widget_set_size_request (crossui.corset->sets ,width ,150);
    g_signal_connect (crossui.autocor ,"toggled"      ,G_CALLBACK (gg_autocor_CB) ,NULL);
  }
  gtk_widget_show_all (crossui.top);
}


/**
 *  Replace do_xcor_proc  compwin.c
 */
static void gg_crossui_AAC_CB (GtkWidget *w ,gint reponse)
{
  int i;
  int gno1, gno2 ,gno_dest ,ns1 ,ns_dest;
  gint *sl1 ,set2 ,*sl_dest;
  int maxlag = gg_get_int (crossui.lag);
  int covar  = gg_get_int (crossui.covar);
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY ) {
    if (gg_GetSrcDestSettings (crossui.srcdest ,SD_NSET1_EQ_NSET2 | SD_EXCLUSIVE
                               ,&gno1 ,&gno_dest
                               ,&ns1  ,&ns_dest
                               ,&sl1  ,&sl_dest) == RETURN_FAILURE) return;
    gint autocor = gg_get_int (crossui.autocor);
    if (autocor) {
      gno2 = gno1;
    } else {
      if (!gg_GetSingleListChoice (GW_LIST(crossui.corset->graphs) ,&gno2)) {
	errmsg("Please select a single graph to define a set to correlate with");
	return;
      }
    }
    for (i = 0; i < ns1; i++) {
      if (autocor == FALSE) {
	if (!gg_GetSingleListChoice (GW_LIST(crossui.corset->sets) ,&set2)) {
	  errmsg("Please select a single  set to correlate with");
	  return;
	}
      } else {
	set2 = sl1[i];
      }
      do_xcor (gno1 ,sl1[i] ,gno2 ,set2 ,maxlag ,covar ,gno_dest ,sl_dest[i]);
      if (follow_me_on) {
	snprintf (bufcmd ,255 ,"@ XCOR ( G%d.s%d , G%d.s%d , %d , %s , G%d.s%d)\n" 
		  ,gno1 ,sl1[i]  ,gno2 ,set2 ,maxlag ,on_or_off(covar) ,gno_dest ,sl_dest[i]);
	gg_cmd_prt (bufcmd);
      }
    }
    gg_update_all_sets_lists ();
    ge_update_explorer ();
    gg_drawgraph ();
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}


/********** Sample a set *************/

/* sample a set */

typedef struct _gg_Samp_ui {
  GtkWidget *top;
  gg_SrcDestStructure *srcdest;
  GtkWidget *type;
  GtkWidget *start;
  GtkWidget *step;
  GtkWidget *expr;
  GtkWidget *region;
  GtkWidget *rinvert;
  SetsMenu3Struct *menu3;
} gg_Samp_ui;

static gg_Samp_ui sampui;

static void gg_samp_method_CB (GtkWidget *w ,gpointer p)
{
  gint value = gg_get_int (sampui.type);
  gtk_widget_set_sensitive (sampui.start ,value == 0);
  gtk_widget_set_sensitive (sampui.step  ,value == 0);
  gtk_widget_set_sensitive (sampui.expr  ,value == 1);
}

/**
 *  Replace create_samp_frame  compwin.c   1452
 */
void gg_create_samp_popup (void)
{
  GtkWidget *vb;

  if (sampui.top == NULL) {
    sampui.top 	   = gg_CreateAACDialog ("Sample points"  ,gg_samp_AAC_CB ,0 ,0 ,&vb);
    sampui.srcdest = gg_CreateSrcDestSelector 	(vb ,GTK_SELECTION_MULTIPLE);
    sampui.type    = gg_combo_new (vb  ,"Sample type:" ,6
				   ,"Start/step"         /* 0 */
				   ,"Expression"         /* 1 */
				   ,"Extremas"        /* 2 */
				   ,"Minimas"         /* 3 */
				   ,"Maximas");       /* 4 */
    sampui.start = gg_entry_new (vb ,"Start:"              ,10);
    sampui.step  = gg_entry_new (vb ,"Step:"               ,10);
    sampui.expr  = gg_entry_new (vb ,"Logical expression:" ,30);
    gtk_widget_set_tooltip_text (sampui.start ,"Index of the first point of sample (starting at 1)");
    gtk_widget_set_tooltip_text (sampui.step  ,"Step of index between two points of the sample");
    gtk_widget_set_tooltip_text (sampui.expr  ,"A logical expression to select points");
    g_signal_connect (sampui.type ,"changed" ,G_CALLBACK (gg_samp_method_CB) ,NULL);
    gg_set_int (sampui.type ,0);
    gg_samp_method_CB (sampui.type ,NULL);
  }
  gtk_widget_show_all (sampui.top);
}

const char *minmaxp[] = {"MINMAX" ,"MIN" ,"MAX"};
static char bufcmd[256];

/**
 *  Replace do_sample_proc  compwin.c   1508
 */
static void gg_samp_AAC_CB (GtkWidget *w ,gint reponse)
{
  int gno1, gno2 ,ns1 ,ns2 ,i;
  gint *sl1 ,*sl2;
  int typeno ,startno ,stepno;
  const char *exprstr = "";
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY ) {
    if (gg_GetSrcDestSettings (sampui.srcdest ,SD_NSET1_EQ_NSET2 | SD_EXCLUSIVE
                               ,&gno1 ,&gno2
                               ,&ns1  ,&ns2
                               ,&sl1  ,&sl2) == RETURN_FAILURE) return;
    typeno = gg_get_int (sampui.type);
    
    if (typeno == 0) {		       /**** start/step method ****/
      if (gg_evalexpri (sampui.start ,&startno) != RETURN_SUCCESS ||
 	  gg_evalexpri (sampui.step  ,&stepno)  != RETURN_SUCCESS) {
 	errmsg ("Sampling start or step value invalid");
 	return;
      }
    } else {
      startno = stepno = 1;
      if (typeno == 1) { 	      /**** Logical expression method ****/
	exprstr = gg_getstr (sampui.expr); 
      }
    }
    for (i = 0; i < ns1; i++) {
      do_sample (gno1 ,sl1[i] ,gno2 ,sl2[i] ,typeno ,exprstr ,startno ,stepno);
      if (follow_me_on) {
	switch (typeno) {
	case 0:         		       /**** start/step method ****/
	  snprintf (bufcmd ,255 ,"@ SAMPLE (G%d.s%d , G%d.s%d ,%d ,%d)\n",gno1 ,sl1[i] ,gno2 ,sl2[i] ,startno ,stepno);
	  break;
	case 1:  			       /**** Logical expression method ****/
	  snprintf (bufcmd ,255 ,"@ SAMPLE (G%d.s%d , G%d.s%d ,\"%s\")\n",gno1 ,sl1[i] ,gno2 ,sl2[i] ,exprstr);
	  break;
	case 2:  			       /**** extremas ****/
	case 3:  			       /**** minimas ****/   
	case 4:  			       /**** maximas ****/   
	  snprintf (bufcmd ,255 ,"@ SAMPLE (G%d.s%d , G%d.s%d ,%s)\n",gno1 ,sl1[i] ,gno2 ,sl2[i] ,minmaxp[typeno-2]);
	  break;
	}
	gg_cmd_prt (bufcmd);
      }
    }
    set_dirtystate();
    gg_drawgraph ();
    gg_update_all_sets_lists ();
    ge_update_explorer ();
    xfree (sl1);
    xfree (sl2);
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}

/************* Prune data *************/

typedef struct _gg_Prune_ui {
  GtkWidget *top;
  gg_SrcDestStructure *srcdest;
  GtkWidget *type;
  GtkWidget *dxtype;
  GtkWidget *dytype;
  GtkWidget *deltatype;
  GtkWidget *dx;
  GtkWidget *dy;
} gg_Prune_ui;

static gg_Prune_ui pruneui;

/**
 *  Replace update_prune_frame  compwin.c   1641
 */
void gg_update_prune_popup (void)
{
  if (pruneui.top != NULL) {
    gg_set_int (pruneui.dxtype ,(get_graph_xscale(get_cg()) == SCALE_LOG) ? 1 : 0);
    gg_set_int (pruneui.dytype ,(get_graph_yscale(get_cg()) == SCALE_LOG) ? 1 : 0);
  }
}


/**
 *  Replace create_prune_frame  compwin.c   1569
 */
void gg_create_prune_popup (void)
{
  GtkWidget *vb ,*sep;;

  if (pruneui.top == NULL) {
    pruneui.top = gg_CreateAACDialog ("Prune data"  ,gg_prune_AAC_CB ,0 ,0 ,&vb);
    gg_label_new (vb ,"Reducing huge sets using a distance");
    pruneui.srcdest = gg_CreateSrcDestSelector (vb ,GTK_SELECTION_MULTIPLE);
    pruneui.type = gg_combo_new (vb  ,"Prune type: ", 5
					 ,"Interpolation"
					 ,"Circle"
					 ,"Ellipse"
					 ,"Rectangle");
    pruneui.dx    = gg_entry_new  (vb ,"Delta X:" ,17);
    pruneui.dy    = gg_entry_new  (vb ,"Delta Y:" ,17);

    sep = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (vb) ,sep ,FALSE ,TRUE ,1);

    pruneui.deltatype = gg_combo_new (vb ,"Type of Delta coordinates:" ,3
				      ,"Viewport"       	/* COORD_VIEW=0 */
				      ,"World");		/* COORD_WORLD=1 */
    pruneui.dxtype    = gg_combo_new (vb ,"Scaling of Delta X:" ,3
				      ,"Linear"       		/* SCALE_NORMAL=0 */ 
				      ,"Logarithmic");          /* SCALE_LOG=1    */
    pruneui.dytype    = gg_combo_new (vb ,"Scaling of Delta Y:" ,3
					      ,"Linear"
					      ,"Logarithmic");
    gg_update_prune_popup ();
    gg_prune_toggle ();

    g_signal_connect (pruneui.type 	,"changed" ,G_CALLBACK (gg_update_prune_popup) ,NULL);
    g_signal_connect (pruneui.type 	,"changed" ,G_CALLBACK (gg_prune_toggle)       ,NULL);
    g_signal_connect (pruneui.deltatype ,"changed" ,G_CALLBACK (gg_prune_toggle) ,NULL);
  }
  gtk_widget_show_all (pruneui.top);
}

static void gg_prune_toggle (void)
{
  int typeno ,deltatypeno;

  typeno      = gg_get_int (pruneui.type);
  deltatypeno = gg_get_int (pruneui.deltatype);

  switch (typeno) {
  case PRUNE_CIRCLE:
    gtk_widget_set_sensitive (pruneui.dx ,TRUE);
    gtk_widget_set_sensitive (pruneui.dy ,FALSE);
    switch (deltatypeno) {
    case COORD_VIEW:
      gtk_widget_set_sensitive (pruneui.dxtype ,FALSE);
      gtk_widget_set_sensitive (pruneui.dytype ,FALSE);
      break;
    case COORD_WORLD:
      gtk_widget_set_sensitive (pruneui.dxtype ,TRUE);
      gtk_widget_set_sensitive (pruneui.dytype ,FALSE);
      break;
    }
    break;
  case PRUNE_ELLIPSE:
  case PRUNE_RECTANGLE:
    gtk_widget_set_sensitive (pruneui.dx ,TRUE);
    gtk_widget_set_sensitive (pruneui.dy ,TRUE);
    switch (deltatypeno) {
    case COORD_VIEW:
      gtk_widget_set_sensitive (pruneui.dxtype ,FALSE);
      gtk_widget_set_sensitive (pruneui.dytype ,FALSE);
      break;
    case COORD_WORLD:
      gtk_widget_set_sensitive (pruneui.dxtype ,TRUE);
      gtk_widget_set_sensitive (pruneui.dytype ,TRUE);
      break;
    }
    break;
  case PRUNE_INTERPOLATION:
    gtk_widget_set_sensitive (pruneui.dx ,FALSE);
    gtk_widget_set_sensitive (pruneui.dy ,TRUE);
    switch (deltatypeno) {
    case COORD_VIEW:
      gtk_widget_set_sensitive (pruneui.dxtype ,FALSE);
      gtk_widget_set_sensitive (pruneui.dytype ,FALSE);
      break;
    case COORD_WORLD:
      gtk_widget_set_sensitive (pruneui.dxtype ,FALSE);
      gtk_widget_set_sensitive (pruneui.dytype ,TRUE);
      break;
    }
    break;
  }
}


/**
 *  Replace  do_prune_proc    compwin.c   1710
 */
static void gg_prune_AAC_CB (GtkWidget *w ,gint reponse)
{
  int gno1 ,gno2 ,ns1 ,ns2 ,i;
  gint *sl1 ,*sl2;
  int typeno ,deltatypeno;
  int dxtype ,dytype;
  double deltax ,deltay;
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY ) {
    if (gg_GetSrcDestSettings (pruneui.srcdest ,SD_NSET1_EQ_NSET2
			       ,&gno1 ,&gno2
			       ,&ns1  ,&ns2
			       ,&sl1  ,&sl2) == RETURN_FAILURE) return;
    typeno      = gg_get_int (pruneui.type);
    deltatypeno = gg_get_int (pruneui.deltatype);
    dxtype      = gg_get_int (pruneui.dxtype);
    dytype      = gg_get_int (pruneui.dytype);
    if (GTK_WIDGET_SENSITIVE (pruneui.dx)) {
      if (gg_evalexpr (pruneui.dx ,&deltax) != RETURN_SUCCESS) {
	errmsg ("Prune: Delta X incorrect");
	return;
      }
    } else{
      deltax = 0.0;
    }
    if (GTK_WIDGET_SENSITIVE (pruneui.dy)) {
      if (gg_evalexpr (pruneui.dy ,&deltay) != RETURN_SUCCESS) {
	errmsg ("Prune: Delta Y incorrect");
	return;
      }
    } else{
      deltay = 0.0;
    }
    for (i = 0; i < ns1; i++) {
      if (do_prune (gno1 ,sl1[i] ,gno2 ,sl2[i]
		    ,typeno ,deltatypeno ,deltax ,deltay ,dxtype ,dytype) == RETURN_FAILURE)
	break;
      if (follow_me_on) {
	snprintf (bufcmd ,255 ,"@ DECIMATE ( G%d.s%d , G%d.s%d , %d ,%s , %g , %g , %d , %d )\n" 
		  ,gno1 ,sl1[i]  ,gno2 ,sl2[i] 
		  ,typeno 
		  ,(deltatypeno == COORD_WORLD) ? "WORLD" : "VIEW"
		  ,deltax ,deltay ,dxtype ,dytype);
	gg_cmd_prt (bufcmd);
      }
    }
    set_dirtystate();
    gg_drawgraph ();
    gg_update_all_sets_lists ();
    ge_update_explorer ();
    xfree (sl1);
    xfree (sl2);
  }

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}

/***** Apply a digital filter in set 2 to set 1 *****/

typedef struct _gg_Digf_ui {
  GtkWidget *top;
  GtkWidget *sel1 ,*sel2;
  GtkWidget *type;
  GtkWidget *region;
  GtkWidget *rinvert;
  SetsMenu3Struct *sel1_menu3;
  SetsMenu3Struct *sel2_menu3;
} gg_Digf_ui;

static gg_Digf_ui digfui;

/**
 *  Replace create_digf_frame  compwin.c  1765
 */
void gg_create_digf_popup (void)
{
  GtkWidget *vb;

  if (digfui.top == NULL) {
    digfui.top  = gg_CreateAACDialog ("Digital filter"  ,gg_digfilter_AAC_CB ,0 ,0 ,&vb);
    digfui.sel1_menu3 = NULL;
    digfui.sel1 = gg_CreateSetChoice ("Apply to set:" ,GTK_SELECTION_BROWSE
				       ,popup_width ,list_height
				      ,NULL
				      ,&(digfui.sel1_menu3)
				      ,NULL);
    gtk_box_pack_start_defaults (GTK_BOX (vb) ,digfui.sel1);

    digfui.sel2_menu3 = NULL;
    digfui.sel2 = gg_CreateSetChoice ("Apply to set:" ,GTK_SELECTION_BROWSE
				       ,popup_width ,list_height
				      ,NULL
				      ,&(digfui.sel2_menu3)
				      ,NULL);
    gtk_box_pack_start_defaults (GTK_BOX (vb) ,digfui.sel2);
  }
  gtk_widget_show_all (digfui.top);
}

/**
 *  Replace  do_digfilter_proc    compwin.c   1804
 */
static void gg_digfilter_AAC_CB (GtkWidget *w ,gint reponse)
{
  int set1, set2;

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY ) {
    set1 = gw_list_get_first_selected_row_num (GW_LIST(digfui.sel1));
    set2 = gw_list_get_first_selected_row_num (GW_LIST(digfui.sel2));
    do_digfilter (set1 ,set2);
    gg_update_all_sets_lists ();
    gg_drawgraph ();
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}


/************ Linear convolution **************/


typedef struct _gg_Lconv_ui {
  GtkWidget *top;
  gg_SrcDestStructure *srcdest;
  gg_GraphSetStructure *convset;
} gg_Lconv_ui;

static gg_Lconv_ui lconvui;

/**
 *  Replace create_lconv_frame  compwin.c  1834
 */
void gg_create_lconv_popup (void)
{
  GtkWidget *vb;
  int width ,height;
  if (lconvui.top == NULL) {
    lconvui.top     = gg_CreateAACDialog ("Linear convolution"  ,gg_linearc_AAC_CB ,0 ,0 ,&vb);
    lconvui.srcdest = gg_CreateSrcDestSelector  (vb ,GTK_SELECTION_MULTIPLE);
    lconvui.convset = gg_CreateGraphSetSelector (vb ,"Convolve with" ,GTK_SELECTION_BROWSE ,&wheat);
    gtk_widget_get_size_request (lconvui.convset->sets ,&width ,&height);
    gtk_widget_set_size_request (lconvui.convset->sets ,width ,150);
  }
  gtk_widget_show_all (lconvui.top);
}

/**
 *  Replace  do_linearc_proc    compwin.c   1873
 */
static void gg_linearc_AAC_CB (GtkWidget *w ,gint reponse)
{
  int i;
  int gno1, gno2 ,gno_dest ,ns1 ,ns_dest;
  gint *sl1 ,set2 ,*sl_dest;
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY ) {
    if (gg_GetSrcDestSettings (lconvui.srcdest ,SD_NSET1_EQ_NSET2 | SD_EXCLUSIVE
                               ,&gno1 ,&gno_dest
                               ,&ns1  ,&ns_dest
                               ,&sl1  ,&sl_dest) == RETURN_FAILURE) return;
    if (!gg_GetSingleListChoice (GW_LIST(lconvui.convset->graphs) ,&gno2)) {
      errmsg("Please select a single graph to define a set to convolve with");
      return;
    }
    for (i = 0; i < ns1; i++) {
      if (!gg_GetSingleListChoice (GW_LIST(lconvui.convset->sets) ,&set2)) {
	errmsg("Please select a single  set to convolve with");
	return;
      }
      do_linearc (gno1 ,sl1[i] ,gno2 ,set2 ,gno_dest ,sl_dest[i]);
      if (follow_me_on) {
	snprintf (bufcmd ,255 ,"@ LINCONV ( G%d.s%d , G%d.s%d , G%d.s%d)\n" 
		  ,gno1 ,sl1[i]  ,gno2 ,set2 ,gno_dest ,sl_dest[i]);
	gg_cmd_prt (bufcmd);
      }
      gg_update_all_sets_lists ();
      ge_update_explorer ();
      gg_drawgraph ();
    }
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}


/************** Rotate, scale, translate ************/

#define GEOM_RESET  1001

typedef struct _gg_Geom_ui {
  GtkWidget *top;
  gg_SrcDestStructure *srcdest;
  GtkWidget *order;
  GtkWidget *degrees;
  GtkWidget *rotx;
  GtkWidget *roty;
  GtkWidget *scalex;
  GtkWidget *scaley;
  GtkWidget *transx;
  GtkWidget *transy;
  GtkWidget *region;   /* not yet used */
  GtkWidget *rinvert;  /* not yet used */
} gg_Geom_ui;

static gg_Geom_ui gui;
static GeomTransform geomtr;
/* The order must conform to gui.order combo */
static char *sorder[] = {"RTS" ,"RST" ,"TSR" ,"TRS" ,"STR" ,"SRT" };

/**
 *  Replace create_geom_frame  compwin.c   1916
 */
void gg_create_geom_popup (void)
{
  GtkWidget *vb ,*fr;

  if (gui.top == NULL) {
    gui.top = gg_CreateAACDialog ("Geometric transformations"  ,gg_geom_AAC_CB ,0 ,0 ,&vb);
    gtk_dialog_add_buttons (GTK_DIALOG(gui.top) ,"Reset" ,GEOM_RESET ,NULL);
    rui.srcdest = gg_CreateSrcDestSelector (vb ,GTK_SELECTION_MULTIPLE);
    gui.order = gg_combo_new (vb ,"Apply in order:" ,7
				      ,"Rotate, translate, scale"
				      ,"Rotate, scale, translate"
				      ,"Translate, scale, rotate"
				      ,"Translate, rotate, scale"
				      ,"Scale, translate, rotate"
				      ,"Scale, rotate, translate");
    fr = gg_frame (vb ," " ,&wheat ,3 ,3);
    gui.degrees = gg_entry_new (fr ,"Rotation (degrees):" ,10);
    gg_label_new (fr ,"Translate");
    gg_label_new (fr ,"Scale");
    gui.rotx    = gg_entry_new (fr ,"Rotate about X = :"  ,10);
    gui.transx  = gg_entry_new (fr ," X:"        ,10);
    gui.scalex  = gg_entry_new (fr ," X:"            ,10);
    gui.roty    = gg_entry_new (fr ,"Rotate about Y = :"  ,10);
    gui.transy  = gg_entry_new (fr ," Y:"        ,10);
    gui.scaley  = gg_entry_new (fr ," Y:"            ,10);

    gg_setstr (gui.degrees ,"0.0");
    gg_setstr (gui.rotx    ,"0.0");
    gg_setstr (gui.roty    ,"0.0");
    gg_setstr (gui.scalex  ,"1.0");
    gg_setstr (gui.scaley  ,"1.0");
    gg_setstr (gui.transx  ,"0.0");
    gg_setstr (gui.transy  ,"0.0");
  }
  gtk_widget_show_all (gui.top);
}

/**
 *  Replace  do_geom_proc    compwin.c   1990
 */
static void gg_geom_AAC_CB (GtkWidget *w ,gint reponse)
{
  int ord;
  int gno1 ,gno2 ,ns1 ,ns2 ,i;
  gint *sl1 ,*sl2;
  if (reponse == GEOM_RESET) {
    gg_setstr (gui.degrees ,"0.0");
    gg_setstr (gui.rotx    ,"0.0");
    gg_setstr (gui.roty    ,"0.0");
    gg_setstr (gui.scalex  ,"1.0");
    gg_setstr (gui.scaley  ,"1.0");
    gg_setstr (gui.transx  ,"0.0");
    gg_setstr (gui.transy  ,"0.0");
  } else if (reponse == GTK_RESPONSE_ACCEPT ||  reponse == GTK_RESPONSE_APPLY) {
    if (gg_GetSrcDestSettings (rui.srcdest ,SD_NSET1_EQ_NSET2
			       ,&gno1 ,&gno2
			       ,&ns1  ,&ns2
			       ,&sl1  ,&sl2) == RETURN_FAILURE) return;
    ord = gg_get_int (gui.order);
    strcpy (geomtr.sorder ,sorder[ord]);
    /* check input fields */
    if (gg_evalexpr (gui.degrees ,&geomtr.degrees) != RETURN_SUCCESS ||
        gg_evalexpr (gui.rotx 	 ,&geomtr.rotx)    != RETURN_SUCCESS ||
        gg_evalexpr (gui.roty 	 ,&geomtr.roty)    != RETURN_SUCCESS ||
        gg_evalexpr (gui.transx  ,&geomtr.transx)  != RETURN_SUCCESS ||
        gg_evalexpr (gui.transy  ,&geomtr.transy)  != RETURN_SUCCESS ||
        gg_evalexpr (gui.scalex  ,&geomtr.scalex)  != RETURN_SUCCESS ||
        gg_evalexpr (gui.scaley  ,&geomtr.scaley)  != RETURN_SUCCESS )
      return;
    for (i = 0; i < ns1; i++) {
      do_geom_transfo (gno1 ,sl1[i] ,gno2 ,sl2[i] ,&geomtr);
      if (follow_me_on) {
	/*                                      */
	snprintf (bufcmd ,255 ,"@ GEOTRANSFORM ( G%d.s%d , G%d.s%d , \"%s\" ,%g , %g , %g , %g ,%g , %g ,%g)\n" 
		  ,gno1 ,sl1[0]  ,gno2 ,sl2[i]
		  ,geomtr.sorder
		  ,geomtr.degrees
		  ,geomtr.rotx   
		  ,geomtr.roty   
		  ,geomtr.transx 
		  ,geomtr.transy 
		  ,geomtr.scalex 
		  ,geomtr.scaley
		  );
	gg_cmd_prt (bufcmd);
      }
    }
    set_dirtystate();
    gg_drawgraph ();
    gg_update_all_sets_lists ();
    ge_update_explorer ();
    xfree (sl1);
    xfree (sl2);
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide_all (gui.top);
  }
}
