/** File gg_eblockwin.c
 *  Transcripted from Motif to GTK+ by P. Vincent to
 *  Replace eblockwin.c
 */

#include <stdio.h>
#include <string.h>

#include <gtk/gtk.h>

#include "globals.h"
#include "graphs.h"
#include "draw.h"
#include "utils.h"
#include "ssdata.h"

#include "gw_list.h"

#include "gg_gutil.h"
#include "gg_gtkinc.h"
#include "gg_protos.h"
#include "ge_protos.h"

#include "zc.h"

extern GdkColor  bleuciel ,wheat;

static int block_curtype = SET_XY;
static int nncols = 0;           /* numeric columns */
static int nscols = 0;           /* string    "     */

static GtkWidget *w_eblock = NULL;

static GtkWidget *w_ncols;
static gg_GraphSetStructure *dest;
static GtkWidget *w_typ;
static GtkWidget *w_nchoices[MAX_SET_COLS];
static GtkWidget *w_schoice;
static GtkWidget *w_auto;
static GtkWidget *w_nxy[2];

static void gg_eblock_AAC_CB (GtkWidget *w ,gint reponse);
static void gg_eblock_update (int gno);
static void gg_w_typ_CB (GtkWidget *w ,gpointer p);

static void gg_dest_graph_CB (GtkWidget *graphs ,gg_GraphSetStructure *gss);

/**
 *  Replace  create_eblock_frame     eblockwin.c   72
 */
void gg_eblock_create_frame (int gno)
{
  GtkWidget *vb ,*hb;
  int nbtyp = NUMBER_OF_SETTYPES;
  int i;
  char buf[32];
  
  if (get_blockncols () == 0) {
    errmsg("Need to read block data first");
    return;
  }
  gg_set_wait_cursor ();
  if (w_eblock == NULL) {
    w_eblock   = gg_CreateAACDialog ("Edit block data" ,gg_eblock_AAC_CB ,0 ,0 ,&vb);
    w_ncols    = gg_label_new (vb ,"tmp");
    dest       = gg_CreateGraphSetSelector (vb ,"Load to:" ,GTK_SELECTION_BROWSE ,&wheat);
    w_typ      = gg_combo_fun (vb ,"Set type:" ,&nbtyp ,set_types ,0);

    for (i = 0; i < MAX_SET_COLS; i++) {
      w_nchoices[i] = NULL;
      sprintf(buf ,"%s from column:" ,dataset_colname (i));
      if (i < 2 ) {
	hb = gg_CreateHContainer (vb);
	w_nchoices[i] = gg_combo_new (hb ,buf ,2 ,"Index");
	w_nxy[i] = gg_spin_new (hb ,"len:" ,2.0 ,1000.0 ,1.0);
	gtk_widget_set_sensitive (w_nxy[i] ,FALSE);
      } else {
	w_nchoices[i] = gg_combo_new (vb ,buf ,2 ,"Index");
      }
    }
    w_schoice = gg_combo_new (vb ,"Strings from column:" ,2 ,"None");
    w_auto           = gg_combo_new (vb ,"Autoscale graph on load:" ,5
						,"None"          /* AUTOSCALE_NONE */
						,"X"             /* AUTOSCALE_X    */
						,"Y"             /* AUTOSCALE_Y    */
						,"XY");          /* AUTOSCALE_XY   */
    gg_set_int (w_auto ,autoscale_onread);
    g_signal_connect (w_typ 	   ,"changed" ,G_CALLBACK (gg_w_typ_CB) ,NULL);
    g_signal_connect (dest->graphs ,"gw_list" ,G_CALLBACK (gg_dest_graph_CB) 	  ,NULL);
  }
  gtk_widget_show_all (w_eblock);


  printf ("gg_eblock_create_frame: gno = %d \n" ,gno );
  //  gg_set_int (dest->graphs ,gno);

  gg_eblock_update    (gno);
  gg_unset_wait_cursor ();
}

/**
 *   Replace  update_eblock              eblockwin.c  128
 */
static void gg_eblock_update (int gno)
{
  int blocklen ,blockncols ,i ,j ,ncols;
  int *blockformats;
  char buf[128];
  if (w_eblock == NULL) return;
  if (is_valid_gno (gno)) {
    gg_set_int (dest->graphs ,gno);
  } else {
    gno = 0;
  }
  /* TODO: check if new data arrived */
  blockncols   = get_blockncols ();
  blocklen     = get_blocknrows ();
  blockformats = get_blockformats ();
  sprintf(buf ,"Block data: %d column(s) of length %d" ,blockncols ,blocklen);
  gtk_label_set_text (GTK_LABEL (w_ncols) ,buf);

  /* Remove all entries except the first (i.e. Index or None) */
  for (i = 0; i < MAX_SET_COLS; i++) {
    for (j = nncols; j > 0; j--) {
      gtk_combo_box_remove_text (GTK_COMBO_BOX (w_nchoices[i]) ,j);
    }
  }
  for (j = nscols; j > 0; j--) {
    gtk_combo_box_remove_text (GTK_COMBO_BOX (w_schoice) ,j);
  }

  /* Recreate items */
  nncols = 0;
  nscols = 0;
  for (i = 0; i < blockncols; i++) {
    sprintf(buf ,"%d" ,i + 1);
    if (blockformats[i] == FFORMAT_STRING) {
      nscols++;
      gtk_combo_box_append_text (GTK_COMBO_BOX (w_schoice) ,buf);
    } else {
      nncols++;
      for (j = 0; j < MAX_SET_COLS; j++) {
	gtk_combo_box_append_text (GTK_COMBO_BOX (w_nchoices[j]) ,buf);
      }
    }
  }
   /* defaults settings */
  if (nncols  > 0) gg_set_int (w_nchoices[0] ,1);    /* X */
  if (nncols  > 1) gg_set_int (w_nchoices[1] ,2);    /* Y */
  if (nscols == 1) gg_set_int (w_schoice     ,1);    /* String */
  
  ncols = settype_cols (block_curtype);
  for (i = 0; i < MAX_SET_COLS; i++) {
    gtk_widget_set_sensitive (w_nchoices[i] ,(i < ncols));
  }
  gtk_widget_set_sensitive (w_schoice ,(nscols > 0));
}



/**
 *  Replace  eblock_accept_notify_proc  eblockwin.c  211
 */
static void gg_eblock_AAC_CB (GtkWidget *w ,gint reponse)
{
  int i ,gno ,setno;
  int cs[MAX_SET_COLS] ,nncols ,scol ,autoscale;
  char *s;

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY) {
    if (!gg_GetSingleListChoice (GW_LIST(dest->graphs) ,&gno)) {
      errmsg("Please select a single graph");
      return;
    }

    if (!gg_GetSingleListChoice (GW_LIST(dest->sets) ,&setno)) {
    	/* no set selected; allocate new one */
      setno = NEW_SET;
    }
    nncols = settype_cols (block_curtype);
    for (i = 0; i < nncols; i++) {
      s = gtk_combo_box_get_active_text (GTK_COMBO_BOX (w_nchoices[i]));
      if (strncmp (s ,"Index" ,5) == 0) {
	cs[i] = -1;
      } else {
	sscanf (s ,"%d" ,&cs[i]); cs[i]--;
      }
    }
    s = gtk_combo_box_get_active_text (GTK_COMBO_BOX (w_schoice));
    if (strncmp (s ,"None" ,4) == 0) {
      scol = -1;
    } else {
      sscanf (s ,"%d" ,&scol); scol--;
    }
    if (block_curtype == SET_XYCMAP) {
      int nx = gg_get_int (w_nxy[0]);
      int ny = gg_get_int (w_nxy[1]);
      set_xymap_nxy (gno ,setno ,nx ,ny);
    }
    autoscale = gg_get_int (w_auto);
    create_set_fromblock (gno ,setno ,block_curtype ,nncols ,cs ,scol ,autoscale);
    gg_update_all ();
    ge_update_explorer ();
    gg_drawgraph();
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w_eblock);
  }
}

/**
 * Called to manage sensitivity of ComboBox used to choose a column
 *        according to the type of set selected in w_typ
 * replace  eblock_type_notify_proc    eblockwin.c  204
 */
static void gg_w_typ_CB (GtkWidget *w ,gpointer p)
{
  int i;
  block_curtype = gg_get_int (w_typ);
  int ncols         = settype_cols (block_curtype);
  for (i = 0; i < MAX_SET_COLS; i++) {
    gtk_widget_set_sensitive (w_nchoices[i] ,(i < ncols));
  }
  gtk_widget_set_sensitive (w_nxy[0] ,block_curtype == SET_XYCMAP);
  gtk_widget_set_sensitive (w_nxy[1] ,block_curtype == SET_XYCMAP);
}

/**
 * Called when destination graph is set or changed
 */
static void gg_dest_graph_CB (GtkWidget *graphs ,gg_GraphSetStructure *gss)
{
  int gno;
  if (!gg_GetSingleListChoice (GW_LIST(dest->graphs) ,&gno)) {
    errmsg("Edit Block Data: please select a single graph");
    return;
  }
  gg_eblock_update (gno);
}
