/**  File gg_fourier_win.c
 *  Replace fourier transform in gg_compwin.c
 */
#define __FOURIER_WIN_

#include <gtk/gtk.h>

#include "defines.h"

#include "gw_list.h"

#include "utils.h"
#include "graphs.h"

#include "gg_gtkinc.h"
#include "gg_protos.h"
#include "ge_protos.h"

#include "fourier.h"


extern GdkColor  bleuciel ,wheat;

static void gg_fourier_AAC_CB (GtkWidget *w ,gint reponse);

static GtkWidget *w_fourier = NULL;
static GtkWidget *vb;
static GtkWidget *w_inverse ,*w_norm    ,*w_complex;
static GtkWidget *w_set1_im;
static GtkWidget *w_xscale  ,*w_dumpDC  ,*w_filter;
static GtkWidget *w_padding ,*w_round2n  ,*w_insegmt;
static GtkWidget *w_out_fft  ,*w_outsegmt;
static GtkWidget *dummy_lab;
static gg_SrcDestStructure *srcdest;


static void gg_fourier_complexin_CB (GtkWidget *w ,gpointer p)
{
  int i = gg_get_int (w_complex);
  gtk_widget_set_sensitive (w_set1_im ,i == FFT_COMPLEX_IN_SET);
}

void gg_fourier_create_win (GtkWidget *w ,gpointer p)
{
  GtkWidget *fr;
  if (w_fourier == NULL) {
    w_fourier = gg_CreateAACDialog ("Fourier transform" ,gg_fourier_AAC_CB ,0 ,0 ,&vb);
    srcdest   = gg_CreateSrcDestSelector (vb ,GTK_SELECTION_MULTIPLE);

    fr = gg_frame (vb ,"General" ,&wheat ,2 ,2);
    w_out_fft  = gg_combo_new (fr ,"Compute:" ,7                
				 ,"Magnitude" 		     /* FFT_OUTPUT_MAGNITUDE */
				 ,"Phase"           	     /* FFT_OUTPUT_PHASE     */
				 ,"Real part"       	     /* FFT_OUTPUT_RE        */
				 ,"Imaginary part"  	     /* FFT_OUTPUT_IM        */
				 ,"Complex"         	     /* FFT_OUTPUT_REIM      */
				 ,"Complex (A, Phi)");	     /* FFT_OUTPUT_APHI      */
    dummy_lab = gg_label_new (fr ," ");
    w_inverse = gg_check_new (fr ,"Backward transform: exp(+)");
    w_norm    = gg_combo_new (fr ,"Normalize:" ,5
				,"None"                      /* FFT_NORM_NONE    */
				,"Symmetric"                 /* FFT_NORM_SYMMETRIC */
				,"Forward"                   /* FFT_NORM_FORWARD   */
				,"Backward");                /* FFT_NORM_BACKWARD  */

    fr = gg_frame (vb ,"Input" ,&bleuciel ,2 ,4);
    w_insegmt = gg_combo_new (fr ,"Input segment" ,3
				,"Index"
				,"[Xmin ,Xmax]");
    w_dumpDC  = gg_check_new (fr ,"Dump DC component");
    w_complex = gg_combo_new (fr ,"Imaginary part" ,4
			      ,"None"                        /* FFT_COMPLEX_IN_NONE  */
			      ,"3d column of the set"        /* FFT_COMPLEX_IN_COL3  */
			      ,"Another set");               /* FFT_COMPLEX_IN_SET   */
    w_set1_im = gg_spin_new (fr ,"Set # (Im)" ,0.0 ,1.0 ,1.0);
    w_filter  = gg_combo_new (fr ,"Apply window:" ,9       
				,"None (Rectangular)"	     /* FFT_WINDOW_NONE       */
				,"Triangular"		     /*	FFT_WINDOW_TRIANGULAR */
				,"Parzen"		     /*	FFT_WINDOW_PARZEN     */
				,"Welch"		     /*	FFT_WINDOW_WELCH      */
				,"Hanning"		     /*	FFT_WINDOW_HANNING    */
				,"Hamming"		     /*	FFT_WINDOW_HAMMING    */
				,"Flat top"		     /*	FFT_WINDOW_FLATTOP    */
				,"Blackman");		     /*	FFT_WINDOW_BLACKMAN   */
    w_padding	     = gg_spin_new (fr ,"Zero padding" ,1.0 ,99.0 ,1.0);
    w_round2n	     = gg_check_new(fr ,"Round to 2^N");

    fr = gg_frame (vb ,"Output" ,&wheat ,2 ,1);
    w_xscale  = gg_combo_new (fr ,"X scale:" ,4
				,"Index"                     /* FFT_XSCALE_INDEX */
				,"Frequency"                 /* FFT_XSCALE_NU    */
				,"Angular frequency");       /* FFT_XSCALE_OMEGA */
    w_outsegmt = gg_combo_new (fr ,"Output segment:" ,4
				 ,"Positive"                 /* FFT_OUTSEGMT_POSITIVE    */
				 ,"Positive half length"     /* FFT_OUTSEGMT_HALF_LENGTH */
				 ,"Centered");               /* FFT_OUTSEGMT_CENTERED    */
    g_signal_connect (w_complex ,"changed" ,G_CALLBACK (gg_fourier_complexin_CB) ,NULL);
    gg_fourier_update (get_cg());
    gg_fourier_complexin_CB (w_complex ,NULL);
  }
  gtk_widget_show_all (w_fourier);
}

void gg_fourier_update (int gno1)
{
  if (w_fourier != NULL) {
    FFTParams p;
    int last_set = number_of_sets (gno1) - 1;
    if (fourier_params_get (&p) == RETURN_SUCCESS) {
      gg_set_int  (w_inverse      , p.invflag);
      gg_set_int  (w_xscale       , p.xscale);
      gg_set_int  (w_norm         , p.norm);
      gg_set_int  (w_insegmt      , p.insegmt);
      gg_set_int  (w_complex      , p.complexin);
      gtk_spin_button_set_range (GTK_SPIN_BUTTON (w_set1_im) ,0.0 ,(double)last_set);
      gg_set_int  (w_set1_im      , p.set1_im);
      gg_set_int  (w_dumpDC       , p.dcdump);
      gg_set_int  (w_round2n      , p.round2n);
      gg_set_int  (w_filter       , p.filter);
      gg_set_dble (w_padding      , p.padding);
      gg_set_int  (w_outsegmt     , p.outsegmt);
      gg_set_int  (w_out_fft      , p.output);
    }
  }
  gg_update_all_sets_lists ();
  ge_update_explorer ();
}


void gg_fourier_AAC_CB (GtkWidget *w ,gint reponse)
{
  FFTParams p;
  static int ns2 = 0;
  int gno1 ,setno1 ,gno2 ,setno2 ,setno2im ,ns1 ,resno;
  gint *sl1 ,*sl2;
  char buf[256] ,cmplxin[20];
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY   ) {
    p.invflag      = gg_get_int (w_inverse);
    p.xscale       = gg_get_int (w_xscale);
    p.norm         = gg_get_int (w_norm);
    p.insegmt      = gg_get_int (w_insegmt);
    p.complexin    = gg_get_int (w_complex);
    p.set1_im      = gg_get_int (w_set1_im);
    p.dcdump       = gg_get_int (w_dumpDC);
    p.round2n      = gg_get_int (w_round2n);
    p.filter       = gg_get_int (w_filter);
    p.padding      = gg_get_int (w_padding);
    p.outsegmt     = gg_get_int (w_outsegmt);
    p.output       = gg_get_int (w_out_fft);
    fourier_params_set (&p);

    ns2 = fourier_nb_out_sets (p.output);
    if (gg_GetSrcDestSettings (srcdest ,SD_NSET2_GIVEN
                               ,&gno1 ,&gno2
                               ,&ns1  ,&ns2
                               ,&sl1  ,&sl2) == RETURN_FAILURE) return;
    if (ns1 != 1) {
      if (p.complexin == FFT_COMPLEX_IN_SET) {
	errmsg ("Please, select a single source set\n\t and use the spin button for the set of the imaginary part");
      } else {
	errmsg ("Please, select a single source set");
      }
      return;
    }
    if (p.complexin == FFT_COMPLEX_IN_COL3 && dataset_cols (gno1 ,sl1[0]) < 3) {
      errmsg ("Fourier transform of complex data required\n\t and the only source set selected does not have a 3d column valid");
      return;
    }
    setno1 = sl1[0];
    setno2 = sl2[0];
    if (ns2 == 2) {
      setno2im = sl2[1];
    } else if (ns2 > 2) {
      errmsg ("Plesae select at most two destination sets");
      return;
    } else {
      setno2im = SET_SELECT_NEXT;
    }
    /* follow_me_on */
    switch (p.complexin) {
    case FFT_COMPLEX_IN_NONE:
      sprintf (cmplxin ,"%s" ,"Real");
      break;
    case FFT_COMPLEX_IN_COL3:
      sprintf (cmplxin ,"%s" ,"Complex");
      break;
    case FFT_COMPLEX_IN_SET:
      snprintf (cmplxin ,19 ,"G%d.s%d" ,gno1 ,p.set1_im);
      break;
    }
    /*  sprintf       3   4   5     6   7   8   9   10  11  12  13  14  15  16  17*/
    /*  pars.yacc     1     3         5     7   9   11  13  15  17  19  21  23  25 */
    sprintf (buf ,"@ %s (G%d.s%d , G%d.s%d ,%s ,%s ,%s ,%s ,%s ,%s ,%g ,%s ,%s ,%s)\n" 
	     ,p.invflag ? "INVFFT" : "FFT"
	     ,gno1 ,setno1 
	     ,gno2 ,setno2 
	     ,fourierloady[p.output]
	     ,fouriernorm[p.norm]
	     ,fourierinseg[p.insegmt]
	     ,cmplxin
	     ,on_or_off(p.dcdump)
	     ,windowtype[p.filter]
	     ,p.padding
	     ,on_or_off(p.round2n)
	     ,fourierloadx[p.xscale]
	     ,fourieroutsg[p.outsegmt]
	     );
    gg_cmd_prt (buf);
    /* perform the computation 
     * the error messages are delivered by do_fourier */
    resno = do_fourier (gno1 ,setno1 ,gno2 ,setno2 ,setno2im);
    if (resno == RETURN_SUCCESS) {
      gg_drawgraph ();
      ge_update_explorer ();
    }
    xfree (sl1);
    xfree (sl2);
  }

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w_fourier);
  }
}
