/**  File gg_nonlwin.c
 *  Transcripted from Motif to GTK+ by P. Vincent to
 *   Replace nonlwin.c
 *   
 *    Non linear curve fitting
 *    with extensions from Nicola Ferralis <feranick@hotmail.com>
 *    and LOESS method
 *
 *    TODO: rationalize the mix Levenberg/LOESS into nonlfit.c
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <math.h>    /* added djconnel Feb 2010 */

#include "globals.h"
#include "graphs.h"
#include "utils.h"
#include "files.h"
#include "parser.h"
#include "noxprotos.h"

#include "gw_pannel.h"
#include "gw_choice.h"
#include "gw_list.h"

#include "gg_gutil.h"
#include "gg_gtkinc.h"
#include "gg_protos.h"
#include "ge_protos.h"
#include "gg_events.h"

#include "nonlfit.h"

extern NONLparams nonl_parms[MAXPARM];
extern NONLopts   nonl_opts;
extern NONL_load_prefs nonl_prefs;

typedef enum {
  NL_RESET
  ,NL_GAUSS_1
  ,NL_GAUSS_2
  ,NL_GAUSS_3
  ,NL_GAUSS_1A
  ,NL_LORENTZ_1
  ,NL_LORENTZ_2
  ,NL_LORENTZ_3
  ,NL_PSVOIGHT_1
  ,NL_PSVOIGHT_2
  ,NL_DS
  ,NL_ASYM2SIG
  ,NL_LOGNORMAL
  ,NL_GCAS
  ,NL_ECS
  ,NL_INVPOLY
  ,NL_SINE
  ,NL_SINE_SQ
  ,NL_SINE_DAMP
  ,NL_EXP_DEC_1
  ,NL_EXP_DEC_2
  ,NL_EXP_GROW_1
  ,NL_EXP_GROW_2
  ,NL_HYPERBOLE
  ,NL_BRADLEY
  ,NL_LOG3
  ,NL_WEIBULLPD
  ,NL_WEIBULLCD
  ,NL_LAST
} NL_library;

static gint popup_width = 200 ,popup_height = 350;
extern GdkColor  bleuciel ,wheat;

static char bufcmd[256];

/* Callbacks */
static void gg_nonl_AAC_CB	(GtkWidget *w ,gint reponse);
static void gg_openfit_CB	(GtkWidget *w ,gpointer p);
static void gg_savefit_CB	(GtkWidget *w ,gpointer p);
static void gg_nonl_reset_CB	(GtkWidget *w ,gpointer p);
static void gg_nonl_load_CB	(GtkWidget *w ,gpointer p);
static void gg_nonl_update_CB	(GtkWidget *w ,gpointer p);
static void gg_constr_CB        (GtkWidget *w ,gpointer p);
static void gg_nonl_toggle_CB   (GtkWidget *w ,gpointer p);
static void gg_nonl_method_CB   (GtkWidget *w ,gpointer p);
static void gg_value1_CB    	(GtkWidget *w ,gpointer p);
static void gg_range_CB    	(GtkWidget *w ,gpointer p);

static int gg_nonl_load_fit (int src_gno ,int set1 ,int dest_gno ,int set2 ,int force);

static char buf[256];



static GtkWidget *w_frame = NULL;
static GtkWidget *w_autol_item;

static gg_SrcDestStructure *srcdest;
static GtkWidget *w_tab          ,*w_tab_main;
static GtkWidget *w_formula ,*w_formula_frame ,*w_formula_box ,*w_nparm;
static GtkWidget *w_tol     ,*w_nsteps;
static GtkWidget *w_logpts;   /* djconnel Feb 2010 */
static GtkWidget *w_nprint;
static GtkWidget *w_parm[MAXPARM];
static GtkWidget *w_value[MAXPARM];
static GtkWidget *w_constr[MAXPARM];
static GtkWidget *w_lowb[MAXPARM];
static GtkWidget *w_uppb[MAXPARM];
static GtkWidget *w_advanced;
static gg_RestrictionStructure  *restr_item;
static GtkWidget *w_weight;
static GtkWidget *w_wfunc;
static GtkWidget *w_load;
static GtkWidget *w_fload_rc;
static GtkWidget *w_start;
static GtkWidget *w_stop;
static GtkWidget *w_npts;

static GtkWidget *w_method;
static GtkWidget *w_loess      ,*w_loess_box;
static GtkWidget *w_confidence ,*w_level;
static GtkWidget *w_blen;
static GtkWidget *w_levenberg  ,*w_levenberg_box;
static GtkWidget *w_family     ,*w_linquad;
static GtkWidget *w_span;
static GtkWidget *w_value1[MAXPARM];
static GtkWidget *w_step1[MAXPARM];

/* Library hacked  from  nonlinear_extended by Nicola Ferralis */
static void nonl_lib_CB (GtkWidget *w ,gpointer p);

static void gg_do_nparm_CB (GtkWidget *widget ,gpointer p);
static void gg_nonl_wf_CB  (GtkWidget *widget ,gpointer p);

static int dest_gno;
static gboolean block_value1_CB = FALSE;

/* ARGSUSED */
/**
 *  Replace  create_nonl_frame       nonlwin.c   115
 */
void gg_create_nonl_popup (GtkWidget *w ,gpointer p)
{
  GtkWidget *vb ,*menubar ,*menupane ,*rc1 ,*rc2 ,*sw ,*sp;
  int i;

  if (w_frame == NULL) {
    w_frame = gg_CreateAACDialog ("Non-linear curve fitting" ,gg_nonl_AAC_CB ,0 ,0 ,&vb);
 
    /* ------------ Menubar -------------- */
    menubar = gg_CreateMenuBar (vb);
    menupane = gg_CreateMenu (menubar,  "File", 'F', FALSE);
    gg_CreateMenuButton      (menupane, "Open...", 'O', NULL ,gg_openfit_CB ,-1);
    gg_CreateMenuButton      (menupane, "Save...", 'S' ,NULL ,gg_savefit_CB ,-1);
    gg_CreateMenuSeparator   (menupane);
    gg_CreateMenuCloseButton (menupane ,w_frame);

    menupane = gg_CreateMenu (menubar,  "Edit", 'E', FALSE);
    gg_CreateMenuButton      (menupane, "Reset fit parameters" , 'R' ,NULL , gg_nonl_reset_CB ,-1);
    gg_CreateMenuSeparator   (menupane);
    gg_CreateMenuButton      (menupane, "Load current fit"     , 'L' ,NULL , gg_nonl_load_CB  ,-1);

    menupane = gg_CreateMenu (menubar, "View", 'V', FALSE);
    w_autol_item = gg_CreateMenuToggle (menupane, "Autoload" ,'A' ,NULL ,NULL ,TRUE);
    gg_CreateMenuSeparator (menupane);
    gg_CreateMenuButton    (menupane , "Update" ,'U'  ,NULL ,gg_nonl_update_CB  ,-1);
    
    menupane = gg_CreateMenu (menubar ,"Library", 'L', FALSE);

    sp = gg_CreateMenu (menupane ,"Gaussian Functions", 'G', FALSE);
    gg_CreateMenuButton    (sp ,"Single" 		      ,'S'  ,NULL ,nonl_lib_CB ,NL_GAUSS_1);
    gg_CreateMenuButton    (sp ,"Double" 		      ,'D'  ,NULL ,nonl_lib_CB ,NL_GAUSS_2);
    gg_CreateMenuButton    (sp ,"Triple" 		      ,'T'  ,NULL ,nonl_lib_CB ,NL_GAUSS_3);
    gg_CreateMenuSeparator (sp);
    gg_CreateMenuButton    (sp ,"Single (chromatography)" ,'c'  ,NULL ,nonl_lib_CB ,NL_GAUSS_1A);

    sp = gg_CreateMenu (menupane ,"Lorentzian Functions", 'L', FALSE);
    gg_CreateMenuButton    (sp ,"Single" 		      ,'S'  ,NULL ,nonl_lib_CB ,NL_LORENTZ_1);
    gg_CreateMenuButton    (sp ,"Double" 		      ,'D'  ,NULL ,nonl_lib_CB ,NL_LORENTZ_2);
    gg_CreateMenuButton    (sp ,"Triple" 		      ,'T'  ,NULL ,nonl_lib_CB ,NL_LORENTZ_3);

    sp = gg_CreateMenu (menupane ,"Peak Functions", 'P', FALSE);
    gg_CreateMenuButton    (sp ,"Pseudo Voigt 1" 	      ,'V'  ,NULL ,nonl_lib_CB ,NL_PSVOIGHT_1);
    gg_CreateMenuButton    (sp ,"Pseudo Voigt 2" 	      ,'o'  ,NULL ,nonl_lib_CB ,NL_PSVOIGHT_2);
    gg_CreateMenuButton    (sp ,"Doniach-Sunjic" 	      ,'D'  ,NULL ,nonl_lib_CB ,NL_DS);
    gg_CreateMenuButton    (sp ,"Asymmetric Double Sigmoidal" ,'S'  ,NULL ,nonl_lib_CB ,NL_ASYM2SIG);
    gg_CreateMenuButton    (sp ,"LogNormal"                   ,'L'  ,NULL ,nonl_lib_CB ,NL_LOGNORMAL);
    gg_CreateMenuButton    (sp ,"Gram-Charlier A-Series"      ,'C'  ,NULL ,nonl_lib_CB ,NL_GCAS);
    gg_CreateMenuButton    (sp ,"Edgeworth-Cramer Series"     ,'E'  ,NULL ,nonl_lib_CB ,NL_ECS);
    gg_CreateMenuButton    (sp ,"Inverse Polynomial"          ,'I'  ,NULL ,nonl_lib_CB ,NL_INVPOLY);

    sp = gg_CreateMenu (menupane ,"Periodic Peak Functions", 'e', FALSE);
    gg_CreateMenuButton    (sp ,"Sine" 	              	      ,'S'  ,NULL ,nonl_lib_CB ,NL_SINE);
    gg_CreateMenuButton    (sp ,"Sine Square" 	      	      ,'q'  ,NULL ,nonl_lib_CB ,NL_SINE_SQ);
    gg_CreateMenuButton    (sp ,"Sine Damp" 	      	      ,'D'  ,NULL ,nonl_lib_CB ,NL_SINE_DAMP);

    sp = gg_CreateMenu (menupane ,"Baseline Functions", 'e', FALSE);
    gg_CreateMenuButton    (sp ,"Exponential Decay 1" 	      ,' '  ,NULL ,nonl_lib_CB ,NL_EXP_DEC_1);
    gg_CreateMenuButton    (sp ,"Exponential Decay 2" 	      ,' '  ,NULL ,nonl_lib_CB ,NL_EXP_DEC_2);
    gg_CreateMenuButton    (sp ,"Exponential Growth 1" 	      ,' '  ,NULL ,nonl_lib_CB ,NL_EXP_GROW_1);
    gg_CreateMenuButton    (sp ,"Exponential Growth 2" 	      ,' '  ,NULL ,nonl_lib_CB ,NL_EXP_GROW_2);
    gg_CreateMenuButton    (sp ,"Hyperbolic Function" 	      ,' '  ,NULL ,nonl_lib_CB ,NL_HYPERBOLE);
    gg_CreateMenuSeparator (sp);
    gg_CreateMenuButton    (sp ,"Bradley" 	              ,' '  ,NULL ,nonl_lib_CB ,NL_BRADLEY);
    gg_CreateMenuButton    (sp ,"Logarithm 3" 	              ,' '  ,NULL ,nonl_lib_CB ,NL_LOG3);
    gg_CreateMenuButton    (sp ,"Weibull Probability Density" ,' '  ,NULL ,nonl_lib_CB ,NL_WEIBULLPD);
    gg_CreateMenuButton    (sp ,"Weibull Cumulative"          ,' '  ,NULL ,nonl_lib_CB ,NL_WEIBULLCD);
    gg_CreateMenuSeparator (sp);

    menupane = gg_CreateMenu (menubar, "Help", 'H', TRUE);
    // A COMPLETER

    /* Create sets_lists */
    srcdest   = gg_srcdest_new (vb ,GTK_SELECTION_BROWSE ,popup_width ,80 ,150);

    /* Notebook */
    w_tab = gtk_notebook_new ();
    gtk_box_pack_start_defaults (GTK_BOX (vb) ,w_tab);

    /* ------------ Main tab -------------- */

    w_tab_main = gg_CreateTabPage (w_tab ,"Main");
    w_method = gg_combo_new (w_tab_main ,"Method" ,4   /* in ../loess/loess.h */
			     ,"Levenberg-Marquardt"    /* NL_LEVENBERG  */
			     ,"Loess"                  /* NL_LOESS   */
			     ,"By hand"                /* NL_BY_HAND */
			     );
    /* Loess widgets */
    w_loess     = gg_frame (w_tab_main ,"Loess parameters" ,&wheat ,2 ,4);
    w_loess_box = gg_get_w_frame_box ();
    w_span    	= gg_spin_new  (w_loess ,"Span" 	     ,0.01 ,10.0 ,0.01);
    w_family  	= gg_combo_new (w_loess ,"Distribution" ,3
				,"symmetric"    	      /* NL_SYMMETRIC */
				,"gaussian");                 /* NL_GAUSSIAN  */
    w_linquad  = gg_combo_new (w_loess ,"Degree" ,3
			      ,"linear"
			      ,"quadratic");
    gg_label_new (w_loess ,NULL);
    w_confidence = gg_check_new (w_loess ,"Compute confidence intervals");
    w_level      = gg_spin_new  (w_loess ,"confidence level" ,0.1 ,0.999 ,0.001);
    w_blen       = gg_spin_new  (w_loess ,"Nb bars" 	     ,2.0 ,100.0 ,1.0);
    /* tool tips */
    gtk_widget_set_tooltip_text (w_family   ,"Hypothesis for noise distribution");
    gtk_widget_set_tooltip_text (w_linquad  ,"Degree of local approximation functions");
    gtk_widget_set_tooltip_text (w_span     ,"Fraction of x_interval used as local set");
    /* widgets common to Levenberg and By_hand */
    w_formula_frame = gg_frame (w_tab_main ,"Formula" ,&wheat ,0 ,1);
    w_formula_box   = gg_get_w_frame_box ();
    w_formula 	    = gg_entry_new (w_formula_frame ,"" ,60);
    w_nparm   	    = gg_combo_new (w_formula_frame ,"Parameters:" ,12
				  ,"0" ,"1" ,"2" ,"3" ,"4" ,"5"
				  ,"6" ,"7" ,"8" ,"9" ,"10");
    sw = gtk_scrolled_window_new (NULL,NULL);
    gtk_scrolled_window_set_policy  (GTK_SCROLLED_WINDOW(sw)
				     ,GTK_POLICY_AUTOMATIC
				     ,GTK_POLICY_AUTOMATIC);
    gtk_widget_set_size_request (GTK_WIDGET (sw) ,popup_width ,180);
    /* parameters enumeration */
    rc2 = gtk_vbox_new (FALSE ,2);
    for (i = 0; i < MAXPARM; i++) {
      w_parm[i] = gg_CreateHContainer(rc2);
      sprintf(buf, "A%1d: ", i);
      w_value[i]  = gg_entry_new (w_parm[i] ,buf ,10);
      w_value1[i] = gg_spin_new  (w_parm[i] ,buf ,0.0 ,10.0 ,0.1);
      w_constr[i] = gg_check_new (w_parm[i], "Bounds:");
      w_lowb[i]   = gg_entry_new (w_parm[i] ,""  ,6);
      sprintf(buf, "< A%1d < ", i);
      gg_label_new (w_parm[i], buf);
      w_uppb[i]   = gg_entry_new (w_parm[i] ,""  ,6);
      w_step1[i]  = gg_entry_new (w_parm[i] ,"step:" ,6);
      g_signal_connect (w_value1[i] ,"value-changed" ,G_CALLBACK (gg_value1_CB) ,GINT_TO_POINTER (i));
      g_signal_connect (w_constr[i] ,"toggled" 	     ,G_CALLBACK (gg_constr_CB) ,GINT_TO_POINTER (i));
      g_signal_connect (w_lowb[i]   ,"activate"      ,G_CALLBACK (gg_range_CB)  ,GINT_TO_POINTER (i));
      g_signal_connect (w_uppb[i]   ,"activate"      ,G_CALLBACK (gg_range_CB)  ,GINT_TO_POINTER (i));
      g_signal_connect (w_step1[i]  ,"activate"      ,G_CALLBACK (gg_range_CB)  ,GINT_TO_POINTER (i));
    }
    gtk_widget_show_all (rc2);
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(sw) ,rc2);
    gtk_widget_show (sw); 
    gtk_box_pack_start (GTK_BOX (w_formula_frame) ,sw ,FALSE ,FALSE , 1);

    /* Levenberg  widgets */
    w_levenberg     = gg_frame (w_tab_main ,"Levenberg parameters" ,&wheat ,0 ,1);
    w_levenberg_box = gg_get_w_frame_box ();
    rc1 = gg_CreateHContainer (w_levenberg);
    w_tol     = gg_entry_new  (rc1 ,"Tolerance:" ,8);
    w_nsteps  = gg_spin_new (rc1 ,"Iterations:" ,0.0 ,500.0 ,5.0);
    w_nprint  = gg_check_new (rc1 ,"Trace on");
    gg_set_int  (w_nprint ,FALSE);
    gg_set_dble (w_nsteps ,5.0);
    gg_set_int  (w_nparm  ,0);

    /* ------------ Advanced tab --------------*/
    
    w_advanced = gg_CreateTabPage (w_tab ,"Advanced");

    restr_item = gg_CreateRestrictionChoice (w_advanced ,"Source data filtering");

    rc1 = gg_frame (w_advanced ,"Weighting" ,&wheat ,0 ,1);
    w_weight = gg_combo_new (rc1 ,"Weights" ,6
			    ,"None"       /* NL_WEIGHT_NONE 	 */
			    ,"1/Y"        /* NL_WEIGHT_Y    	 */
			    ,"1/Y^2"      /* NL_WEIGHT_Y2   	 */
			    ,"1/dY^2"     /* NL_WEIGHT_DY   	 */
			    ,"Custom"     /* NL_WEIGHT_CUSTOM */
			    );
    w_wfunc = gg_entry_new   (rc1 ,"Function:" ,30);

    rc1 = gg_frame (w_advanced ,"Load options" ,&wheat ,0 ,1);
    w_load  = gg_combo_new (rc1 ,"Load" ,4
			    ,"Fitted values"  	     /* LOAD_VALUES    */
			    ,"Residuals"      	     /* LOAD_RESIDUALS */
			    ,"Function from formula" /* LOAD_FUNCTION  */
			    );
    w_fload_rc = gg_CreateVContainer (rc1);
    rc2        = gg_CreateHContainer (w_fload_rc);
    w_start    = gg_entry_new  (rc2 ,"Start load at:" ,6);
    w_stop     = gg_entry_new  (rc2 ,"Stop load at:"  ,6);
    rc2        = gg_CreateHContainer (w_fload_rc);
    w_npts     = gg_entry_new  (rc2 ,"# of points:"   ,4);
    w_logpts   = gg_check_new  (rc2, "logarithmic");

    /* ------------ Connect callbacks --------------*/
    g_signal_connect (w_load   ,"changed" ,G_CALLBACK (gg_nonl_toggle_CB) ,NULL);
    g_signal_connect (w_nparm  ,"changed" ,G_CALLBACK (gg_do_nparm_CB) 	  ,NULL);
    g_signal_connect (w_weight ,"changed" ,G_CALLBACK (gg_nonl_wf_CB) 	  ,NULL);
    g_signal_connect (w_method ,"changed" ,G_CALLBACK (gg_nonl_method_CB) ,NULL);

    /* ------------Initialize widgets  --------------*/
#ifndef HAVE_FORTRAN
    gtk_widget_set_sensitive (w_method ,FALSE);
    gtk_widget_hide (w_loess_box);
#endif /* HAVE_FORTRAN */
    gg_set_int  (w_method  ,nonl_opts.nonl_method);
    gg_set_int  (w_family  ,nonl_opts.family);
    gg_set_int  (w_linquad ,nonl_opts.linquad);
    gg_set_dble (w_span    ,nonl_opts.span);
    gg_set_dble (w_level   ,nonl_opts.confidence_level);
  }
  gtk_widget_show_all (w_frame);
  gg_nonl_update ();

}


/**************** C A L L B A C K S ****************/

static void gg_nonl_method_CB   (GtkWidget *w ,gpointer p)
{
  nonl_opts.nonl_method = gg_get_int (w);
  switch (nonl_opts.nonl_method) {
  case NL_LEVENBERG:
  case NL_BY_HAND:
    gtk_widget_show (w_formula_box);
    gtk_widget_hide (w_loess_box);
    if (nonl_opts.nonl_method == NL_LEVENBERG) {
      gtk_widget_show (w_levenberg_box);
    } else {
      gtk_widget_hide (w_levenberg_box);
    }
    gg_do_nparm_CB (w_nparm ,NULL);
    gg_nonl_update (); //053a
    break;
  case NL_LOESS:
    gtk_widget_show (w_loess_box);
    gtk_widget_hide (w_levenberg_box);
    gtk_widget_hide (w_formula_box);
    break;
  }
}


/**
 *  Replace  do_nparm_toggle	    nonlwin.c   261
 */
static void gg_do_nparm_CB (GtkWidget *widget ,gpointer p)
{
  int i  ,n;

  n = gg_get_int (widget);
  for (i = 0; i < MAXPARM; i++) {
    if (i < n) {
      gtk_widget_show_all (w_parm[i]);
      if (nonl_opts.nonl_method == NL_LEVENBERG) {
	gtk_widget_set_sensitive (w_lowb[i] ,nonl_parms[i].constr);
	gtk_widget_set_sensitive (w_uppb[i] ,nonl_parms[i].constr);
	gtk_widget_hide (w_value1[i]);
	gtk_widget_hide (w_step1[i]);
      } else {
	gtk_widget_set_sensitive (w_lowb[i] ,TRUE);
	gtk_widget_set_sensitive (w_uppb[i] ,TRUE);
	gtk_widget_hide (w_value[i]);
	gtk_widget_hide (w_constr[i]);
      }
    } else {
      gtk_widget_hide (w_parm[i]);
    }
  }
}

/**
 *  Replace  reset_nonl_frame_cb    nonlwin.c   273
 */
static void  gg_nonl_reset_CB	(GtkWidget *w ,gpointer p)
{
  nonlfit_set_defaults ();
  gg_nonl_update ();
}



/**
 * Replace  update_nonl_frame_cb   nonlwin.c   279
 */
static void gg_nonl_update_CB (GtkWidget *w ,gpointer p)
{
  gg_nonl_update (); 
}


/**
 *  Replace  update_nonl_frame	    nonlwin.c   284
 */
void gg_nonl_update (void)
{
  int i;
  if (w_frame) {
    block_value1_CB = TRUE;    
    gg_set_int (w_method ,nonl_opts.nonl_method);
    switch (nonl_opts.nonl_method) {
    case NL_LEVENBERG:
    case NL_BY_HAND:
      gg_setstr   (w_formula      ,nonl_opts.formula);
      gg_setstr_d (w_tol    ,"%g" ,nonl_opts.tolerance ,buf);
      gg_set_int  (w_nparm        ,nonl_opts.parnum);
      for (i = 0; i < MAXPARM; i++) {
	gg_setstr_d (w_value[i]  ,"%g" ,nonl_parms[i].value ,buf);
	gg_set_dble (w_value1[i]       ,nonl_parms[i].value     );
	gg_set_int  (w_constr[i]       ,nonl_parms[i].constr    );
	gg_setstr_d (w_lowb[i]   ,"%g" ,nonl_parms[i].min   ,buf);
	gg_setstr_d (w_uppb[i]   ,"%g" ,nonl_parms[i].max   ,buf);
	gg_setstr_d (w_step1[i]  ,"%g" ,nonl_parms[i].step  ,buf);
      }
      gg_do_nparm_CB (w_nparm ,NULL);
      gg_set_int (w_autol_item ,nonl_prefs.autoload);
      gg_set_int (w_load  ,nonl_prefs.load);
      if (nonl_prefs.load == NL_LOAD_FUNCTION) {
	gtk_widget_set_sensitive (w_fload_rc ,TRUE);
      } else {
	gtk_widget_set_sensitive (w_fload_rc ,FALSE);
      }
      if (nonl_opts.wmethod == NL_WEIGHT_CUSTOM) {
	gtk_widget_set_sensitive (w_wfunc ,TRUE);
      } else {
	gtk_widget_set_sensitive (w_wfunc ,FALSE);
      }
      gg_setstr_d (w_start  ,"%g" ,nonl_prefs.start   ,buf);
      gg_setstr_d (w_stop   ,"%g" ,nonl_prefs.stop    ,buf);
      sprintf(buf ,"%d" ,nonl_prefs.npoints);
      gg_setstr(w_npts ,buf);
      gg_set_int (w_logpts ,nonl_opts.logpts); /* added djconnel Feb 2010 */
      break;
    case NL_LOESS:
      gg_set_int  (w_family  	,nonl_opts.family);
      gg_set_int  (w_linquad 	,nonl_opts.linquad);
      gg_set_dble (w_span    	,nonl_opts.span);
      gg_set_int  (w_confidence ,nonl_opts.confidence_compute);
      gg_set_int  (w_blen       ,nonl_opts.confidence_len);
      gg_set_dble (w_level 	,nonl_opts.confidence_level);
      break;
    }
    block_value1_CB = FALSE;
  }
}

/**
 *  Replace  nonl_wf_cb		    nonlwin.c   348
 */
static void gg_nonl_wf_CB (GtkWidget *w ,gpointer p)
{
  nonl_opts.wmethod = gg_get_int (w_weight);
  gtk_widget_set_sensitive (w_wfunc ,nonl_opts.wmethod == NL_WEIGHT_CUSTOM);
}

/**
 *  Replace  do_nonl_toggle	    nonlwin.c   359
 */
static void gg_nonl_toggle_CB  (GtkWidget *w ,gpointer p)
{
  int value = gg_get_int (w_load);

  if (value == NL_LOAD_FUNCTION) {
    gtk_widget_set_sensitive (w_fload_rc ,TRUE);
  } else {
    gtk_widget_set_sensitive (w_fload_rc ,FALSE);
  }
}

/**
 *  Replace  do_constr_toggle	    nonlwin.c   370
 */
static void gg_constr_CB (GtkWidget *w ,gpointer p)
{
  int i = GPOINTER_TO_INT (p);
  int onoff = gg_get_int (w);
  if (onoff) {
    gtk_widget_set_sensitive (w_lowb[i], TRUE);
    gtk_widget_set_sensitive (w_uppb[i], TRUE);
    nonl_parms[i].constr = TRUE;
  } else {
    gtk_widget_set_sensitive (w_lowb[i], FALSE);
    gtk_widget_set_sensitive (w_uppb[i], FALSE);
    nonl_parms[i].constr = FALSE;
  }
}

/*
set1 
set2
* formula    nonl_opts.formula             FIT FORMULA sexpr
* nb params  nonl_opts.parnum		   FIT WITH nexpr PARAMETERS
* tolerance  nonl_opts.tolerance	   FIT PREC expr
* iterations nonl_opts.nsteps              FIT MAX STEPS nexpr
* trace on   nonl_opts.trace_on            FIT TRACE onoff
* les paramaetres A0,...  et leurs contraintes
* 	     nonl_parms[i].value           An
nonl_opts.rtype: none, region 1,...       NONLFIT (selectset ,selectset ,REGNUM ,onoff ')'
nonl_opts.rnegate:                                                               ici
nonl_opts.wmethod: none ,1/Y, 1/Y^2, ... Formula
Load options
   Fitted values
   Residuals
   Function
       start at , stop at, # of points, logarithmic

       NONLFIT (set1 ,set2 ,restriction ,nonl_opts.wmethod ,weight_formula ,
*/

/* See pars.yacc */
static char *nonlloadtype[] = {"VALUES"   ,"RESIDUALS" ,"FORMULA"};
static char *nonlfamily[]   = {"SYMMETRIC" ,"GAUSSIAN"};   /* See ../loess/loess.h */

static void gg_nonl_follow_me (int src_gno ,int src_setno ,int dest_gno ,int dest_setno)
{
  int i;
  snprintf (bufcmd ,255 ,"\n# Non linear fit: ");
  gg_cmd_prt (bufcmd);
  switch (nonl_opts.nonl_method) {
  case NL_LEVENBERG:
    snprintf   (bufcmd ,255 ,"(Levenberg method) %s\n" 	     ,PSTRING (nonl_opts.title));
    gg_cmd_prt (bufcmd);
    snprintf   (bufcmd ,255 ,"@   FIT FORMULA \"%s\"\n"      ,PSTRING (nonl_opts.formula));
    gg_cmd_prt (bufcmd);
    snprintf   (bufcmd ,255 ,"@   FIT WITH %d PARAMETERS\n"  ,nonl_opts.parnum);
    gg_cmd_prt (bufcmd);
    snprintf   (bufcmd ,255 ,"@   FIT PREC %g\n" 	     ,nonl_opts.tolerance);
    gg_cmd_prt (bufcmd);
    snprintf   (bufcmd ,255 ,"@   FIT MAX STEPS %d\n" 	     ,nonl_opts.nsteps);
    gg_cmd_prt (bufcmd);
    snprintf   (bufcmd ,255 ,"@   FIT TRACE %s\n" 	     ,on_or_off (nonl_opts.trace_on));
    gg_cmd_prt (bufcmd);
    snprintf   (bufcmd ,255 ,"@   FIT LOAD %s\n" 	     ,nonlloadtype[nonl_prefs.load]);
    gg_cmd_prt (bufcmd);
    for (i = 0; i < nonl_opts.parnum; i++) {
      snprintf 	 (bufcmd ,255 ,"@   a%1d = %g;\n" ,i ,nonl_parms[i].value);
      gg_cmd_prt (bufcmd);
      snprintf 	 (bufcmd ,255 ,"@   a%1d constraints %s" ,i ,on_or_off (nonl_parms[i].constr));
      gg_cmd_prt (bufcmd);
      if (nonl_parms[i].constr) {
	snprintf (bufcmd ,255 ,";\t a%1dmin = %g;\t a%1dmax = %g\n" ,i ,nonl_parms[i].min ,i ,nonl_parms[i].max);
      } else {
	snprintf (bufcmd ,255 ,"\n");
      }
      gg_cmd_prt (bufcmd);
    }
    if (nonl_opts.wmethod == NL_WEIGHT_NONE) {
      snprintf   (bufcmd ,255 ,"@   NONLFIT ( G%d.s%d , G%d.s%d) \n" ,src_gno ,src_setno ,dest_gno ,dest_setno);
    } else {
      if (nonl_opts.wmethod == NL_WEIGHT_CUSTOM) {
	snprintf   (bufcmd ,255 ,"@   FIT WEIGHT FORMULA \"%s\" \n" ,nonl_opts.wfunc);
	gg_cmd_prt (bufcmd);
      }
      snprintf   (bufcmd ,255 ,"@   NONLFIT ( G%d.s%d , G%d.s%d , %d ) \n" 
		  ,src_gno ,src_setno ,dest_gno ,dest_setno ,nonl_opts.wmethod);
    }
    gg_cmd_prt (bufcmd);
    break;
    case NL_LOESS:
      snprintf   (bufcmd ,255 ,"LOESS method\n");
      gg_cmd_prt (bufcmd);
      snprintf   (bufcmd ,255 ,"LOESS (G%d.s%d ,%s ,%d ,%g ,%s ,%g ,%d , G%d.s%d)\n" 
		  ,src_gno ,src_setno ,nonlfamily[nonl_opts.family]
		  ,nonl_opts.linquad + 1
		  ,nonl_opts.span
		  ,on_or_off (nonl_opts.confidence_compute)
		  ,nonl_opts.confidence_level
		  ,nonl_opts.confidence_len
		  ,dest_gno ,dest_setno
		  );
      gg_cmd_prt (bufcmd);
      break;
  default:
    /* No follow me for NL_BY_HAND */
      break;
    }

}

/**
 *  Replace  do_nonl_proc	    nonlwin.c   385
 */
static void  gg_nonl_AAC_CB (GtkWidget *w ,gint reponse)
{
  int src_gno ,src_setno ,dest_setno ,bar_setno;
  static int ns1 ,ns2 = 1;
  gint *sl1 ,*sl2;
  plotarr pa;
  int resno ,i;
  double *warray;
  char *rarray;     	      /* Actually, rarray is an array of boolean */

  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_APPLY   ) {
    if (gg_GetSrcDestSettings (srcdest ,SD_NSET2_GIVEN
                               ,&src_gno ,&dest_gno
                               ,&ns1  ,&ns2
                               ,&sl1  ,&sl2) == RETURN_FAILURE) return;

    src_setno  = sl1[0];
    dest_setno = sl2[0];
    xfree (sl1);
    xfree (sl2);

    nonl_opts.nonl_method = gg_get_int (w_method);
    nonl_prefs.load 	  = gg_get_int (w_load);
    nonl_opts.wmethod     = gg_get_int (w_weight);
    warray = NULL;
    if (nonl_opts.wmethod == NL_WEIGHT_CUSTOM) {
      nonl_opts.wfunc = copy_string (nonl_opts.wfunc ,gg_getstr (w_wfunc));
    }
    nonl_opts.rtype   = gg_get_int (restr_item->r_sel) - 1; /* combo box None=0  <-> nonl_opts.rtype=-1*/
    switch (nonl_opts.nonl_method) {
    case NL_LEVENBERG:
      nonl_opts.formula   = copy_string (nonl_opts.formula ,gg_getstr (w_formula));
      nonl_opts.nsteps    = gg_get_int (w_nsteps);
      nonl_opts.tolerance = atof (gg_getstr (w_tol));
      nonl_opts.trace_on  = gg_get_int (w_nprint);
      nonl_opts.parnum    = gg_get_int (w_nparm);
      for (i = 0; i < nonl_opts.parnum; i++) {
	strcpy (buf ,gg_getstr (w_value[i]));
	if (sscanf(buf ,"%lf" ,&nonl_parms[i].value) != 1) {
	  errmsg ("Invalid input in parameter field");
	  return;
	}
	nonl_parms[i].constr = gg_get_int (w_constr[i]);
	if (nonl_parms[i].constr) {
	  strcpy (buf ,gg_getstr (w_lowb[i]));
	  if (sscanf (buf ,"%lf" ,&nonl_parms[i].min) != 1) {
	    errmsg ("Invalid input in low-bound field");
	    return;
	  }
	  strcpy (buf ,gg_getstr (w_uppb[i]));
	  if (sscanf (buf ,"%lf" ,&nonl_parms[i].max) != 1) {
	    errmsg( "Invalid input in upper-bound field");
	    return;
	  }
	  if ((nonl_parms[i].value < nonl_parms[i].min) || (nonl_parms[i].value > nonl_parms[i].max)) {
	    errmsg ("Initial values must be within bounds");
	    return;
	  }
	}
      }
      if (nonl_opts.nsteps) {
	/* compute the weigth array "warray" */
	if (compute_weights (src_gno ,src_setno ,nonl_opts.wmethod ,nonl_opts.wfunc ,&warray) != RETURN_SUCCESS) return;
	/* apply restriction */
	nonl_opts.rnegate = gg_get_int (restr_item->negate);
	resno = get_restriction_array (src_gno ,src_setno ,nonl_opts.rtype ,nonl_opts.rnegate ,&rarray);
	if (resno != RETURN_SUCCESS) {
	  errmsg ("Error in restriction evaluation");
	  xfree (warray);
	  return;
	}
      }
      if (follow_me_on) gg_nonl_follow_me (src_gno ,src_setno ,src_gno ,dest_setno);

      /* The fit itself, results go into the array nonl_parms[] */
      resno = do_nonlfit (src_gno ,src_setno ,warray ,rarray ,dest_gno);
      xfree (warray);
      xfree (rarray);
      warray = NULL;
      if (resno != RETURN_SUCCESS) {
	errmsg("Fatal error in do_nonlfit()");  
	return;  	
      }
      for (i = 0; i < nonl_opts.parnum; i++) {
	gg_setstr_d (w_value[i] ,"%g" ,nonl_parms[i].value ,buf);
      }
      /*  select & activate a set to load results to and compute (x,y) using nonl_parms[] */    
      gg_nonl_load_fit (src_gno ,src_setno ,dest_gno ,dest_setno ,FALSE);
      break;
    case NL_LOESS:
      nonl_opts.family   	   = gg_get_int  (w_family);
      nonl_opts.linquad 	   = gg_get_int  (w_linquad);
      nonl_opts.span          	   = gg_get_dble (w_span);
      nonl_opts.confidence_compute = gg_get_int  (w_confidence);
      if (nonl_opts.confidence_compute == TRUE) {
	nonl_opts.confidence_level = gg_get_dble (w_level);
	nonl_opts.confidence_len   = gg_get_int  (w_blen);
      }
      if (follow_me_on) gg_nonl_follow_me (src_gno ,src_setno ,src_gno ,dest_setno);
	/* compute the weigth array "warray" */
      if (compute_weights (src_gno ,src_setno ,nonl_opts.wmethod ,nonl_opts.wfunc ,&warray) != RETURN_SUCCESS) return;
      if (nonl_opts.rtype != RESTRICT_NONE) {
	errmsg ("LOESS: Source data filtering not yet allowed\n\tSelect None in Advanced tab or use Levenberg");
	return;
	  }
      /* The fit itself, assume that  dest_gno == src_gno */
      if (nonlfit_loess_comand (src_gno ,src_setno 
				,nonl_opts.family ,nonl_opts.linquad + 1 ,nonl_opts.span
				,warray
				,nonl_prefs.load == NL_LOAD_RESIDUALS
				,nonl_opts.confidence_compute 
				,nonl_opts.confidence_level
				,nonl_opts.confidence_len
				,dest_gno ,dest_setno ,&bar_setno) 
	  == RETURN_SUCCESS) {
	/* update canvas */
	gg_update_all_sets_lists ();
	ge_update_explorer ();
	ge_expand_graph (dest_gno);
	gg_drawgraph   ();
	set_dirtystate ();
      }
      break;
    case NL_BY_HAND:
      do_compute (src_gno ,src_setno ,dest_gno ,dest_setno ,NULL ,nonl_opts.formula);
      comment_and_legend_set (dest_gno ,dest_setno ,"Fitting candidate ");
      /* We prefer connected points and no symbols for fitting curve */
      get_graph_plotarr (dest_gno ,dest_setno ,&pa);
      if (pa.linet == LINE_TYPE_NONE) pa.linet = LINE_TYPE_STRAIGHT;
      pa.sym = SYM_NONE;
      set_graph_plotarr (dest_gno ,dest_setno ,&pa);
      gg_drawgraph   ();
      set_dirtystate ();
      break;
    }
  }
  if (reponse == GTK_RESPONSE_ACCEPT || reponse == GTK_RESPONSE_CLOSE) {
    gtk_widget_hide (w);
  }
}



/**
 * Replace   load_nonl_fit_cb	    nonlwin.c   543
 */
static void  gg_nonl_load_CB (GtkWidget *w ,gpointer p)
{
  int src_gno;
  static int ns1 ,ns2 = 1;
  gint *sl1 ,*sl2;
  if (gg_GetSrcDestSettings (srcdest ,SD_NSET2_GIVEN
			     ,&src_gno ,&dest_gno
			     ,&ns1  ,&ns2
			     ,&sl1  ,&sl2) == RETURN_FAILURE) return;
  gg_nonl_load_fit (src_gno ,sl1[0] ,dest_gno ,sl2[0] ,TRUE);
  xfree (sl1);
  xfree (sl2);
}


/**
 *  Used by NL_LEVENBERG to compute the fitted set
 *  using the coefficients stored in array nonl_parms[]
 *  and 
 *  Replace  load_nonl_fit	    nonlwin.c   560
 */
static int gg_nonl_load_fit (int src_gno ,int set1 ,int dest_gno ,int set2 ,int force)
{
  nonl_opts.formula   = copy_string (nonl_opts.formula ,gg_getstr (w_formula));
  nonl_opts.logpts    = gg_get_int (w_logpts);   /* added djconnel Feb 2010 */
  nonl_prefs.autoload = gg_get_int (w_autol_item);
  if (nonl_prefs.load == NL_LOAD_FUNCTION) {
    if (gg_evalexpr (w_start, &nonl_prefs.start) != RETURN_SUCCESS) {
      errmsg ("Invalid input in start field");
      return RETURN_FAILURE;
    }
    if (gg_evalexpr (w_stop, &nonl_prefs.stop) != RETURN_SUCCESS) {
      errmsg ("Invalid input in stop field");
      return RETURN_FAILURE;
    }
    if (gg_evalexpri (w_npts, &nonl_prefs.npoints) != RETURN_SUCCESS) {
      errmsg ("Invalid input in # of points field");
      return RETURN_FAILURE;
    }
  }
  if (nonlfit_compute_and_load (src_gno ,set1 ,dest_gno ,set2 ,force) == RETURN_FAILURE) return RETURN_FAILURE;
  gg_update_all_sets_lists ();
  ge_update_explorer 	     ();
  ge_expand_graph    	     (dest_gno);
  gg_drawgraph       	     ();
  return RETURN_SUCCESS;
}

static void gg_value1_CB (GtkWidget *w ,gpointer p)
{
  int src_gno;
  gint *sl1 ,*sl2;
  plotarr pa;
  static int ns1 ,ns2 = 1;
  int i = GPOINTER_TO_INT (p);
  if (block_value1_CB) return;
  if (gg_GetSrcDestSettings (srcdest ,SD_NSET2_GIVEN
			     ,&src_gno ,&dest_gno
			     ,&ns1  ,&ns2
			     ,&sl1  ,&sl2) == RETURN_FAILURE) return;
  nonl_parms[i].value = gg_get_dble (w);
  do_compute (src_gno ,sl1[0] ,dest_gno ,sl2[0] ,NULL ,nonl_opts.formula);
  sprintf(buf, "A%1d = %g", i ,nonl_parms[i].value);
  comment_and_legend_set (dest_gno ,sl2[0] ,buf);
  /* We prefer connected points and no symbols for fitting curve */
  get_graph_plotarr (dest_gno ,sl2[0] ,&pa);
  if (pa.linet == LINE_TYPE_NONE) pa.linet = LINE_TYPE_STRAIGHT;
  pa.sym = SYM_NONE;
  set_graph_plotarr (dest_gno ,sl2[0] ,&pa);

  gg_drawgraph 	 ();
  set_dirtystate ();
  xfree (sl1);
  xfree (sl2);
}

static void gg_range_CB    	(GtkWidget *w ,gpointer p)
{
  int i = GPOINTER_TO_INT (p);
  double lowb = gg_get_dble (w_lowb[i]);
  double uppb = gg_get_dble (w_uppb[i]);
  double step = gg_get_dble (w_step1[i]);

  printf ("gg_range_CB: lowb=%f uppb=%f step=%f\n" ,lowb ,uppb ,step);
  gtk_spin_button_set_range 	 (GTK_SPIN_BUTTON (w_value1[i]) ,lowb ,uppb);
  gtk_spin_button_set_increments (GTK_SPIN_BUTTON (w_value1[i]) ,step ,10.0*step);
  
}

/**
 *  Replace  create_openfit_popup    nonlwin.c   646
 *  and      do_savefit_proc         nonlwin.c   702
 */
static void  gg_openfit_CB	(GtkWidget *w ,gpointer p)
{
  static GtkWidget *fsb = NULL;
  char *filename;
  if (fsb == NULL) {
    fsb = gg_file_select_new ("Open fit parameter file" ,"*.fit"
				,popup_width ,popup_height
				,NULL
				,FS_DIALOG_OPEN);
    while (gtk_dialog_run (GTK_DIALOG(fsb)) != GTK_RESPONSE_CANCEL) {
      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(fsb));
      if (getparms (filename) == 1) {
	gg_nonl_update ();
      } else {
	errmsg ("Error reading fit parameter file");
      }
      g_free (filename);
    }
    gtk_widget_destroy (GTK_WIDGET (fsb));
    fsb = NULL;
  }
}

/**
 *  Replace  create_savefit_popup    nonlwin.c   678
 *  and      do_savefit_proc         nonlwin.c   702
 */
static void  gg_savefit_CB (GtkWidget *w ,gpointer p)
{
  static GtkWidget *fsb ,*vb ,*title_item;
  char *filename;
  gint response;
  FILE *pp;
  gg_set_wait_cursor();

  /* extra_widget */
  vb =  gtk_vbox_new (FALSE, 2);
  title_item = gg_entry_new (vb ,"Title: " ,25);
  fsb = gg_file_select_new ("Save fit parameter file" ,"*.fit"
			      ,popup_width ,popup_height
			      ,vb
			      ,FS_DIALOG_SAVE);

  response = gtk_dialog_run (GTK_DIALOG (fsb) ); 
  switch (response) {
  case GTK_RESPONSE_CANCEL :
    break;
  case  GTK_RESPONSE_ACCEPT :
    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(fsb));
    pp = grace_openw (filename);
    if (pp != NULL) {
      nonl_opts.title = copy_string (nonl_opts.title, gg_getstr (title_item));
      put_fitparms (pp, 0);
      grace_close (pp);
    }
  }
  gtk_widget_destroy (GTK_WIDGET (fsb));
  gg_unset_wait_cursor();
  // if (follow_me_on) nn_cmd_nonlfit_file (); // No differential print for nonlfi
}

/* ------------------------ Extended Library ------------------------ */
/* Originated by N. Feralis in Grace */

typedef struct _nonl_lib_entry nonl_lib_entry;

struct _nonl_lib_entry {
  NL_library entry;     /* to check synchronization */
  int parnum;         /* number of parameters used */
  int  action;        /* to take with the mouse (see PEAK_POS CanvasActionFlag in gg_events.h) */
  char *title;        /* displayed */
  char *formula;      /* to perform the fit */
  char *desc;
};


/* Must be synchronized with NL_library enum */
/* entry , parnum ,event , title , formula , param description */
nonl_lib_entry nle[] = {
  {NL_RESET       ,0  ,DO_NOTHING ,"Reset"
   ,""
   ,""
  }
  ,{NL_GAUSS_1    ,4  ,PEAK_POS    ,"Gaussian function"
    ,"y = A0 + (A3*2*sqrt(ln(2)/pi)/A2)*exp(-4*ln(2)*((x-A1)/A2)^2)"
    ,"A0: Baseline offset\nA1: Center of the peak\nA2: Full width at half maximum\nA3: Peak area\n\n"
  }
  ,{NL_GAUSS_2    ,7 ,PEAK_POS1    ,"Double Gaussian function"
    ,"y = A0 + (A3*2*sqrt(ln(2)/pi)/A2)*exp(-4*ln(2)*((x-A1)/A2)^2) + (A6*2*sqrt(ln(2)/pi)/A5)*exp(-4*ln(2)*((x-A4)/A5)^2)"
    ,"A0: Baseline offset\nA1, A4: Center of peaks 1, 2\nA2, A5: Full width at half maximum of peaks 1, 2\nA3, A6: Area of peaks 1, 2\n\n"
  }
  ,{NL_GAUSS_3   ,10 ,PEAK_POS1B   ,"Triple Gaussian function"
    ,"y = A0 + (A3*2*sqrt(ln(2)/pi)/A2)*exp(-4*ln(2)*((x-A1)/A2)^2) + (A6*2*sqrt(ln(2)/pi)/A5)*exp(-4*ln(2)*((x-A4)/A5)^2)+ (A9*2*sqrt(ln(2)/pi)/A8)*exp(-4*ln(2)*((x-A7)/A8)^2)"
    ,"A0: Baseline offset\nA1, A4: Center of peaks 1, 2\nA2, A5: Full width at half maximum of peaks 1, 2\nA3, A6: Area of peaks 1, 2\n\n"
  }
  ,{NL_GAUSS_1A  ,4  ,PEAK_POS    ,"Gaussian (chromatography) function"
    //    ,"y = A0 + (1/sqrt(2*pi))*(A3/A2)*exp(-(x-A1)^2/2*A2^2)"
    ,"y = A0 + (1/sqrt(2*pi))*(A3/A2)*exp(-(x-A1)^2/2/A2^2)"
    ,"A0: Baseline offset\nA1: Center of the peak (retention time)\nA2: Standard deviation of the peak\nA3: Peak area\n\n"
  }
  ,{NL_LORENTZ_1 ,4  ,PEAK_POS    ,"Lorentzian function"
    ,"y = A0 + (2*A2*A3/pi)/(4*(x-A1)^2 + A2^2)"
    ,"A0: Baseline offset\nA1: Center of the peak\nA2: Full width at half maximum\nA3: Peak area\n\n"
  }
  ,{NL_LORENTZ_2 ,7 ,PEAK_POS1    ,"Double Lorentzian function"
    ,"y = A0 + (2*A2*A3/pi)/(4*(x-A1)^2 + A2^2) + (2*A5*A6/pi)/(4*(x-A4)^2 + A5^2)"
    ,"A0: Baseline offset\nA1, A4: Center of peaks 1, 2\nA2, A5: Full width at half maximum of peaks 1, 2\nA3, A6: Area of peaks 1, 2\n\n"
  }
  ,{NL_LORENTZ_3 ,10 ,PEAK_POS1B  ,"Triple Lorentzian function"
    ,"y = A0 + (2*A2*A3/pi)/(4*(x-A1)^2 + A2^2) + (2*A5*A6/pi)/(4*(x-A4)^2 + A5^2) + (2*A8*A9/pi)/(4*(x-A7)^2 + A8^2)"
    ,"A0: Baseline offset\nA1, A4, A7: Center of peaks 1, 2, 3\nA2, A5, A7: Full width at half maximum of peaks 1, 2, 3\nA3, A6, A9: Area of peaks 1, 2, 3\n\n"
  }
  ,{NL_PSVOIGHT_1 ,5 ,PEAK_POS   ,"Pseudo Voigt 1 function"
    ,"y = A0 + A3 * (A4*(2/pi)*A2/(4*(x-A1)^2+A2^2) + (1-A4)*exp(-4*ln(2)*(x-A1)^2/A2^2)*(sqrt(4*ln(2))/(A2*sqrt(pi))))"
    ,"Gaussian and Lorentzian have the same width\nA0: Baseline offset\nA1: Center of the peak\nA2: Full width at half maximum\nA3: Amplitude\nA4: Profile shape factor \n\n"
  }
  ,{NL_PSVOIGHT_2 ,6 ,PEAK_POS   ,"Pseudo Voigt 2 function"
    ,"y = A0 + A3 * (A5*(2/pi)*A2/(4*(x-A1)^2+A2^2) + (1-A5)*exp(-4*ln(2)*(x-A1)^2/A4^2)*(sqrt(4*ln(2))/(A2*sqrt(pi))))"
    ,"Gaussian and Lorentzian have different width\nA0: Baseline offset\nA1: Center of the peak\nA2: Full width at half maximum (Lorentzian)\nA3: Amplitude\nA4: Full width at half maximum (Gaussian) \nA5: Profile shape factor \n\n"
  }
  ,{NL_DS        ,5  ,PEAK_POS   ,"Doniach-Sunjic function"
    ,"y = A0 + A3*cos((pi*A4/2)+(1-A4)*atan((x-A1)/A2))/(A2^2+(x-A1)^2)^((1-A4)/2)"
    ,"A0: Baseline offset\nA1: Center of the peak\nA2: Full width at half maximum\nA3: Peak area\nA4: Asymmetry parameter \n\n"
  }
  ,{NL_ASYM2SIG   ,6 ,PEAK_POS   ,"Asymmetric double sigmoidal function"
    ,"y = A0 + A3*(1/(1+exp(-(x-A1+A2/2)/A4)))*(1-(1/(1+exp(-(x-A1-A2/2)/A5))))"
    ,"A0: Baseline offset\nA1: Center of the peak\nA2: Width 1\nA3: Amplitude\nA4: Width 2\nA5: Width 5 \n\n"
  }
  ,{NL_LOGNORMAL  ,4  ,PEAK_POS  ,"Log Normal Function"
    ,"y = A0 + A3*exp(-((ln(x)-ln(A1))^2)/(2*A2))"
    ,"A0: Baseline offset\nA1: Center of the peak\nA2: Width\nA3: Amplitude\n\n"
  }
  ,{NL_GCAS       ,5  ,PEAK_POS  ,"Gram-Charlier A-Series"
    ,"y = A0 + A3/(A2*sqrt(2*pi))*exp(-0.5*((x-A1)/A2)^2)*(1+(A4/6)*(((x-A1)/A2)^3-3*(x-A1)/A2)+(A5/24)*(((x-A1)/A2)^4-6*((x-A1)/A2)^3+3))"
    ,"A0: Baseline offset\nA1: Center of the peak\nA2: Standard deviation\nA3: Peak Area\nA4: Skew\nA5: Excess\n\n"
  }
  ,{NL_ECS        ,5  ,PEAK_POS  ,"Edgeworth-Cramer Series"
    ,"y = A0 + A3/(A2*sqrt(2*pi))*exp(-0.5*((x-A1)/A2)^2)*(1+(A4/6)*(((x-A1)/A2)^3-3*(x-A1)/A2)+(A5/24)*(((x-A1)/A2)^4-6*((x-A1)/A2)^3+3) + (A5^2/720)*(((x-A1)/A2)^6-15*((x-A1)/A2)^4+45*((x-A1)/A2)^2-15))"
    ,"A0: Baseline offset\nA1: Center of the peak\nA2: Standard deviation\nA3: Peak Area\nA4: Skew\nA5: Excess\n\n"
  }
  ,{NL_INVPOLY    ,7 ,PEAK_POS   ,"Inverse Polynomial Function"
    ,"y = A0 + A3/(1+ A4*(2*(x-A1)/A2)^2 + A5*(2*(x-A1)/A2)^4 + A6*(2*(x-A1)/A2)^6)"
    ,"A0: Baseline offset\nA1: Center of the peak\nA2: Standard deviation\nA3: Peak Area\nA4, A5, A6: Parameters\n\n"
  }
  ,{NL_SINE       ,4 ,PEAK_POS   ,"Sine Function"
    ,"y = A0 + A3*sin(pi*(x-A1)/A2)"
    ,"A0: Baseline offset\nA1: Center\nA2: Width\nA3: Amplitude\n\n"
  }
  ,{NL_SINE_SQ    ,4 ,PEAK_POS   ,"Sine Square Function"
    ,"y = A0 + A3*sin(pi*(x-A1)/A2)^2"
    ,"A0: Baseline offset\nA1: Center\nA2: Width\nA3: Amplitude\n\n"
  }
  ,{NL_SINE_DAMP  ,5 ,PEAK_POS   ,"Sine Damp Function"
    ,"y = A0 + A3*exp(-x/A4)*sin(pi*(x-A1)/A2)"
    ,"A0: Baseline offset; A1: Center; A2: Width; A3: Amplitude; A4: Decay time. "
  }
  ,{NL_EXP_DEC_1  ,4 ,DO_NOTHING ,"Exponential Decay 1"
    ,"y = A0 + A3*exp(-(x-A1)/A2)"
    ,""
  }
  ,{NL_EXP_DEC_2  ,7 ,DO_NOTHING ,"Exponential Decay 2"
    ,"y = A0 + A3*exp(-(x-A1)/A2)+A6*exp(-(x-A4)/A5)"
    ,""
  }
  ,{NL_EXP_GROW_1 ,4 ,DO_NOTHING ,"Exponential Growth 1"
    ,"y = A0 + A3*exp((x-A1)/A2)"
    ,""
  }
  ,{NL_EXP_GROW_2 ,7 ,DO_NOTHING ,"Exponential Growth 2"
    ,"y = A0 + A3*exp((x-A1)/A2)+A6*exp((x-A4)/A5)"
    ,""
  }
  ,{NL_HYPERBOLE  ,3 ,DO_NOTHING ,"Hyperbolic"
    ,"y = A0 + (A1*x)/(A2+x)"
    ,""
  }
  ,{NL_BRADLEY    ,2 ,DO_NOTHING ,"Bradley"
    ,"y = A0 * ln(-A1*ln(x))"
    ,""
  }
  ,{NL_LOG3       ,3 ,DO_NOTHING ,"Logarithm 3 Par."
    ,"y = A0 - A1*ln(x+A2)"
    ,""
  }
  ,{NL_WEIBULLPD  ,2 ,DO_NOTHING ,"Weibull Probability Density"
    ,"y = (A0/A1) * ((x/A1)^(A0-1)) * exp(-(x/A1)^A0)"
    ,""
  }
  ,{NL_WEIBULLCD  ,2 ,DO_NOTHING ,"Weibull Cumulative Distribution"
    ,"y = 1 - exp(-(x/A1)^A0)"
    ,""
  }
};

static void nonl_lib_CB (GtkWidget *w ,gpointer p)
{
  NL_library n = GPOINTER_TO_INT (p);
  int i;
  if (nle[n].entry != n) {
    errmsg ("nonl_lib_CB internal ERROR: desync nle entry");
    return;
  }
  nonl_opts.title   = copy_string (nonl_opts.title   ,nle[n].title);
  nonl_opts.formula = copy_string (nonl_opts.formula ,nle[n].formula);
  nonl_opts.parnum  = nle[n].parnum;
  for (i = 0; i < nonl_opts.parnum; i++) nonl_parms[i].value = 1;
  /* Some functions use special parameters settings */
  switch (n) {
  case NL_PSVOIGHT_1:
  case NL_DS:
    nonl_parms[4].value = 0.5;
    break;
  case NL_PSVOIGHT_2:
    nonl_parms[5].value = 0.5;
    break;
  default:
    break;
  }
  if (strcmp ("" ,nle[n].desc)) {
    stufftext (nle[n].title);
    stufftext ("\n");
    stufftext (nle[n].desc);
  }
  gg_set_action  (nle[n].action);
  gg_nonl_update ();
}


