/** File nonlfit.h
 */

#ifndef __NONLFIT_H_
#define __NONLFIT_H_

#include "../loess/loess.h"

#define MAXPARM 10        /* max number of parameters for non-lin fit */

typedef enum {
  NL_WEIGHT_NONE = 0
  ,NL_WEIGHT_Y
  ,NL_WEIGHT_Y2
  ,NL_WEIGHT_DY
  ,NL_WEIGHT_CUSTOM
} NL_weight_method;



/* parameters for Levenberg non-linear fit */
typedef struct _NONLparams NONLparams;
struct _NONLparams {
    double value;       /* parameter itself */
    int constr;         /* whether or not to use constraints */
    double min;         /* low bound constraint */
    double max;         /* upper bound constraint */
  double step;          /* for spin buttons */
};

/* options for non-linear fit */
typedef struct _NONLopts NONLopts;
struct _NONLopts {
  NL_method 	   nonl_method;  /* NL_method defined in ../loess/loess.h */
  char 		   *title;    	 /* fit title */
  int 		   rtype;     	 /* restriction type */
  int 		   rnegate;   	 /* negate restriction region */
  NL_weight_method wmethod;      /* Define a wheight for each point */
  char 		   *wfunc;    	 /* formula used when NL_WEIGHT_CUSTOM */
  /* Levenberg only */
  char 	 *formula;      /* fit function */
  int 	 parnum;        /* # of fit parameters */
  double tolerance;   	/* tolerance */
  int 	 nsteps;        /* nb max of steps */
  int 	 trace_on;      /* TRUE if trace all steps */
  int 	 logpts;        /* whether to progress logarithmically (djconnel Feb 2010)*/
  /* LOESS only */
  NL_family family;
  double    span;
  int       confidence_compute;  /* whether we compute confidence bars */
  double    confidence_level;    /* the level used to compute confidence bars */
  int       confidence_len;      /* the number of confidence bars to be computed */
  int 	    linquad;  	      	 /* 0:linear 1:quadratic BEWARE TO THE SHIFT */
};

/* prefs for loading results of non-linear fit */
typedef enum {
  NL_LOAD_VALUES
  ,NL_LOAD_RESIDUALS
  ,NL_LOAD_FUNCTION
} NL_load_results;

typedef struct {
  int autoload;       /* do autoload */
  int load;           /* load to... */
  int npoints;        /* # of points to evaluate function at */
  double start;       /* start... */
  double stop;        /* stop ... */
} NONL_load_prefs;

void nonlfit_set_defaults     (void);
void nonlfit_set_parm_value   (int i ,double x);
int  nonlfit_compute_and_load (int src_gno ,int set1 ,int dest_gno ,int set2 ,int force);
int  nonlfit_loess_comand     (int src_gno ,int src_setno 
			       ,NL_family nonl_family ,int nonl_deg ,double nonl_span
			       ,double *warray
			       ,int    load_residuals
			       ,int    nonl_confidence_compute 
			       ,double nonl_confidence_level
			       ,int 	  nonl_confidence_len
			       ,int dest_gno ,int dest_setno ,int *bar_setno);

#endif /* __NL_H_ */
